@@ -35,8 +35,11 @@
 #include "GPU_osd.h"
 #include <iosfwd>
 
+//#undef FORCEINLINE
+//#define FORCEINLINE
+
 void gpu_savestate(std::ostream* os);
-bool gpu_loadstate(std::istream* is);
+bool gpu_loadstate(std::istream* is, int size);
 
 /*******************************************************************************
     this structure is for display control,
@@ -402,7 +405,11 @@ void register_gl_fun(fun_gl_Begin beg,fun_gl_End end);
 #define ADDRESS_STEP_8KB		0x02000
 #define ADDRESS_STEP_16KB	   0x04000
 #define ADDRESS_STEP_32KB	   0x08000
-#define ADDRESS_STEP_64kB	   0x10000
+#define ADDRESS_STEP_64KB	   0x10000
+#define ADDRESS_STEP_128KB	   0x20000
+#define ADDRESS_STEP_256KB	   0x40000
+#define ADDRESS_STEP_512KB	   0x80000
+#define ADDRESS_MASK_256KB	   (ADDRESS_STEP_256KB-1)
 
 #ifdef WORDS_BIGENDIAN
 struct _TILEENTRY
@@ -529,7 +536,7 @@ typedef struct
 /*12*/    unsigned PaletteIndex:4;
 /*10*/    unsigned Priority:2;
 // attr3
-unsigned attr3:16;    
+          unsigned attr3:16;    
 #else
 // attr0
 /* 0*/	unsigned Y:8;
@@ -555,14 +562,24 @@ unsigned attr3:16;
 
 typedef struct
 {
+#ifdef WORDS_BIGENDIAN
+	 u8 attr00;
+	 u8 attr01;
+	 u8 attr10;
+	 u8 attr11;
+	 u8 attr20;
+	 u8 attr21;
+	 u8 attr30;
+	 u8 attr31;
+#else
 	 u16 attr0;
 	 u16 attr1;
 	 u16 attr2;
 	 u16 attr3;
+#endif
 } OAM;
 
 
-
 typedef struct
 {
 	 s16 x;
@@ -590,15 +607,25 @@ typedef struct
 #define ARM9MEM_BBG		0x06200000
 #define ARM9MEM_AOBJ	0x06400000
 #define ARM9MEM_BOBJ	0x06600000
+#define ARM9MEM_LCDC	0x06800000
 
 extern CACHE_ALIGN u8 gpuBlendTable555[17][17][32][32];
 
+enum BGType {
+	BGType_Invalid=0, BGType_Text=1, BGType_Affine=2, BGType_Large8bpp=3, 
+	BGType_AffineExt=4, BGType_AffineExt_256x16=5, BGType_AffineExt_256x1=6, BGType_AffineExt_Direct=7
+};
+
 struct GPU
 {
 	// some structs are becoming redundant
 	// some functions too (no need to recopy some vars as it is done by MMU)
 	REG_DISPx * dispx_st;
 
+	_BGxCNT & bgcnt(int num) { return (dispx_st)->dispx_BGxCNT[num].bits; }
+	_DISPCNT & dispCnt() { return dispx_st->dispx_DISPCNT.bits; }
+	template<bool MOSAIC> void modeRender(int layer);
+
 	DISPCAPCNT dispCapCnt;
 	BOOL LayersEnable[5];
 	itemsForPriority_t itemsForPriority[NB_PRIORITIES];
@@ -606,12 +633,14 @@ struct GPU
 #define BGBmpBB BG_bmp_ram
 #define BGChBB BG_tile_ram
 
+	u32 BG_bmp_large_ram[4];
 	u32 BG_bmp_ram[4];
 	u32 BG_tile_ram[4];
 	u32 BG_map_ram[4];
 
 	u8 BGExtPalSlot[4];
 	u32 BGSize[4][2];
+	BGType BGTypes[4];
 
 	struct MosaicColor {
 		u16 bg[4][256];
@@ -679,6 +708,8 @@ struct GPU
 	u8	BLDALPHA_EVA;
 	u8	BLDALPHA_EVB;
 	u8	BLDY_EVY;
+	u16 *currentFadeInColors, *currentFadeOutColors;
+	bool blend2[8];
 
 	u8	MasterBrightMode;
 	u32 MasterBrightFactor;
@@ -690,6 +721,9 @@ struct GPU
 	bool blend1;
 	u8* currDst;
 
+	u16* _3dColorLine;
+	u8* _3dAlphaLine;
+
 
 	static struct MosaicLookup {
 
@@ -716,26 +750,48 @@ struct GPU
 	u16 blend(u16 colA, u16 colB);
 
 	typedef void (*FinalOBJColFunct)(GPU *gpu, u32 passing, u8 *dst, u16 color, u8 alpha, u8 type, u16 x);
-	typedef void (*Final3DColFunct)(GPU *gpu, u32 passing, u8 *dst, u16 color, u8 alpha, u16 x);
-	typedef void (*SpriteRenderFunct) (GPU * gpu, u16 l, u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab);
+	typedef void (*Final3DColFunct)(GPU *gpu, int dstX, int srcX);
 
 	int setFinalColorBck_funcNum;
+	int bgFunc;
+	int setFinalColor3d_funcNum;
 	FinalOBJColFunct setFinalColorSpr;
-	Final3DColFunct setFinalColor3D;
-	SpriteRenderFunct spriteRender;
+	//Final3DColFunct setFinalColor3D;
+	enum SpriteRenderMode {
+		SPRITE_1D, SPRITE_2D
+	} spriteRenderMode;
+
+	template<GPU::SpriteRenderMode MODE>
+	void _spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab);
+	void spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab);
 
 	void setFinalColorBG(u16 color, u8 x);
-	FORCEINLINE void setFinalBGColorSpecialNone(u16 color, u8 x, bool blend1);
-	FORCEINLINE void setFinalBGColorSpecialBlend(u16 color, u8 x, bool blend1);
-	FORCEINLINE void setFinalBGColorSpecialIncrease(u16 color, u8 x, bool blend1);
-	FORCEINLINE void setFinalBGColorSpecialDecrease(u16 color, u8 x, bool blend1);
-	FORCEINLINE void setFinalBGColorSpecialNoneWnd(u16 color, u8 x, bool blend1);
-	FORCEINLINE void setFinalBGColorSpecialBlendWnd(u16 color, u8 x, bool blend1);
-	FORCEINLINE void setFinalBGColorSpecialIncreaseWnd(u16 color, u8 x, bool blend1);
-	FORCEINLINE void setFinalBGColorSpecialDecreaseWnd(u16 color, u8 x, bool blend1);
+	void setFinalColor3d(int dstX, int srcX);
 
-	void __setFinalColorBck(u16 color, u8 x, bool opaque);
+	FORCEINLINE void setFinalBGColorSpecialNone(u16 &color, u8 x);
+	FORCEINLINE void setFinalBGColorSpecialBlend(u16 &color, u8 x);
+	FORCEINLINE void setFinalBGColorSpecialIncrease(u16 &color, u8 x);
+	FORCEINLINE void setFinalBGColorSpecialDecrease(u16 &color, u8 x);
+	FORCEINLINE bool setFinalBGColorSpecialNoneWnd(u16 &color, u8 x);
+	FORCEINLINE bool setFinalBGColorSpecialBlendWnd(u16 &color, u8 x);
+	FORCEINLINE bool setFinalBGColorSpecialIncreaseWnd(u16 &color, u8 x);
+	FORCEINLINE bool setFinalBGColorSpecialDecreaseWnd(u16 &color, u8 x);
+	
+	FORCEINLINE void setFinal3DColorSpecialNone(int dstX, int srcX);
+	FORCEINLINE void setFinal3DColorSpecialBlend(int dstX, int srcX);
+	FORCEINLINE void setFinal3DColorSpecialIncrease(int dstX, int srcX);
+	FORCEINLINE void setFinal3DColorSpecialDecrease(int dstX, int srcX);
+	FORCEINLINE void setFinal3DColorSpecialNoneWnd(int dstX, int srcX);
+	FORCEINLINE void setFinal3DColorSpecialBlendWnd(int dstX, int srcX);
+	FORCEINLINE void setFinal3DColorSpecialIncreaseWnd(int dstX, int srcX);
+	FORCEINLINE void setFinal3DColorSpecialDecreaseWnd(int dstX, int srcX);
+
+
+	template<bool MOSAIC> void __setFinalColorBck(u16 color, const u8 x, const bool opaque);
 	void setAffineStart(int layer, int xy, u32 val);
+	void setAffineStartWord(int layer, int xy, u16 val, int word);
+	u32 getAffineStart(int layer, int xy);
+	void refreshAffineStartRegs(const int num, const int xy);
 
 	struct AffineInfo {
 		AffineInfo() : x(0), y(0) {}
@@ -751,7 +807,7 @@ struct GPU
 	void setBLDALPHA(u16 val)
 	{
 		BLDALPHA_EVA = (val&0x1f) > 16 ? 16 : (val&0x1f); 
-		BLDALPHA_EVB = (val>>8&0x1f) > 16 ? 16 : (val>>8&0x1f);
+		BLDALPHA_EVB = ((val>>8)&0x1f) > 16 ? 16 : ((val>>8)&0x1f);
 		updateBLDALPHA();
 	}
 
@@ -776,25 +832,25 @@ struct GPU
 	}
 	
 };
-/*
+#if 0
 // normally should have same addresses
 static void REG_DISPx_pack_test(GPU * gpu)
 {
 	REG_DISPx * r = gpu->dispx_st;
-	printf ("%08x %02x\n",  r, (long)(&r->dispx_DISPCNT) - (long)r);
-	printf ("\t%02x\n", (long)(&r->dispA_DISPSTAT) - (long)r);
-	printf ("\t%02x\n", (long)(&r->dispx_VCOUNT) - (long)r);
-	printf ("\t%02x\n", (long)(&r->dispx_BGxCNT[0]) - (long)r);
-	printf ("\t%02x\n", (long)(&r->dispx_BGxOFS[0]) - (long)r);
-	printf ("\t%02x\n", (long)(&r->dispx_BG2PARMS) - (long)r);
-	printf ("\t%02x\n", (long)(&r->dispx_BG3PARMS) - (long)r);
-	printf ("\t%02x\n", (long)(&r->dispx_WINCNT) - (long)r);
-	printf ("\t%02x\n", (long)(&r->dispx_MISC) - (long)r);
-	printf ("\t%02x\n", (long)(&r->dispA_DISP3DCNT) - (long)r);
-	printf ("\t%02x\n", (long)(&r->dispA_DISPCAPCNT) - (long)r);
-	printf ("\t%02x\n", (long)(&r->dispA_DISPMMEMFIFO) - (long)r);
+	printf ("%08x %02x\n",  (u32)r, (u32)(&r->dispx_DISPCNT) - (u32)r);
+	printf ("\t%02x\n", (u32)(&r->dispA_DISPSTAT) - (u32)r);
+	printf ("\t%02x\n", (u32)(&r->dispx_VCOUNT) - (u32)r);
+	printf ("\t%02x\n", (u32)(&r->dispx_BGxCNT[0]) - (u32)r);
+	printf ("\t%02x\n", (u32)(&r->dispx_BGxOFS[0]) - (u32)r);
+	printf ("\t%02x\n", (u32)(&r->dispx_BG2PARMS) - (u32)r);
+	printf ("\t%02x\n", (u32)(&r->dispx_BG3PARMS) - (u32)r);
+//	printf ("\t%02x\n", (u32)(&r->dispx_WINCNT) - (u32)r);
+	printf ("\t%02x\n", (u32)(&r->dispx_MISC) - (u32)r);
+	printf ("\t%02x\n", (u32)(&r->dispA_DISP3DCNT) - (u32)r);
+	printf ("\t%02x\n", (u32)(&r->dispA_DISPCAPCNT) - (u32)r);
+	printf ("\t%02x\n", (u32)(&r->dispA_DISPMMEMFIFO) - (u32)r);
 }
-*/
+#endif
 
 CACHE_ALIGN extern u8 GPU_screen[4*256*192];
 CACHE_ALIGN extern u8 GPU_tempScreen[4*256*192];
@@ -814,9 +870,7 @@ namespace GPU_EXT
 void sprite1D(GPU * gpu, u16 l, u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab);
 void sprite2D(GPU * gpu, u16 l, u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab);
 
-extern const short sizeTab[4][4][2];
 extern const size sprSizeTab[4][4];
-extern const s8 mode2type[8][4];
 
 typedef struct {
 	GPU * gpu;
@@ -839,7 +893,7 @@ extern MMU_struct MMU;
 
 #define GFXCORE_FULLSCREEN	  (1 << 0)
 
-typedef struct
+struct GraphicsInterface_struct
 {
    int id; // ID number for core(see above defines)
    const char *Name; // Name of core
@@ -848,7 +902,7 @@ typedef struct
    void (*DeInit)(); // Deinitializes stuff related to core
    void (*Resize)(int width, int height, BOOL fullscreen); // Resizes window or fullscreen
    void (*OnScreenText)(char *string, ...); // For handling save state messages, etc.
-} GraphicsInterface_struct;
+} ;
 
 extern GraphicsInterface_struct GFXDummy;
 
@@ -869,43 +923,46 @@ void GPU_set_DISPCAPCNT(u32 val) ;
 void GPU_ligne(NDS_Screen * screen, u16 l) ;
 void GPU_setMasterBrightness (GPU *gpu, u16 val);
 
-#define GPU_setWIN0_H(gpu, val) {gpu->WIN0H0 = val >> 8; gpu->WIN0H1 = val&0xFF; gpu->need_update_winh[0] = true; }
-#define GPU_setWIN0_H0(gpu, val) { gpu->WIN0H0 = val;  gpu->need_update_winh[0] = true; }
-#define GPU_setWIN0_H1(gpu, val) { gpu->WIN0H1 = val;  gpu->need_update_winh[0] = true; }
+inline void GPU_setWIN0_H(GPU* gpu, u16 val) { gpu->WIN0H0 = val >> 8; gpu->WIN0H1 = val&0xFF; gpu->need_update_winh[0] = true; }
+inline void GPU_setWIN0_H0(GPU* gpu, u8 val) { gpu->WIN0H0 = val;  gpu->need_update_winh[0] = true; }
+inline void GPU_setWIN0_H1(GPU* gpu, u8 val) { gpu->WIN0H1 = val;  gpu->need_update_winh[0] = true; }
 
-#define GPU_setWIN0_V(gpu, val) {gpu->WIN0V0 = val >> 8; gpu->WIN0V1 = val&0xFF;}
-#define GPU_setWIN0_V0(gpu, val) gpu->WIN0V0 = val
-#define GPU_setWIN0_V1(gpu, val) gpu->WIN0V1 = val
+inline void GPU_setWIN0_V(GPU* gpu, u16 val) { gpu->WIN0V0 = val >> 8; gpu->WIN0V1 = val&0xFF;}
+inline void GPU_setWIN0_V0(GPU* gpu, u8 val) { gpu->WIN0V0 = val; }
+inline void GPU_setWIN0_V1(GPU* gpu, u8 val) { gpu->WIN0V1 = val; }
 
-#define GPU_setWIN1_H(gpu, val) {gpu->WIN1H0 = val >> 8; gpu->WIN1H1 = val&0xFF;  gpu->need_update_winh[1] = true; }
-#define GPU_setWIN1_H0(gpu, val) { gpu->WIN1H0 = val;  gpu->need_update_winh[1] = true; }
-#define GPU_setWIN1_H1(gpu, val) { gpu->WIN1H1 = val;  gpu->need_update_winh[1] = true; }
+inline void GPU_setWIN1_H(GPU* gpu, u16 val) {gpu->WIN1H0 = val >> 8; gpu->WIN1H1 = val&0xFF;  gpu->need_update_winh[1] = true; }
+inline void GPU_setWIN1_H0(GPU* gpu, u8 val) { gpu->WIN1H0 = val;  gpu->need_update_winh[1] = true; }
+inline void GPU_setWIN1_H1(GPU* gpu, u8 val) { gpu->WIN1H1 = val;  gpu->need_update_winh[1] = true; }
 
-#define GPU_setWIN1_V(gpu, val) {gpu->WIN1V0 = val >> 8; gpu->WIN1V1 = val&0xFF;}
-#define GPU_setWIN1_V0(gpu, val) gpu->WIN1V0 = val
-#define GPU_setWIN1_V1(gpu, val) gpu->WIN1V1 = val
+inline void GPU_setWIN1_V(GPU* gpu, u16 val) { gpu->WIN1V0 = val >> 8; gpu->WIN1V1 = val&0xFF; }
+inline void GPU_setWIN1_V0(GPU* gpu, u8 val) { gpu->WIN1V0 = val; }
+inline void GPU_setWIN1_V1(GPU* gpu, u8 val) { gpu->WIN1V1 = val; }
 
-#define GPU_setWININ(gpu, val) {gpu->WININ0=val&0x1F;\
-								gpu->WININ0_SPECIAL=(val>>5)&1;\
-								gpu->WININ1=(val>>8)&0x1F;\
-								gpu->WININ1_SPECIAL=(val>>13)&1;\
-								}
-#define GPU_setWININ0(gpu, val) {gpu->WININ0 = val&0x1F; gpu->WININ0_SPECIAL = (val>>5)&1;}
-#define GPU_setWININ1(gpu, val) {gpu->WININ1 = val&0x1F; gpu->WININ1_SPECIAL = (val>>5)&1;}
+inline void GPU_setWININ(GPU* gpu, u16 val) {
+	gpu->WININ0=val&0x1F;
+	gpu->WININ0_SPECIAL=(val>>5)&1;
+	gpu->WININ1=(val>>8)&0x1F;
+	gpu->WININ1_SPECIAL=(val>>13)&1;
+}
 
-#define GPU_setWINOUT16(gpu, val) { gpu->WINOUT=val&0x1F;\
-									gpu->WINOUT_SPECIAL=(val>>5)&1;\
-									gpu->WINOBJ=(val>>8)&0x1F;\
-									gpu->WINOBJ_SPECIAL=(val>>13)&1;\
-									}
-#define GPU_setWINOUT(gpu, val) {gpu->WINOUT = val&0x1F; gpu->WINOUT_SPECIAL = (val>>5)&1;}
-#define GPU_setWINOBJ(gpu, val) {gpu->WINOBJ = val&0x1F; gpu->WINOBJ_SPECIAL = (val>>5)&1;}
+inline void GPU_setWININ0(GPU* gpu, u8 val) { gpu->WININ0 = val&0x1F; gpu->WININ0_SPECIAL = (val>>5)&1; }
+inline void GPU_setWININ1(GPU* gpu, u8 val) { gpu->WININ1 = val&0x1F; gpu->WININ1_SPECIAL = (val>>5)&1; }
+
+inline void GPU_setWINOUT16(GPU* gpu, u16 val) {
+	gpu->WINOUT=val&0x1F;
+	gpu->WINOUT_SPECIAL=(val>>5)&1;
+	gpu->WINOBJ=(val>>8)&0x1F;
+	gpu->WINOBJ_SPECIAL=(val>>13)&1;
+}
+inline void GPU_setWINOUT(GPU* gpu, u8 val) { gpu->WINOUT = val&0x1F; gpu->WINOUT_SPECIAL = (val>>5)&1; }
+inline void GPU_setWINOBJ(GPU* gpu, u8 val) { gpu->WINOBJ = val&0x1F; gpu->WINOBJ_SPECIAL = (val>>5)&1; }
 
 // Blending
 void SetupFinalPixelBlitter (GPU *gpu);
-#define GPU_setBLDCNT_LOW(gpu, val) {gpu->BLDCNT = (gpu->BLDCNT&0xFF00) | val; SetupFinalPixelBlitter (gpu);}
+#define GPU_setBLDCNT_LOW(gpu, val) {gpu->BLDCNT = (gpu->BLDCNT&0xFF00) | (val); SetupFinalPixelBlitter (gpu);}
 #define GPU_setBLDCNT_HIGH(gpu, val) {gpu->BLDCNT = (gpu->BLDCNT&0xFF) | (val<<8); SetupFinalPixelBlitter (gpu);}
-#define GPU_setBLDCNT(gpu, val) {gpu->BLDCNT = val; SetupFinalPixelBlitter (gpu);}
+#define GPU_setBLDCNT(gpu, val) {gpu->BLDCNT = (val); SetupFinalPixelBlitter (gpu);}
 
 
 
@@ -918,5 +975,10 @@ void SetupFinalPixelBlitter (GPU *gpu);
 void gpu_UpdateRender();
 void gpu_SetRotateScreen(u16 angle);
 
+//#undef FORCEINLINE
+//#define FORCEINLINE __forceinline
+
+extern OSDCLASS	*osd;
+
 #endif
 
