@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <algorithm>
 #include <math.h>
+#include <zlib.h>
 
 #include "common.h"
 #include "NDSSystem.h"
@@ -36,26 +37,28 @@
 #include "ROMReader.h"
 #include "gfx3d.h"
 #include "utils/decrypt/decrypt.h"
+#include "utils/decrypt/crc.h"
 #include "bios.h"
 #include "debug.h"
 #include "cheatSystem.h"
+#include "movie.h"
+#include "Disassembler.h"
 
 #ifdef _WIN32
 #include "./windows/disView.h"
 #endif
 
-//#define USE_REAL_BIOS
-
 TCommonSettings CommonSettings;
-static Driver _stub_driver;
-Driver* driver = &_stub_driver;
+static BaseDriver _stub_driver;
+BaseDriver* driver = &_stub_driver;
 std::string InputDisplayString;
 
 static BOOL LidClosed = FALSE;
 static u8	countLid = 0;
 char pathToROM[MAX_PATH];
 char pathFilenameToROMwithoutExt[MAX_PATH];
-char ROMserial[20];
+
+GameInfo gameInfo;
 
 /* the count of bytes copied from the firmware into memory */
 #define NDS_FW_USER_SETTINGS_MEM_BYTE_COUNT 0x70
@@ -179,7 +182,7 @@ static BOOL initKeycode(u32 idCode, int level, u32 modulo)
 
 static u16 getBootCodeCRC16()
 {
-	int i, j;
+	unsigned int i, j;
 	u32 crc = 0xFFFF;
 	const u16 val[8] = {0xC0C1, 0xC181, 0xC301, 0xC601, 0xCC01, 0xD801, 0xF001, 0xA001};
 
@@ -517,8 +520,7 @@ int NDS_Init( void) {
 
 #ifdef EXPERIMENTAL_WIFI
 	WIFI_Init(&wifiMac) ;
-	if(wifiMac.netEnabled)
-		WIFI_SoftAP_Init(&wifiMac);
+	WIFI_SoftAP_Init(&wifiMac);
 #endif
 
 	nds.FW_ARM9BootCode = NULL;
@@ -538,9 +540,9 @@ void NDS_DeInit(void) {
 	gpu3D->NDS_3D_Close();
 
 #ifdef EXPERIMENTAL_WIFI
-	if(wifiMac.netEnabled)
-		WIFI_SoftAP_Shutdown(&wifiMac);
+	WIFI_SoftAP_Shutdown(&wifiMac);
 #endif
+	cheatsSearchClose();
 }
 
 BOOL NDS_SetROM(u8 * rom, u32 mask)
@@ -671,11 +673,12 @@ static u32 ones32(u32 x)
 }
 #endif
 
-static void NDS_SetROMSerial()
+void GameInfo::populate()
 {
-	NDS_header * header;
+	NDS_header * _header = NDS_getROMHeader();
+	header = *_header;
+	delete _header;
 
-	header = NDS_getROMHeader();
 	if (
 		// ??? in all Homebrews game title have is 2E0000EA
 		//(
@@ -685,15 +688,15 @@ static void NDS_SetROMSerial()
 		//(header->gameTile[3] == 0xEA)
 		//) &&
 		(
-			((header->gameCode[0] == 0x23) && 
-			(header->gameCode[1] == 0x23) && 
-			(header->gameCode[2] == 0x23) && 
-			(header->gameCode[3] == 0x23)
+			((header.gameCode[0] == 0x23) && 
+			(header.gameCode[1] == 0x23) && 
+			(header.gameCode[2] == 0x23) && 
+			(header.gameCode[3] == 0x23)
 			) || 
-			(header->gameCode[0] == 0x00) 
+			(header.gameCode[0] == 0x00) 
 		)
 		&&
-			header->makerCode == 0x0
+			header.makerCode == 0x0
 		)
 	{
 		memset(ROMserial, 0, sizeof(ROMserial));
@@ -702,19 +705,17 @@ static void NDS_SetROMSerial()
 	else
 	{
 		memset(ROMserial, '_', sizeof(ROMserial));
-		memcpy(ROMserial, header->gameTile, strlen(header->gameTile) < 12 ? strlen(header->gameTile) : 12);
-		memcpy(ROMserial+12+1, header->gameCode, 4);
-		memcpy(ROMserial+12+1+4, &header->makerCode, 2);
+		memcpy(ROMserial, header.gameTile, strlen(header.gameTile) < 12 ? strlen(header.gameTile) : 12);
+		memcpy(ROMserial+12+1, header.gameCode, 4);
+		memcpy(ROMserial+12+1+4, &header.makerCode, 2);
 		memset(ROMserial+19, '\0', 1);
 	}
-	delete header;
-	INFO("\nROM serial: %s\n\n", ROMserial);
 }
 
 #ifdef EXPERIMENTAL_GBASLOT
-int NDS_LoadROM( const char *filename, int bmtype, u32 bmsize)
+int NDS_LoadROM( const char *filename)
 #else
-int NDS_LoadROM( const char *filename, int bmtype, u32 bmsize,
+int NDS_LoadROM( const char *filename,
 				const char *cflash_disk_image_file)
 #endif
 {
@@ -758,9 +759,14 @@ int NDS_LoadROM( const char *filename, int bmtype, u32 bmsize,
 			break;
 		}
 
-	type = ROM_NDS;
-	if ( !strcmp(extROM, ".gba") && !strcmp(extROM2, ".ds")) 
-			type = ROM_DSGBA;
+	if(!strcasecmp(extROM, ".zip"))
+		type = ROM_NDS;
+	else if ( !strcasecmp(extROM, ".nds"))
+		type = ROM_NDS;
+	else if ( !strcasecmp(extROM, ".gba") && !strcasecmp(extROM2, ".ds")) 
+		type = ROM_DSGBA;
+	else
+		type = ROM_NDS;
 
 	file = reader->Init(filename);
 	if (!file)
@@ -813,9 +819,16 @@ int NDS_LoadROM( const char *filename, int bmtype, u32 bmsize,
 	//decrypt if necessary..
 	//but this is untested and suspected to fail on big endian, so lets not support this on big endian
 #ifndef WORDS_BIGENDIAN
-	DecryptSecureArea(data,size);
+	bool okRom = DecryptSecureArea(data,size);
+
+	if(!okRom) {
+		printf("Specified file is not a valid rom\n");
+		return -1;
+	}
 #endif
 
+
+	cheatsSearchClose();
 	MMU_unsetRom();
 	NDS_SetROM(data, mask);
 	NDS_Reset();
@@ -828,17 +841,19 @@ int NDS_LoadROM( const char *filename, int bmtype, u32 bmsize,
 
 	memset(buf, 0, MAX_PATH);
 	strcpy(buf, pathFilenameToROMwithoutExt);
-	strcat(buf, ".sav");							// DeSmuME memory card	:)
+	strcat(buf, ".dsv");							// DeSmuME memory card	:)
 
-	mc_realloc(&MMU.bupmem, bmtype, bmsize);
-	mc_load_file(&MMU.bupmem, buf);
+	MMU_new.backupDevice.load_rom(buf);
 
 	memset(buf, 0, MAX_PATH);
 	strcpy(buf, pathFilenameToROMwithoutExt);
 	strcat(buf, ".dct");							// DeSmuME cheat		:)
 	cheatsInit(buf);
 
-	NDS_SetROMSerial();
+	gameInfo.populate();
+	gameInfo.crc = crc32(0,data,size);
+	INFO("\nROM crc: %08X\n\n", gameInfo.crc);
+	INFO("\nROM serial: %s\n", gameInfo.ROMserial);
 
 	return ret;
 }
@@ -853,7 +868,8 @@ void NDS_FreeROM(void)
 	MMU.bupmem.fp = NULL;
 }
 
-void NDS_Reset( void)
+bool _HACK_DONT_STOPMOVIE = false;
+void NDS_Reset()
 {
 	unsigned int i;
 	u32 src;
@@ -863,12 +879,11 @@ void NDS_Reset( void)
 
 	if (!header) return ;
 
-	lagframecounter=0;
-	LagFrameFlag=0;
-	lastLag=0;
-	TotalLagFrames=0;
+	if(movieMode != MOVIEMODE_INACTIVE && !_HACK_DONT_STOPMOVIE)
+		movie_reset_command = true;
 
 	MMU_clearMem();
+	MMU_new.backupDevice.reset();
 
 	//ARM7 BIOS IRQ HANDLER
 	if(CommonSettings.UseExtBIOS == true)
@@ -978,6 +993,7 @@ void NDS_Reset( void)
 	
 	nds.ARM9Cycle = 0;
 	nds.ARM7Cycle = 0;
+	nds.wifiCycle = 0;
 	nds.cycles = 0;
 	memset(nds.timerCycle, 0, sizeof(s32) * 2 * 4);
 	memset(nds.timerOver, 0, sizeof(BOOL) * 2 * 4);
@@ -1028,20 +1044,16 @@ void NDS_Reset( void)
 
 	delete header;
 
-	GPU_Reset(MainScreen.gpu, 0);
-	GPU_Reset(SubScreen.gpu, 1);
+	Screen_Reset();
 	gfx3d_reset();
 	gpu3D->NDS_3D_Reset();
 	SPU_Reset();
-	cheatsSearchClose();
 
 #ifdef EXPERIMENTAL_WIFI
 	WIFI_Init(&wifiMac);
 
-	if(wifiMac.netEnabled) {
-		WIFI_SoftAP_Shutdown(&wifiMac);
-		WIFI_SoftAP_Init(&wifiMac);
-	}
+	WIFI_SoftAP_Shutdown(&wifiMac);
+	WIFI_SoftAP_Init(&wifiMac);
 #endif
 
 	memcpy(FW_Mac, (MMU.fw.data + 0x36), 6);
@@ -1053,11 +1065,156 @@ int NDS_ImportSave(const char *filename)
 		return 0;
 
 	if (memcmp(filename+strlen(filename)-4, ".duc", 4) == 0)
-		return mc_load_duc(&MMU.bupmem, filename);
+		return MMU_new.backupDevice.load_duc(filename);
+	else
+		return MMU_new.backupDevice.load_raw(filename);
 
 	return 0;
 }
 
+bool NDS_ExportSave(const char *filename)
+{
+	if (strlen(filename) < 4)
+		return false;
+
+	if (memcmp(filename+strlen(filename)-4, ".sav", 4) == 0)
+		return MMU_new.backupDevice.save_raw(filename);
+
+	return false;
+}
+
+static int WritePNGChunk(FILE *fp, uint32 size, const char *type, const uint8 *data)
+{
+	uint32 crc;
+
+	uint8 tempo[4];
+
+	tempo[0]=size>>24;
+	tempo[1]=size>>16;
+	tempo[2]=size>>8;
+	tempo[3]=size;
+
+	if(fwrite(tempo,4,1,fp)!=1)
+		return 0;
+	if(fwrite(type,4,1,fp)!=1)
+		return 0;
+
+	if(size)
+		if(fwrite(data,1,size,fp)!=size)
+			return 0;
+
+	crc = crc32(0,(uint8 *)type,4);
+	if(size)
+		crc = crc32(crc,data,size);
+
+	tempo[0]=crc>>24;
+	tempo[1]=crc>>16;
+	tempo[2]=crc>>8;
+	tempo[3]=crc;
+
+	if(fwrite(tempo,4,1,fp)!=1)
+		return 0;
+	return 1;
+}
+int NDS_WritePNG(const char *fname)
+{
+	int x, y;
+	int width=256;
+	int height=192*2;
+	u16 * bmp = (u16 *)GPU_tempScreen;
+	FILE *pp=NULL;
+	uint8 *compmem = NULL;
+	uLongf compmemsize = (uLongf)( (height * (width + 1) * 3 * 1.001 + 1) + 12 );
+
+	if(!(compmem=(uint8 *)malloc(compmemsize)))
+		return 0;
+
+	if(!(pp=fopen(fname, "wb")))
+	{
+		return 0;
+	}
+	{
+		static uint8 header[8]={137,80,78,71,13,10,26,10};
+		if(fwrite(header,8,1,pp)!=1)
+			goto PNGerr;
+	}
+
+	{
+		uint8 chunko[13];
+
+		chunko[0] = width >> 24;		// Width
+		chunko[1] = width >> 16;
+		chunko[2] = width >> 8;
+		chunko[3] = width;
+
+		chunko[4] = height >> 24;		// Height
+		chunko[5] = height >> 16;
+		chunko[6] = height >> 8;
+		chunko[7] = height;
+
+		chunko[8]=8;				// 8 bits per sample(24 bits per pixel)
+		chunko[9]=2;				// Color type; RGB triplet
+		chunko[10]=0;				// compression: deflate
+		chunko[11]=0;				// Basic adapative filter set(though none are used).
+		chunko[12]=0;				// No interlace.
+
+		if(!WritePNGChunk(pp,13,"IHDR",chunko))
+			goto PNGerr;
+	}
+
+	{
+		uint8 *tmp_buffer;
+		uint8 *tmp_inc;
+		tmp_inc = tmp_buffer = (uint8 *)malloc((width * 3 + 1) * height);
+
+		for(y=0;y<height;y++)
+		{
+			*tmp_inc = 0;
+			tmp_inc++;
+			for(x=0;x<width;x++)
+			{
+				int r,g,b;
+				u16 pixel = bmp[y*256+x];
+				r = pixel>>10;
+				pixel-=r<<10;
+				g = pixel>>5;
+				pixel-=g<<5;
+				b = pixel;
+				r*=255/31;
+				g*=255/31;
+				b*=255/31;
+				tmp_inc[0] = b;
+				tmp_inc[1] = g;
+				tmp_inc[2] = r;
+				tmp_inc += 3;
+			}
+		}
+
+		if(compress(compmem, &compmemsize, tmp_buffer, height * (width * 3 + 1))!=Z_OK)
+		{
+			if(tmp_buffer) free(tmp_buffer);
+			goto PNGerr;
+		}
+		if(tmp_buffer) free(tmp_buffer);
+		if(!WritePNGChunk(pp,compmemsize,"IDAT",compmem))
+			goto PNGerr;
+	}
+	if(!WritePNGChunk(pp,0,"IEND",0))
+		goto PNGerr;
+
+	free(compmem);
+	fclose(pp);
+
+	return 1;
+
+PNGerr:
+	if(compmem)
+		free(compmem);
+	if(pp)
+		fclose(pp);
+	return(0);
+}
+
 typedef struct
 {
 	u32 size;
@@ -1090,7 +1247,7 @@ int NDS_WriteBMP(const char *filename)
 	bmpimgheader_struct imageheader;
 	FILE *file;
 	int i,j;
-	u16 * bmp = (u16 *)GPU_screen;
+	u16 * bmp = (u16 *)GPU_tempScreen;
 	size_t elems_written = 0;
 
 	memset(&fileheader, 0, sizeof(fileheader));
@@ -1418,26 +1575,31 @@ int NDS_LoadFirmware(const char *filename)
 
 void NDS_Sleep() { nds.sleeping = TRUE; }
 
-bool skipThisFrame = false;
-bool skipThisFrame3D = false;
+bool SkipCur2DFrame = false, SkipNext2DFrame = false;
+bool SkipCur3DFrame = false;
 
-void NDS_SkipFrame(bool skip) { skipThisFrame = skip; }
+void NDS_SkipNextFrame() { SkipNext2DFrame = true; SkipCur3DFrame = true; }
 
 #define INDEX(i) ((((i)>>16)&0xFF0)|(((i)>>4)&0xF))
 
+//#define LOG_ARM9
+//#define LOG_ARM7
+//static bool dolog = false;
+
 
 template<bool FORCE>
-u32 NDS_exec(s32 nb)
+void NDS_exec(s32 nb)
 {
 	int i, j;
 
-	//TODO - since NDS_exec is not necessarily called one frame at a time, this could be wrong.
 	LagFrameFlag=1;
 
-	nb += nds.cycles;//(nds.cycles>>26)<<26;
+	if((currFrameCounter&63) == 0)
+		MMU_new.backupDevice.lazy_flush();
 
 	//increase this to execute more instructions in each batch (reducing overhead)
 	//the value of 4 seems to optimize speed.. do lower values increase precision? 
+	//answer: YES, MAYBE. and after 0.9.3 we need to study it (spider-man)
 	const int INSTRUCTIONS_PER_BATCH = 4;
 
 	//decreasing this should increase precision at the cost of speed
@@ -1450,19 +1612,6 @@ u32 NDS_exec(s32 nb)
 		{
 			if(nds.ARM9Cycle<=nds.cycles)
 			{
-#ifdef LOG_ARM9
-				if(logcount==3){
-					if(NDS_ARM9.CPSR.bits.T)
-						des_thumb_instructions_set[(NDS_ARM9.instruction)>>6](NDS_ARM9.instruct_adr, NDS_ARM9.instruction, logbuf);
-					else
-						des_arm_instructions_set[INDEX(NDS_ARM9.instruction)](NDS_ARM9.instruct_adr, NDS_ARM9.instruction, logbuf);
-					sprintf(logbuf + strlen(logbuf), "%s\t%08X\n\t R00: %08X, R01: %08X, R02: %08X, R03: %08X, R04: %08X, R05: %08X, R06: %08X, R07: %08X,\n\t R08: %08X, R09: %08X, R10: %08X, R11: %08X, R12: %08X, R13: %08X, R14: %08X, R15: %08X,\n\t CPSR: %08X , SPSR: %08X",
-						NDS_ARM9.instruction, NDS_ARM9.R[0],  NDS_ARM9.R[1],  NDS_ARM9.R[2],  NDS_ARM9.R[3],  NDS_ARM9.R[4],  NDS_ARM9.R[5],  NDS_ARM9.R[6],  NDS_ARM9.R[7], 
-						NDS_ARM9.R[8],  NDS_ARM9.R[9],  NDS_ARM9.R[10],  NDS_ARM9.R[11],  NDS_ARM9.R[12],  NDS_ARM9.R[13],  NDS_ARM9.R[14],  NDS_ARM9.R[15],
-						NDS_ARM9.CPSR, NDS_ARM9.SPSR);  
-					LOG(logbuf);
-				}
-#endif
 				for (i = 0; i < INSTRUCTIONS_PER_BATCH && (!FORCE) && (execute); i++)
 				{
 					if(nds.sleeping) {
@@ -1472,12 +1621,33 @@ u32 NDS_exec(s32 nb)
 						nds.ARM9Cycle += CYCLES_TO_WAIT_FOR_IRQ;
 						nds.idleCycles += CYCLES_TO_WAIT_FOR_IRQ;
 						break; //it is rather pointless to do this more than once
-					} else
+					} else {
+						#ifdef LOG_ARM9
+						if(dolog)
+						{
+							char dasmbuf[1024];
+							char logbuf[4096];
+							if(NDS_ARM9.CPSR.bits.T)
+								des_thumb_instructions_set[((NDS_ARM9.instruction)>>6)&1023](NDS_ARM9.instruct_adr, NDS_ARM9.instruction, dasmbuf);
+							else
+								des_arm_instructions_set[INDEX(NDS_ARM9.instruction)](NDS_ARM9.instruct_adr, NDS_ARM9.instruction, dasmbuf);
+		
+							printf("%05d %08d 9:%08X %08X %-30s %08X R00:%08X R01:%08X R02:%08X R03:%08X R04:%08X R05:%08X R06:%08X R07:%08X R08:%08X R09:%08X R10:%08X R11:%08X R12:%08X R13:%08X R14:%08X R15:%08X\n",
+								currFrameCounter, nds.cycles, 
+								NDS_ARM9.instruct_adr,NDS_ARM9.instruction, dasmbuf, 
+								NDS_ARM9.R[0],  NDS_ARM9.R[1],  NDS_ARM9.R[2],  NDS_ARM9.R[3],  NDS_ARM9.R[4],  NDS_ARM9.R[5],  NDS_ARM9.R[6],  NDS_ARM9.R[7], 
+								NDS_ARM9.R[8],  NDS_ARM9.R[9],  NDS_ARM9.R[10],  NDS_ARM9.R[11],  NDS_ARM9.R[12],  NDS_ARM9.R[13],  NDS_ARM9.R[14],  NDS_ARM9.R[15]);  
+						}
+						#endif
+
+						bool r7zero = NDS_ARM9.R[7] == 0;
 						nds.ARM9Cycle += armcpu_exec<ARMCPU_ARM9>();
+						bool r7one = NDS_ARM9.R[7] == 1;
+					}
 				}
 #ifdef _WIN32
 #ifdef DEVELOPER
-				DisassemblerTools_Refresh(ARMCPU_ARM9);
+				DisassemblerTools_Refresh<ARMCPU_ARM9>();
 #endif
 #endif
 			}
@@ -1488,6 +1658,9 @@ u32 NDS_exec(s32 nb)
 			if((nds.ARM7Cycle % 0x3F03) == 0)
 			{
 				/* 3F03 arm7 cycles = ~1usec */
+				// WRONG!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+				// 1 usec = ~33 ARM7 cycles because ARM7 runs at 33.51 mhz
+				// 33.51 mhz = 33.51 million cycles per second = 33.51 cycles per usec
 				WIFI_usTrigger(&wifiMac) ;
 			}
 #endif
@@ -1497,19 +1670,6 @@ u32 NDS_exec(s32 nb)
 
 			if(nds.ARM7Cycle<=nds.cycles)
 			{
-#ifdef LOG_ARM7
-				if(logcount==1){
-					if(NDS_ARM7.CPSR.bits.T)
-						des_thumb_instructions_set[(NDS_ARM7.instruction)>>6](NDS_ARM7.instruct_adr, NDS_ARM7.instruction, logbuf);
-					else
-						des_arm_instructions_set[INDEX(NDS_ARM7.instruction)](NDS_ARM7.instruct_adr, NDS_ARM7.instruction, logbuf);
-					sprintf(logbuf + strlen(logbuf), "%s\n\t R00: %08X, R01: %08X, R02: %08X, R03: %08X, R04: %08X, R05: %08X, R06: %08X, R07: %08X,\n\t R08: %08X, R09: %08X, R10: %08X, R11: %08X, R12: %08X, R13: %08X, R14: %08X, R15: %08X,\n\t CPSR: %08X , SPSR: %08X",
-						NDS_ARM7.R[0],  NDS_ARM7.R[1],  NDS_ARM7.R[2],  NDS_ARM7.R[3],  NDS_ARM7.R[4],  NDS_ARM7.R[5],  NDS_ARM7.R[6],  NDS_ARM7.R[7], 
-						NDS_ARM7.R[8],  NDS_ARM7.R[9],  NDS_ARM7.R[10],  NDS_ARM7.R[11],  NDS_ARM7.R[12],  NDS_ARM7.R[13],  NDS_ARM7.R[14],  NDS_ARM7.R[15],
-						NDS_ARM7.CPSR, NDS_ARM7.SPSR);  
-					LOG(logbuf);
-				}
-#endif
 				for (i = 0; i < INSTRUCTIONS_PER_BATCH && (!FORCE) && (execute); i++)
 				{
 					if(nds.sleeping) {
@@ -1521,35 +1681,81 @@ u32 NDS_exec(s32 nb)
 						break; //it is rather pointless to do this more than once
 					}
 					else
+					{
+						#ifdef LOG_ARM7
+						if(dolog)
+						{
+							char dasmbuf[1024];
+							if(NDS_ARM7.CPSR.bits.T)
+								des_thumb_instructions_set[((NDS_ARM7.instruction)>>6)&1023](NDS_ARM7.instruct_adr, NDS_ARM7.instruction, dasmbuf);
+							else
+								des_arm_instructions_set[INDEX(NDS_ARM7.instruction)](NDS_ARM7.instruct_adr, NDS_ARM7.instruction, dasmbuf);
+							printf("%05d %08d 7:%08X %08X %-30s %08X R00:%08X R01:%08X R02:%08X R03:%08X R04:%08X R05:%08X R06:%08X R07:%08X R08:%08X R09:%08X R10:%08X R11:%08X R12:%08X R13:%08X R14:%08X R15:%08X\n",
+								currFrameCounter, nds.cycles, 
+								NDS_ARM7.instruct_adr,NDS_ARM7.instruction, dasmbuf, 
+								NDS_ARM7.R[0],  NDS_ARM7.R[1],  NDS_ARM7.R[2],  NDS_ARM7.R[3],  NDS_ARM7.R[4],  NDS_ARM7.R[5],  NDS_ARM7.R[6],  NDS_ARM7.R[7], 
+								NDS_ARM7.R[8],  NDS_ARM7.R[9],  NDS_ARM7.R[10],  NDS_ARM7.R[11],  NDS_ARM7.R[12],  NDS_ARM7.R[13],  NDS_ARM7.R[14],  NDS_ARM7.R[15],
+								NDS_ARM7.CPSR, NDS_ARM7.SPSR);  
+						}
+						#endif
 						nds.ARM7Cycle += (armcpu_exec<ARMCPU_ARM7>()<<1);
+					}
 				}
 #ifdef _WIN32
 #ifdef DEVELOPER
-				DisassemblerTools_Refresh(ARMCPU_ARM7);
+				DisassemblerTools_Refresh<ARMCPU_ARM7>();
 #endif
 #endif
 			}
 
+//this doesnt work anyway. why take a speed hit for public releases?
+// Luigi__: I don't agree. We should start include wifi emulation in public releases
+// because it already allows games to not hang during wifi operations
+// and thus games that perform wifi checks upon boot shouldn't hang with wifi
+// emulation turned on.
+#ifndef PUBLIC_RELEASE
 #ifdef EXPERIMENTAL_WIFI
-			/* TODO: the wifi stuff isn't actually clocked by the ARM7 clock, */
-			/* but by a 22 mhz oscillator. */
-			if(nds7old < nds.ARM7Cycle)
-			{
-				nds7old %= 67;
-				int nds7new = (nds.ARM7Cycle % 67);
+			// FIXME: the wifi stuff isn't actually clocked by the ARM7 clock, 
+			// but by a 22 mhz oscillator.
+			// But do we really care? while we're triggering wifi core every microsecond,
+			// everything is fine, isn't it?
 
-				if(nds7old > nds7new)
-				{
-					WIFI_usTrigger(&wifiMac);
-					WIFI_SoftAP_usTrigger(&wifiMac);
-				}
+			//zeromus way: i did this in a rush and it is not perfect, but it is more like what it needs to be
+			//nds.wifiCycle += (nds.ARM7Cycle-nds7old)<<16;
+			//while(nds.wifiCycle > 0)
+			//{
+			//	nds.wifiCycle -= 946453; //22*22000000*65536/ARM7_CLOCK;
+			//	WIFI_usTrigger(&wifiMac);
+			//	WIFI_SoftAP_usTrigger(&wifiMac);
+			//}
+			//
+			//luigi's way>
+			// new one, more like zeromus way
+			// but I can't understand why it uses that value (22*22000000*65536/ARM7_CLOCK)
+			nds.wifiCycle += (nds.ARM7Cycle - nds7old) << 16;
+			while (nds.wifiCycle > 0)
+			{
+				// 2196372 ~= (ARM7_CLOCK << 16) / 1000000
+				// This value makes more sense to me, because:
+				// ARM7_CLOCK   = 33.51 mhz
+				//				= 33513982 cycles per second
+				// 				= 33.513982 cycles per microsecond
+				// The value is left shifted by 16 for more precision.
+				nds.wifiCycle -= (2196372 << 1);
+
+				WIFI_usTrigger(&wifiMac);
+				WIFI_SoftAP_usTrigger(&wifiMac);
 			}
 #endif
-		}
+#endif
+		} // for (j = 0; j < INSTRUCTIONS_PER_BATCH && (!FORCE) && (execute); j++)
 
 		if(!nds.sleeping)
 		{
 
+		//	for(i = 0; i < INSTRUCTIONS_PER_BATCH && (!FORCE) && (execute); i++)
+		//	{
+
 		nds.cycles = std::min(nds.ARM9Cycle,nds.ARM7Cycle);
 
 		//debug();
@@ -1562,17 +1768,17 @@ u32 NDS_exec(s32 nb)
 				T1WriteWord(MMU.ARM7_REG, 4, T1ReadWord(MMU.ARM7_REG, 4) | 2);
 				NDS_ARM9HBlankInt();
 				NDS_ARM7HBlankInt();
-
+				SPU_Emulate_core();
+				driver->AVI_SoundUpdate(SPU_core->outbuf,spu_core_samples);
+				WAV_WavSoundUpdate(SPU_core->outbuf,spu_core_samples);
 				if(nds.VCount<192)
 				{
-					if(!skipThisFrame)
+					if(!SkipCur2DFrame)
 					{
 						GPU_ligne(&MainScreen, nds.VCount);
 						GPU_ligne(&SubScreen, nds.VCount);
 					}
 
-					skipThisFrame3D = skipThisFrame;
-
 					if(MMU.DMAStartTime[0][0] == 2)
 						MMU_doDMA<ARMCPU_ARM9>(0);
 					if(MMU.DMAStartTime[0][1] == 2)
@@ -1651,6 +1857,14 @@ u32 NDS_exec(s32 nb)
 					//osdA->update();
 					gfx3d_VBlankSignal();
 
+					if(SkipNext2DFrame)
+					{
+						SkipCur2DFrame = true;
+						SkipNext2DFrame = false;
+					}
+					else
+						SkipCur2DFrame = false;
+
 					T1WriteWord(ARM9Mem.ARM9_REG, 4, T1ReadWord(ARM9Mem.ARM9_REG, 4) | 1);
 					T1WriteWord(MMU.ARM7_REG, 4, T1ReadWord(MMU.ARM7_REG, 4) | 1);
 					NDS_ARM9VBlankInt();
@@ -1681,9 +1895,10 @@ u32 NDS_exec(s32 nb)
 					if(MMU.DMAStartTime[1][3] == 1)
 						MMU_doDMA<ARMCPU_ARM7>(3);
 				}
-				else if(nds.VCount==215)
+				else if(nds.VCount==214)
 				{
-					gfx3d_VBlankEndSignal(false);
+					gfx3d_VBlankEndSignal(SkipCur3DFrame);
+					SkipCur3DFrame = false;
 				}
 				else if(nds.VCount==263)
 				{
@@ -1739,7 +1954,9 @@ u32 NDS_exec(s32 nb)
 				}
 
 				T1WriteWord(ARM9Mem.ARM9_REG, 6, nds.VCount);
+				T1WriteWord(ARM9Mem.ARM9_REG, 0x1006, nds.VCount);
 				T1WriteWord(MMU.ARM7_REG, 6, nds.VCount);
+				T1WriteWord(MMU.ARM7_REG, 0x1006, nds.VCount);
 
 				vmatch = T1ReadWord(ARM9Mem.ARM9_REG, 4);
 				if(nds.VCount==((vmatch>>8)|((vmatch<<1)&(1<<8))))
@@ -1761,7 +1978,16 @@ u32 NDS_exec(s32 nb)
 				else
 					T1WriteWord(MMU.ARM7_REG, 4, T1ReadWord(MMU.ARM7_REG, 4) & 0xFFFB);
 			}
-		}
+		} // if(nds.cycles >= nds.nextHBlank)
+
+	/*	} // for(i = 0; i < INSTRUCTIONS_PER_BATCH && (!FORCE) && (execute); i++)
+
+		} // if(!nds.sleeping)
+
+		} // for (j = 0; j < INSTRUCTIONS_PER_BATCH && (!FORCE) && (execute); j++)
+
+		if(!nds.sleeping)
+		{*/
 
 		if(MMU.divRunning)
 		{
@@ -2235,7 +2461,7 @@ u32 NDS_exec(s32 nb)
 		lagframecounter = 0;
 	}
 
-	return nds.cycles;
+	currFrameCounter++;
 }
 
 static std::string MakeInputDisplayString(u16 pad, const std::string* Buttons, int count) {
@@ -2259,30 +2485,95 @@ static std::string MakeInputDisplayString(u16 pad, u16 padExt) {
     return s;
 }
 
+void ClearAutoHold(void) {
+	
+	for (int i=0; i < 12; i++) {
+		AutoHold.hold(i)=false;
+	}
+}
+
 void NDS_setPadFromMovie(u16 pad)
 {
 #define FIX(b,n) (((pad>>n)&1)!=0)
 	NDS_setPad(
-		FIX(pad,0),
-		FIX(pad,1),
+		FIX(pad,12), //R
+		FIX(pad,11), //L
+		FIX(pad,10), //D
+		FIX(pad,9), //U 
+		FIX(pad,7), //Select
+		FIX(pad,8), //Start
+		FIX(pad,6), //B
+		FIX(pad,5), //A
+		FIX(pad,4), //Y
+		FIX(pad,3), //X
 		FIX(pad,2),
-		FIX(pad,3),
-		FIX(pad,4),
-		FIX(pad,5),
-		FIX(pad,6),
-		FIX(pad,7),
-		FIX(pad,8),
-		FIX(pad,9),
-		FIX(pad,10),
-		FIX(pad,11),
-		FIX(pad,12),
-		FIX(pad,13)
+		FIX(pad,1),
+		FIX(pad,0),
+		movie_lid
 		);
 #undef FIX
+
 }
 
+turbo Turbo;
+turbotime TurboTime;
+
+static void SetTurbo(bool (&pad) [12]) {
+
+	bool turbo[4] = {true, false, true, false};
+	bool currentbutton;
+
+	for (int i=0; i < 12; i++) {
+		currentbutton=Turbo.button(i);
+
+		if(currentbutton) {
+			pad[i]=turbo[TurboTime.time(i)-1];
+			
+			if(TurboTime.time(i) >= (int)ARRAY_SIZE(turbo))
+				TurboTime.time(i)=0;
+		}
+		else
+			TurboTime.time(i)=0; //reset timer if the button isn't pressed
+	}
+	for (int i=0; i<12; i++)
+		TurboTime.time(i)++;
+}
+
+autohold AutoHold;
+
 void NDS_setPad(bool R,bool L,bool D,bool U,bool T,bool S,bool B,bool A,bool Y,bool X,bool W,bool E,bool G, bool F)
 {
+
+	bool padarray[12] = {R, L, D, U, T, S, B, A, Y, X, W, E};
+
+	SetTurbo(padarray);
+
+	R=padarray[0];
+	L=padarray[1];
+	D=padarray[2];
+	U=padarray[3];
+	T=padarray[4];
+	S=padarray[5];
+	B=padarray[6];
+	A=padarray[7];
+	Y=padarray[8];
+	X=padarray[9];
+	W=padarray[10];
+	E=padarray[11];
+
+	if(AutoHold.Right) R=!padarray[0];
+	if(AutoHold.Left)  L=!padarray[1];
+	if(AutoHold.Down)  D=!padarray[2];
+	if(AutoHold.Up)    U=!padarray[3];
+	if(AutoHold.Select)T=!padarray[4];
+	if(AutoHold.Start) S=!padarray[5];
+	if(AutoHold.B)     B=!padarray[6];
+	if(AutoHold.A)     A=!padarray[7];
+	if(AutoHold.Y)     Y=!padarray[8];
+	if(AutoHold.X)     X=!padarray[9];
+	if(AutoHold.L)     W=!padarray[10];
+	if(AutoHold.R)     E=!padarray[11];
+
 	//this macro is the opposite of what you would expect
 #define FIX(b) (b?0:0x80)
 
@@ -2367,22 +2658,22 @@ void NDS_setPad(bool R,bool L,bool D,bool U,bool T,bool S,bool B,bool A,bool Y,b
 	g = FIX(G);
 	f = FIX(F);
 
+	if(f) movie_lid=true;
+	else movie_lid=false;
 
 	nds.pad =
-		(FIX(r)<<0)|
-		(FIX(l)<<1)|
-		(FIX(d)<<2)|
-		(FIX(u)<<3)|
-		(FIX(t)<<4)|
-		(FIX(s)<<5)|
+		(FIX(r)<<12)|
+		(FIX(l)<<11)|
+		(FIX(d)<<10)|
+		(FIX(u)<<9)|
+		(FIX(s)<<8)|
+		(FIX(t)<<7)|
 		(FIX(b)<<6)|
-		(FIX(a)<<7)|
-		(FIX(y)<<8)|
-		(FIX(x)<<9)|
-		(FIX(w)<<10)|
-		(FIX(e)<<11)|
-		(FIX(g)<<12)|
-		(FIX(f)<<13);
+		(FIX(a)<<5)|
+		(FIX(y)<<4)|
+		(FIX(x)<<3)|
+		(FIX(w)<<2)|
+		(FIX(e)<<1);
 
 	// TODO: low power IRQ
 }
@@ -2393,5 +2684,5 @@ void emu_halt() {
 }
 
 //these templates needed to be instantiated manually
-template u32 NDS_exec<FALSE>(s32 nb);
-template u32 NDS_exec<TRUE>(s32 nb);
+template void NDS_exec<FALSE>(s32 nb);
+template void NDS_exec<TRUE>(s32 nb);
