@@ -2,6 +2,8 @@
     yopyop156@ifrance.com
     yopyop156.ifrance.com
 
+    Copyright (C) 2008-2009 DeSmuME team
+
     This file is part of DeSmuME
 
     DeSmuME is free software; you can redistribute it and/or modify
@@ -16,7 +18,7 @@
 
     You should have received a copy of the GNU General Public License
     along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 #ifndef NDSSYSTEM_H
@@ -25,15 +27,73 @@
 #include <string.h>
 #include "armcpu.h"
 #include "MMU.h"
-
+#include "driver.h"
 #include "GPU.h"
 #include "SPU.h"
-
 #include "mem.h"
 #include "wifi.h"
 
 #include <string>
 
+struct turbo {
+	bool Right;
+	bool Left;
+	bool Down;
+	bool Up;
+	bool Select;
+	bool Start;
+	bool B;
+	bool A;
+	bool Y;
+	bool X;
+	bool L;
+	bool R;
+
+	bool &button(int i) { return ((bool*)this)[i]; }
+};
+
+extern turbo Turbo;
+
+struct turbotime {
+	int Right;
+	int Left;
+	int Down;
+	int Up;
+	int Select;
+	int Start;
+	int B;
+	int A;
+	int Y;
+	int X;
+	int L;
+	int R;
+
+	int &time(int i) { return ((int*)this)[i]; }
+};
+
+extern turbotime TurboTime;
+
+struct autohold {
+	bool Right;
+	bool Left;
+	bool Down;
+	bool Up;
+	bool Select;
+	bool Start;
+	bool B;
+	bool A;
+	bool Y;
+	bool X;
+	bool L;
+	bool R;
+
+	bool &hold(int i) { return ((bool*)this)[i]; }
+};
+
+extern autohold AutoHold;
+
+int NDS_WritePNG(const char *fname);
+
 extern volatile BOOL execute;
 extern BOOL click;
 extern char pathToROM[MAX_PATH];
@@ -113,6 +173,7 @@ typedef struct
 {
 	s32 ARM9Cycle;
 	s32 ARM7Cycle;
+	s32 wifiCycle;
 	s32 cycles;
 	s32 timerCycle[2][4];
 	BOOL timerOver[2][4];
@@ -208,6 +269,16 @@ NDS_FillDefaultFirmwareConfigData( struct NDS_fw_config_data *fw_config);
 
 BOOL NDS_SetROM(u8 * rom, u32 mask);
 NDS_header * NDS_getROMHeader(void);
+
+struct GameInfo
+{
+	u32 crc;
+	NDS_header header;
+	char ROMserial[20];
+	void populate();
+};
+
+extern GameInfo gameInfo;
  
 void NDS_setTouchPos(u16 x, u16 y);
 void NDS_releaseTouch(void);
@@ -215,14 +286,15 @@ void NDS_setPad(bool R,bool L,bool D,bool U,bool T,bool S,bool B,bool A,bool Y,b
 void NDS_setPadFromMovie(u16 pad);
 
 #ifdef EXPERIMENTAL_GBASLOT
-int NDS_LoadROM(const char *filename, int bmtype, u32 bmsize);
+int NDS_LoadROM(const char *filename);
 #else
-int NDS_LoadROM(const char *filename, int bmtype, u32 bmsize,
+int NDS_LoadROM(const char *filename,
                  const char *cflash_disk_image_file);
 #endif
 void NDS_FreeROM(void);
-void NDS_Reset(void);
+void NDS_Reset();
 int NDS_ImportSave(const char *filename);
+bool NDS_ExportSave(const char *filename);
 
 int NDS_WriteBMP(const char *filename);
 int NDS_LoadFirmware(const char *filename);
@@ -230,15 +302,13 @@ int NDS_CreateDummyFirmware( struct NDS_fw_config_data *user_settings);
 
 void NDS_Sleep();
 
-void NDS_SkipFrame(bool skip);
+void NDS_SkipNextFrame();
+#define NDS_SkipFrame(s) if(s) NDS_SkipNext2DFrame();
 
-template<bool FORCE>
-u32 NDS_exec(s32 nb);
+template<bool FORCE> void NDS_exec(s32 nb = 560190<<1);
 
 extern int lagframecounter;
 
-inline u32 NDS_exec(s32 nb) { return NDS_exec<false>(nb); }
-
        static INLINE void NDS_ARM9HBlankInt(void)
        {
             if(T1ReadWord(ARM9Mem.ARM9_REG, 4) & 0x10)
@@ -298,6 +368,10 @@ extern struct TCommonSettings {
 		, BootFromFirmware(false)
 		, DebugConsole(false)
 		, wifiBridgeAdapterNum(0)
+		, spuInterpolationMode(SPUInterpolation_Linear)
+		, spuAdpcmCache(false)
+		, gfx3d_flushMode(0)
+		, manualBackupType(0)
 	{
 		strcpy(ARM9BIOS, "biosnds9.bin");
 		strcpy(ARM7BIOS, "biosnds7.bin");
@@ -317,23 +391,35 @@ extern struct TCommonSettings {
 	bool DebugConsole;
 	
 	int wifiBridgeAdapterNum;
+
+	SPUInterpolationMode spuInterpolationMode;
+	bool spuAdpcmCache;
+
+	//this is a temporary hack until we straighten out the flushing logic and/or gxfifo
+	int gfx3d_flushMode;
+
+	//this is the user's choice of manual backup type, for cases when the autodetection can't be trusted
+	int manualBackupType;
+
+	struct _ShowGpu {
+		_ShowGpu() : main(true), sub(true) {}
+		union {
+			struct { bool main,sub; };
+			bool screens[2];
+		};
+	} showGpu;
+
 } CommonSettings;
 
-extern char ROMserial[20];
-
-//this should be moved to a driver.h later, but for now, here they are.
-//each platform needs to implement this, although it doesnt need to implement any functions
-class Driver {
-public:
-	virtual BOOL WIFI_Host_InitSystem() { return FALSE; }
-	virtual void WIFI_Host_ShutdownSystem() {}
-};
-extern Driver* driver;
 
 extern std::string InputDisplayString;
 extern int LagFrameFlag;
 extern int lastLag, TotalLagFrames;
 
+void MovieSRAM();
+
+void ClearAutoHold(void);
+
 #endif
 
  	  	 
