@@ -1,4 +1,7 @@
-/*  Copyright (C) 2006 Theo Berkau
+/*  SPU.h
+
+	Copyright 2006 Theo Berkau
+    Copyright (C) 2006-2009 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -14,7 +17,7 @@
 
     You should have received a copy of the GNU General Public License
     along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 #ifndef SPU_H
@@ -22,12 +25,19 @@
 
 #include "types.h"
 #include <iosfwd>
+#include <string>
 
 #define SNDCORE_DEFAULT         -1
 #define SNDCORE_DUMMY           0
-#define SNDCORE_FILEWRITE       1
 
-typedef struct
+enum SPUInterpolationMode
+{
+	SPUInterpolation_None = 0,
+	SPUInterpolation_Linear = 1,
+	SPUInterpolation_Cosine = 2
+};
+
+struct SoundInterface_struct
 {
    int id;
    const char *Name;
@@ -38,13 +48,19 @@ typedef struct
    void (*MuteAudio)();
    void (*UnMuteAudio)();
    void (*SetVolume)(int volume);
-} SoundInterface_struct;
+};
 
 extern SoundInterface_struct SNDDummy;
 extern SoundInterface_struct SNDFile;
+extern int SPU_currentCoreNum;
+
+class ADPCMCacheItem;
 
 struct channel_struct
 {
+	channel_struct()
+		: cacheItem(NULL)
+	{}
 	int num;
    u8 vol;
    u8 datashift;
@@ -58,16 +74,21 @@ struct channel_struct
    u16 timer;
    u16 loopstart;
    u32 length;
-   s8 *buf8;
-   s16 *buf16;
+   u32 totlength;
+   double double_totlength_shifted;
+   union {
+		s8 *buf8;
+		s16 *buf16;
+   };
    double sampcnt;
    double sampinc;
    // ADPCM specific
-   int lastsampcnt;
+   u32 lastsampcnt;
    s16 pcm16b, pcm16b_last;
    int index;
    u16 x;
    s16 psgnoise_last;
+   ADPCMCacheItem *cacheItem;
 } ;
 
 class SPU_struct
@@ -112,6 +133,24 @@ extern SPU_struct *SPU_core, *SPU_user;
 extern int spu_core_samples;
 
 void spu_savestate(std::ostream* os);
-bool spu_loadstate(std::istream* is);
+bool spu_loadstate(std::istream* is, int size);
+
+class WavWriter
+{
+public:
+	WavWriter();
+	bool open(const std::string & fname);
+	void close();
+	void update(void* soundData, int numSamples);
+	bool isRecording() const;
+private:
+	FILE *spufp;
+};
+
+
+void WAV_End();
+bool WAV_Begin(const char* fname);
+bool WAV_IsRecording();
+void WAV_WavSoundUpdate(void* soundData, int numSamples);
 
 #endif
