@@ -22,6 +22,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <assert.h>
+#include "types.h"
 #include "arm_instructions.h"
 #include "thumb_instructions.h"
 #include "cp15.h"
@@ -365,7 +366,7 @@ template<u32 PROCNUM>
 static u32
 armcpu_prefetch()
 {
-	armcpu_t* armcpu = &ARMPROC;
+	armcpu_t* const armcpu = &ARMPROC;
 #ifdef GDB_STUB
 	u32 temp_instruction;
 #endif
