@@ -22,14 +22,14 @@
 #ifndef ARM_CPU
 #define ARM_CPU
 
-#include "types.h"
-#include "bits.h"
-#include "MMU.h"
-
 #define ARMCPU_ARM7 1
 #define ARMCPU_ARM9 0
 #define ARMPROC (PROCNUM?NDS_ARM7:NDS_ARM9)
 
+#include "types.h"
+#include "bits.h"
+#include "MMU.h"
+
 #define CODE(i)     (((i)>>25)&0x7)
 #define OPCODE(i)   (((i)>>21)&0xF)
 #define SIGNEBIT(i) BIT_N(i,20)
@@ -54,21 +54,22 @@ template<typename T>
 inline T SIGNED_UNDERFLOW(T a,T b,T c) { return ((BIT31(a)&BIT31(~(b))&BIT31(~c))|
 										  (BIT31(~a)&BIT31(b)&BIT31(c))); }
 
-#define EQ	0x0
-#define NE	0x1
-#define CS	0x2
-#define CC	0x3
-#define MI	0x4
-#define PL	0x5
-#define VS	0x6
-#define VC	0x7
-#define HI	0x8
-#define LS	0x9
-#define GE	0xA
-#define LT	0xB
-#define GT	0xC
-#define LE	0xD
-#define AL	0xE
+//zero 15-feb-2009 - these werent getting used and they were getting in my way
+//#define EQ	0x0
+//#define NE	0x1
+//#define CS	0x2
+//#define CC	0x3
+//#define MI	0x4
+//#define PL	0x5
+//#define VS	0x6
+//#define VC	0x7
+//#define HI	0x8
+//#define LS	0x9
+//#define GE	0xA
+//#define LT	0xB
+//#define GT	0xC
+//#define LE	0xD
+//#define AL	0xE
 
 extern const unsigned char arm_cond_table[16*16];
 
