@@ -6,11 +6,10 @@
 #ifndef __CFLASH_H__
 #define __CFLASH_H__
 
-#include "fat.h"
+#include "common.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+#ifndef EXPERIMENTAL_GBASLOT
+#include "fat.h"
 
 typedef struct {
 	int level,parent,filesInDir;
@@ -24,9 +23,6 @@ unsigned int cflash_read(unsigned int address);
 void cflash_write(unsigned int address,unsigned int data);
 
 void cflash_close( void);
-
-#ifdef __cplusplus
-}
 #endif
 
 #endif
