@@ -22,7 +22,6 @@
 #include <SDL/SDL_thread.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libgen.h>
 
 
 #ifndef VERSION
@@ -33,7 +32,7 @@
  * FIXME: Not sure how to detect OpenGL in a platform portable way.
  */
 #ifdef HAVE_GL_GL_H
-#define INCLUDE_OPENGL_2D 1
+#define INCLUDE_OPENGL_2D
 #endif
 
 #ifdef INCLUDE_OPENGL_2D
@@ -54,18 +53,16 @@
 #include "../render3D.h"
 #include "../rasterize.h"
 #include "../saves.h"
+#include "../mic.h"
 #ifdef GDB_STUB
 #include "../gdbstub.h"
 #endif
 
 volatile BOOL execute = FALSE;
 
-int savetype=MC_TYPE_AUTODETECT;
-u32 savesize=1;
-
 static float nds_screen_size_ratio = 1.0f;
 
-#define DISPLAY_FPS 1
+#define DISPLAY_FPS
 
 #ifdef DISPLAY_FPS
 #define NUM_FRAMES_TO_TIME 60
@@ -77,11 +74,11 @@ static float nds_screen_size_ratio = 1.0f;
 static SDL_Surface * surface;
 
 /* Flags to pass to SDL_SetVideoMode */
-static int sdl_videoFlags = 0;
+static int sdl_videoFlags;
 
 SoundInterface_struct *SNDCoreList[] = {
   &SNDDummy,
-  &SNDFile,
+  &SNDDummy,
   &SNDSDL,
   NULL
 };
@@ -105,6 +102,7 @@ const char * save_type_names[] = {
 
 
 /* Our keyboard config is different because of the directional keys */
+/* Please note that m is used for fake microphone */
 const u16 cli_kb_cfg[NB_KEYS] =
   { 
     SDLK_x,         // A
@@ -123,6 +121,8 @@ const u16 cli_kb_cfg[NB_KEYS] =
     SDLK_o          // BOOST
   };
 
+static BOOL enable_fake_mic;
+
 struct my_config {
   int load_slot;
   u16 arm9_gdb_port;
@@ -137,7 +137,10 @@ struct my_config {
   int opengl_2d;
   int soft_colour_convert;
 #endif
+  int auto_pause;
   int disable_limiter;
+  int frameskip;
+  int fps_limiter_frame_period;
 
   int firmware_language;
 
@@ -154,7 +157,10 @@ init_config( struct my_config *config) {
 
   config->disable_sound = 0;
 
+  config->auto_pause = 0;
   config->disable_limiter = 0;
+  config->frameskip = 0;
+  config->fps_limiter_frame_period = FPS_LIMITER_FRAME_PERIOD;
 
   config->nds_file = NULL;
 
@@ -185,8 +191,11 @@ fill_config( struct my_config *config,
       printf( "USAGE: %s [options] <nds-file>\n", argv[0]);
       printf( "OPTIONS:\n");
       printf( "   --load-slot=NUM     Loads savegame from slot NUM\n");
+      printf( "   --auto-pause        Pause emulation of focus is lost.\n");
       printf( "   --disable-sound     Disables the sound emulation\n");
       printf( "   --disable-limiter   Disables the 60 fps limiter\n");
+      printf( "   --frameskip=N       Set frameskip to N\n");
+      printf( "   --limiter-period=N  Set frame period of the fps limiter to N (default: %d)\n", FPS_LIMITER_FRAME_PERIOD);
       printf( "   --3d-engine=ENGINE  Select 3d rendering engine, available ENGINES:\n");
       printf( "                         0 = 3d disabled\n");
       printf( "                         1 = internal desmume software rasterizer (default)\n");
@@ -246,9 +255,36 @@ fill_config( struct my_config *config,
       config->soft_colour_convert = 1;
     }
 #endif
+    else if ( strcmp( argv[i], "--auto-pause") == 0) {
+      config->auto_pause = 1;
+    }
     else if ( strcmp( argv[i], "--disable-limiter") == 0) {
       config->disable_limiter = 1;
     }
+    else if ( strncmp( argv[i], "--frameskip=", 12) == 0) {
+      char *end_char;
+      int frameskip = strtoul(&argv[i][12], &end_char, 10);
+
+      if ( frameskip >= 0 ) {
+        config->frameskip = frameskip;
+      }
+      else {
+        fprintf(stderr, "frameskip must be >=0\n");
+        good_args = 0;
+      }
+    }
+    else if ( strncmp( argv[i], "--limiter-period=", 17) == 0) {
+      char *end_char;
+      int period = strtoul(&argv[i][17], &end_char, 10);
+
+      if ( period >= 0 && period <= 30 ) {
+        config->fps_limiter_frame_period = period;
+      }
+      else {
+        fprintf(stderr, "fps lmiter period must be >=0 and <= 30!\n");
+        good_args = 0;
+      }
+    }
     else if ( strncmp( argv[i], "--3d-engine=", 12) == 0) {
       char *end_char;
       int engine = strtoul( &argv[i][12], &end_char, 10);
@@ -396,14 +432,9 @@ fps_limiter_fn( Uint32 interval, void *param) {
 static int
 initGL( GLuint *screen_texture) {
   GLenum errCode;
-  int init_good = 1;
-  int i;
   u16 blank_texture[256 * 512];
 
-  for ( i = 0; i < 256 * 512; i++) {
-    blank_texture[i] = 0x001f;
-  }
-
+  memset(blank_texture, 0x001f, sizeof(blank_texture));
 
   /* Enable Texture Mapping */
   glEnable( GL_TEXTURE_2D );
@@ -441,10 +472,10 @@ initGL( GLuint *screen_texture) {
     errString = gluErrorString(errCode);
     fprintf( stderr, "Failed to init GL: %s\n", errString);
 
-    init_good = 0;
+    return 0;
   }
 
-  return init_good;
+  return 1;
 }
 
 static void
@@ -509,12 +540,6 @@ resizeWindow( u16 width, u16 height) {
       right = 256.0 * ((double)width / other_dimen);
     }
 
-    /*
-    printf("%d,%d\n", width, height);
-    printf("l %lf, r %lf, t %lf, b %lf, other dimen %lf\n",
-           left, right, top, bottom, other_dimen);
-    */
-
     /* get the area (0,0) to (256,384) into the middle of the viewport */
     gluOrtho2D( left, right, bottom, top);
   }
@@ -614,11 +639,85 @@ Draw( void) {
   return;
 }
 
+static void desmume_cycle(int *sdl_quit, int *boost, struct my_config * my_config)
+{
+    static unsigned short keypad;
+    static int focused = 1;
+    SDL_Event event;
+
+    /* Look for queued events and update keypad status */
+    /* IMPORTANT: Reenable joystick events iif needed. */
+    if(SDL_JoystickEventState(SDL_QUERY) == SDL_IGNORE)
+      SDL_JoystickEventState(SDL_ENABLE);
+
+    /* There's an event waiting to be processed? */
+    while ( !*sdl_quit &&
+        (SDL_PollEvent(&event) || (!focused && SDL_WaitEvent(&event))))
+      {
+        process_ctrls_event( event, &keypad, nds_screen_size_ratio);
+
+        switch (event.type)
+        {
+#ifdef INCLUDE_OPENGL_2D
+          case SDL_VIDEORESIZE:
+            resizeWindow( event.resize.w, event.resize.h);
+            break;
+#endif
+          case SDL_ACTIVEEVENT:
+            if (my_config->auto_pause && (event.active.state & SDL_APPINPUTFOCUS ))
+            {
+              if (event.active.gain)
+              {
+                focused = 1;
+                SPU_Pause(0);
+              }
+              else
+              {
+                focused = 0;
+                SPU_Pause(1);
+              }
+            }
+            break;
+
+          case SDL_KEYUP:
+            switch (event.key.keysym.sym)
+            {
+              case SDLK_ESCAPE:
+                *sdl_quit = 1;
+                break;
+              case SDLK_m:
+                enable_fake_mic = !enable_fake_mic;
+                Mic_DoNoise(enable_fake_mic);
+                break;
+              case SDLK_o:
+                *boost = !(*boost);
+                break;
+              default:
+                break;
+            }
+            break;
+ 
+          case SDL_QUIT:
+            *sdl_quit = 1;
+            break;
+        }
+      }
+
+    /* Update mouse position and click */
+    if(mouse.down) NDS_setTouchPos(mouse.x, mouse.y);
+    if(mouse.click)
+      { 
+        NDS_releaseTouch();
+        mouse.click = FALSE;
+      }
+
+    update_keypad(keypad);     /* Update keypad */
+    NDS_exec<false>();
+    SPU_Emulate_user();
+}
 
 int main(int argc, char ** argv) {
-  static unsigned short keypad = 0;
   struct my_config my_config;
-  u32 last_cycle = 0;
 #ifdef GDB_STUB
   gdbstub_handle_t arm9_gdb_stub;
   gdbstub_handle_t arm7_gdb_stub;
@@ -632,6 +731,8 @@ int main(int argc, char ** argv) {
   SDL_sem *fps_limiter_semaphore = NULL;
   SDL_TimerID limiter_timer = NULL;
   int sdl_quit = 0;
+  int boost = 0;
+  int error;
 
 #ifdef DISPLAY_FPS
   u32 fps_timing = 0;
@@ -704,9 +805,14 @@ int main(int argc, char ** argv) {
 
   NDS_3D_ChangeCore(my_config.engine_3d);
 
-  mmu_select_savetype(my_config.savetype, &savetype, &savesize);
+  backup_setManualBackupType(my_config.savetype);
 
-  if (NDS_LoadROM( my_config.nds_file, savetype, savesize, my_config.cflash_disk_image_file) < 0) {
+#ifdef EXPERIMENTAL_GBASLOT
+  error = NDS_LoadROM( my_config.nds_file );
+#else
+  error = NDS_LoadROM( my_config.nds_file, my_config.cflash_disk_image_file );
+#endif
+  if (error < 0) {
     fprintf(stderr, "error while loading %s\n", my_config.nds_file);
     exit(-1);
   }
@@ -724,12 +830,6 @@ int main(int argc, char ** argv) {
   }
 #endif
 
-  /*      // This has to get fixed yet
-          strncpy(szRomPath, dirname(argv[1]), ARRAY_SIZE(szRomPath));
-          cflash_close();
-          cflash_init();
-  */
-  
   execute = TRUE;
 
   if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) == -1)
@@ -817,7 +917,7 @@ int main(int argc, char ** argv) {
 
     /* start a SDL timer for every FPS_LIMITER_FRAME_PERIOD frames to keep us at 60 fps */
     if ( fps_limiter_semaphore != NULL) {
-      limiter_timer = SDL_AddTimer( 16 * FPS_LIMITER_FRAME_PERIOD,
+      limiter_timer = SDL_AddTimer( 16 * my_config.fps_limiter_frame_period,
                                   fps_limiter_fn, fps_limiter_semaphore);
     }
 
@@ -837,24 +937,7 @@ int main(int argc, char ** argv) {
   }
 
   while(!sdl_quit) {
-    /* Look for queued events and update keypad status */
-#ifdef INCLUDE_OPENGL_2D
-    sdl_quit = process_ctrls_events( &keypad, resizeWindow, nds_screen_size_ratio);
-#else
-    sdl_quit = process_ctrls_events( &keypad, NULL, nds_screen_size_ratio);
-#endif
-    /* Update mouse position and click */
-    if(mouse.down) NDS_setTouchPos(mouse.x, mouse.y);
-    if(mouse.click)
-      { 
-        NDS_releaseTouch();
-        mouse.click = FALSE;
-      }
-
-    update_keypad(keypad);     /* Update keypad */
-    last_cycle = NDS_exec<false>((560190 << 1) - last_cycle);
-    SPU_Emulate_user();
-	SPU_Emulate_core();
+    desmume_cycle(&sdl_quit, &boost, &my_config);
 
 #ifdef INCLUDE_OPENGL_2D
     if ( my_config.opengl_2d) {
@@ -864,9 +947,14 @@ int main(int argc, char ** argv) {
 #endif
       Draw();
 
-    if ( !my_config.disable_limiter) {
-      limiter_frame_counter += 1;
-      if ( limiter_frame_counter >= FPS_LIMITER_FRAME_PERIOD) {
+    for ( int i = 0; i < my_config.frameskip; i++ ) {
+        NDS_SkipNextFrame();
+        desmume_cycle(&sdl_quit, &boost, &my_config);
+    }
+
+    if ( !my_config.disable_limiter && !boost) {
+      limiter_frame_counter += 1 + my_config.frameskip;
+      if ( limiter_frame_counter >= my_config.fps_limiter_frame_period) {
         limiter_frame_counter = 0;
 
         /* wait for the timer to expire */
@@ -881,16 +969,15 @@ int main(int argc, char ** argv) {
     fps_previous_time = fps_temp_time;
 
     if ( fps_frame_counter == NUM_FRAMES_TO_TIME) {
-      char win_title[100];
+      char win_title[20];
       float fps = (float)fps_timing;
       fps /= NUM_FRAMES_TO_TIME * 1000.f;
       fps = 1.0f / fps;
 
-      //printf("fps %f\n", fps);
       fps_frame_counter = 0;
       fps_timing = 0;
 
-      sprintf( win_title, "Desmume %f", fps);
+      snprintf( win_title, sizeof(win_title), "Desmume %f", fps);
 
       SDL_WM_SetCaption( win_title, NULL);
     }
