@@ -61,16 +61,12 @@ NULL
 
 SoundInterface_struct *SNDCoreList[] = {
 &SNDDummy,
-&SNDFile,
 #ifdef DESMUME_COCOA
 &SNDOSX,
 #endif
 NULL
 };
 
-static int backupmemorytype=MC_TYPE_AUTODETECT;
-static u32 backupmemorysize=1;
-
 struct NDS_fw_config_data firmware;
 
 bool opengl_init()
@@ -380,7 +376,7 @@ bool opengl_init()
 		flash = NULL;
 
 	//load the rom
-	if(!NDS_LoadROM([filename cStringUsingEncoding:NSUTF8StringEncoding], backupmemorytype, backupmemorysize, flash) > 0)
+	if(!NDS_LoadROM([filename cStringUsingEncoding:NSUTF8StringEncoding], flash) > 0)
 	{
 		//if it didn't work give an error and dont unpause
 		messageDialog(NSLocalizedString(@"Error", nil), @"Could not open file");
@@ -628,6 +624,20 @@ bool opengl_init()
 	return speed_limit;
 }
 
+- (void)setSaveType:(int)savetype
+{
+	if(savetype < 0 || savetype > 6) savetype = 0;
+	
+	// Set the savetype
+	backup_setManualBackupType(savetype);
+}
+
+- (int)saveType
+{
+	return CommonSettings.manualBackupType;
+}
+
+
 - (void)touch:(NSPoint)point
 {
 	NDS_setTouchPos((unsigned short)point.x, (unsigned short)point.y);
@@ -1328,8 +1338,6 @@ bool opengl_init()
 	CGLLockContext((CGLContextObj)[gl_context CGLContextObj]);
 #endif
 
-	u32 cycles = 0;
-
 	NSDate *frame_start_date, *frame_end_date, *ideal_frame_end_date;
 
 	int frames_to_skip = 0;
@@ -1352,7 +1360,7 @@ bool opengl_init()
 
 			[execution_lock lock];
 
-			cycles = NDS_exec<FALSE>((560190<<1)-cycles);
+			NDS_exec<false>();
 
 			[sound_lock lock];
 			int x;
