@@ -88,9 +88,8 @@ void update_keypad(u16 keys);
 u16 get_keypad( void);
 u16 lookup_key (u16 keyval);
 u16 lookup_joy_key (u16 keyval);
-int
-process_ctrls_events( u16 *keypad,
-                      void (*external_videoResizeFn)( u16 width, u16 height),
+void
+process_ctrls_event( SDL_Event& event, u16 *keypad,
                       float nds_screen_size_ratio);
 
 void
