@@ -1,8 +1,8 @@
 /*	Copyright (C) 2006 yopyop
     yopyop156@ifrance.com
-    yopyop156.ifrance.com 
+    yopyop156.ifrance.com
 
-	Copyright (C) 2008-2009 DeSmuME team
+    Copyright (C) 2008-2009 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -23,11 +23,12 @@
 
 //This file implements the geometry engine hardware component.
 //This handles almost all of the work of 3d rendering, leaving the renderer
-// plugin responsible only for drawing primitives.
+//plugin responsible only for drawing primitives.
 
 #include <algorithm>
 #include <assert.h>
 #include <math.h>
+#include <string.h>
 #include "armcpu.h"
 #include "debug.h"
 #include "gfx3d.h"
@@ -38,9 +39,31 @@
 #include "mem.h"
 #include "types.h"
 #include "saves.h"
+#include "NDSSystem.h"
 #include "readwrite.h"
 #include "FIFO.h"
 
+/*
+thoughts on flush timing:
+I think a flush is supposed to queue up and wait to happen during vblank sometime.
+But, we have some games that continue to do work after a flush but before a vblank.
+Since our timing is bad anyway, and we're not sure when the flush is really supposed to happen,
+then this leaves us in a bad situation.
+What makes it worse is that if flush is supposed to be deferred, then we have to queue these
+errant geometry commands. That would require a better gxfifo we have now, and some mechanism to block
+while the geometry engine is stalled (which doesnt exist).
+Since these errant games are nevertheless using flush command to represent the end of a frame, we deem this
+a good time to execute an actual flush.
+I think we originally didnt do this because we found some game that it glitched, but that may have been
+resolved since then by deferring actual rendering to the next vcount=0 (giving textures enough time to upload).
+But since we're not sure how we'll eventually want this, I am leaving it sort of reconfigurable, doing all the work
+in this function: */
+static void gfx3d_doFlush();
+
+
+using std::max;
+using std::min;
+
 GFX3D gfx3d;
 
 //tables that are provided to anyone
@@ -79,6 +102,11 @@ CACHE_ALIGN const u8 material_3bit_to_8bit[] = {
 	0x00, 0x24, 0x49, 0x6D, 0x92, 0xB6, 0xDB, 0xFF
 };
 
+//maybe not very precise
+CACHE_ALIGN const u8 material_3bit_to_5bit[] = {
+	0, 4, 8, 13, 17, 22, 26, 31
+};
+
 CACHE_ALIGN const u8 alpha_5bit_to_4bit[] = {
 	0x00, 0x00,
 	0x01, 0x01,
@@ -98,6 +126,13 @@ CACHE_ALIGN const u8 alpha_5bit_to_4bit[] = {
 	0x10, 0x10
 };
 
+CACHE_ALIGN const u16 alpha_lookup[] = {
+	0x0000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,
+	0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,
+	0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,
+	0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000,0x8000};
+
+
 //private acceleration tables
 static float float16table[65536];
 static float float10Table[1024];
@@ -107,6 +142,11 @@ static float normalTable[1024];
 #define fix2float(v)    (((float)((s32)(v))) / (float)(1<<12))
 #define fix10_2float(v) (((float)((s32)(v))) / (float)(1<<9))
 
+CACHE_ALIGN u16 gfx3d_convertedScreen[256*192];
+
+//this extra *2 is a HACK to salvage some savestates. remove me when the savestate format changes.
+CACHE_ALIGN u8 gfx3d_convertedAlpha[256*192*2];
+
 // Matrix stack handling
 static CACHE_ALIGN MatrixStack	mtxStack[4] = {
 	MatrixStack(1), // Projection stack
@@ -165,7 +205,7 @@ static u32 lightColor[4] = {0,0,0,0};
 static u32 lightDirection[4] = {0,0,0,0};
 //material state:
 static u16 dsDiffuse, dsAmbient, dsSpecular, dsEmission;
-/* Shininess */
+// Shininess
 static float shininessTable[128] = {0};
 static int shininessInd = 0;
 
@@ -188,10 +228,10 @@ static CACHE_ALIGN float cacheHalfVector[4][4];
 
 
 //-------------poly and vertex lists and such things
-POLYLIST polylists[2];
-POLYLIST* polylist = &polylists[0];
-VERTLIST vertlists[2];
-VERTLIST* vertlist = &vertlists[0];
+POLYLIST* polylists = NULL;
+POLYLIST* polylist = NULL;
+VERTLIST* vertlists = NULL;
+VERTLIST* vertlist = NULL;
 
 int listTwiddle = 1;
 int triStripToggle;
@@ -259,6 +299,8 @@ static void makeTables() {
 
 void gfx3d_init()
 {
+	if(polylists == NULL) { polylists = new POLYLIST[2]; polylist = &polylists[0]; }
+	if(vertlists == NULL) { vertlists = new VERTLIST[2]; vertlist = &vertlists[0]; }
 	makeTables();
 	gfx3d_reset();
 }
@@ -267,6 +309,7 @@ void gfx3d_reset()
 {
 	gfx3d = GFX3D();
 
+	drawPending = FALSE;
 	flushPending = FALSE;
 	listTwiddle = 1;
 	twiddleLists();
@@ -297,6 +340,11 @@ void gfx3d_reset()
 	last_t = 0;
 	last_s = 0;
 	viewport = 0xBFFF0000;
+
+	memset(gfx3d_convertedScreen,0,sizeof(gfx3d_convertedScreen));
+	memset(gfx3d_convertedAlpha,0,sizeof(gfx3d_convertedAlpha));
+
+	gfx3d.clearDepth = gfx3d_extendDepth_15_to_24(0x7FFF);
 	
 	GFX_FIFOclear();
 }
@@ -336,9 +384,8 @@ void gfx3d_glFogOffset (u32 v)
 
 void gfx3d_glClearDepth(u32 v)
 {
-	//formula from http://nocash.emubase.de/gbatek.htm#ds3drearplane
-	v &= 0x7FFFF;
-	gfx3d.clearDepth = (v*0x200)+((v+1)/0x8000)*0x01FF;
+	v &= 0x7FFF;
+	gfx3d.clearDepth = gfx3d_extendDepth_15_to_24(v);
 }
 
 void gfx3d_glMatrixMode(u32 v)
@@ -355,7 +402,7 @@ void gfx3d_glLoadIdentity()
 		MatrixIdentity (mtxCurrent[1]);
 }
 
-BOOL gfx3d_glLoadMatrix4x4(signed long v)
+BOOL gfx3d_glLoadMatrix4x4(s32 v)
 {
 	mtxCurrent[mode][ML4x4ind] = fix2float(v);
 
@@ -369,7 +416,7 @@ BOOL gfx3d_glLoadMatrix4x4(signed long v)
 	return TRUE;
 }
 
-BOOL gfx3d_glLoadMatrix4x3(signed long v)
+BOOL gfx3d_glLoadMatrix4x3(s32 v)
 {
 	mtxCurrent[mode][ML4x3ind] = fix2float(v);
 
@@ -392,10 +439,13 @@ void gfx3d_glStoreMatrix(u32 v)
 	//this command always works on both pos and vector when either pos or pos-vector are the current mtx mode
 	short mymode = (mode==1?2:mode);
 
-	//for the projection matrix, the provided value is supposed to be reset to zero
-	if(mymode==0)
+	//limit height of these stacks.
+	//without the mymode==3 namco classics galaxian will try to use pos=1 and overrun the stack, corrupting emu
+	if(mymode==0 || mymode==3)
 		v = 0;
 
+	if(v==31) v=30; //? what should happen in this case?
+
 	MatrixStackLoadMatrix (&mtxStack[mymode], v&31, mtxCurrent[mymode]);
 	if(mymode==2)
 		MatrixStackLoadMatrix (&mtxStack[1], v&31, mtxCurrent[1]);
@@ -406,10 +456,13 @@ void gfx3d_glRestoreMatrix(u32 v)
 	//this command always works on both pos and vector when either pos or pos-vector are the current mtx mode
 	short mymode = (mode==1?2:mode);
 
-	//for the projection matrix, the provided value is supposed to be reset to zero
-	if(mymode==0)
+	//limit height of these stacks
+	//without the mymode==3 namco classics galaxian will try to use pos=1 and overrun the stack, corrupting emu
+	if(mymode==0 || mymode==3)
 		v = 0;
 
+	if(v==31) v=30; //? what should happen in this case?
+
 	MatrixCopy (mtxCurrent[mymode], MatrixStackGetPos(&mtxStack[mymode], v&31));
 	if (mymode == 2)
 		MatrixCopy (mtxCurrent[1], MatrixStackGetPos(&mtxStack[1], v&31));
@@ -438,7 +491,7 @@ void gfx3d_glPushMatrix()
 	T1WriteLong(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x600, gxstat);
 }
 
-void gfx3d_glPopMatrix(signed long i)
+void gfx3d_glPopMatrix(s32 i)
 {
 	u32 gxstat = T1ReadLong(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x600);
 
@@ -464,7 +517,7 @@ void gfx3d_glPopMatrix(signed long i)
 	T1WriteLong(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x600, gxstat);
 }
 
-BOOL gfx3d_glTranslate(signed long v)
+BOOL gfx3d_glTranslate(s32 v)
 {
 	trans[transind] = fix2float(v);
 
@@ -480,7 +533,7 @@ BOOL gfx3d_glTranslate(signed long v)
 	return TRUE;
 }
 
-BOOL gfx3d_glScale(signed long v)
+BOOL gfx3d_glScale(s32 v)
 {
 	short mymode = (mode==2?1:mode);
 
@@ -501,7 +554,7 @@ BOOL gfx3d_glScale(signed long v)
 	return TRUE;
 }
 
-BOOL gfx3d_glMultMatrix3x3(signed long v)
+BOOL gfx3d_glMultMatrix3x3(s32 v)
 {
 	mtxTemporal[MM3x3ind] = fix2float(v);
 
@@ -525,7 +578,7 @@ BOOL gfx3d_glMultMatrix3x3(signed long v)
 	return TRUE;
 }
 
-BOOL gfx3d_glMultMatrix4x3(signed long v)
+BOOL gfx3d_glMultMatrix4x3(s32 v)
 {
 	mtxTemporal[MM4x3ind] = fix2float(v);
 
@@ -547,7 +600,7 @@ BOOL gfx3d_glMultMatrix4x3(signed long v)
 	return TRUE;
 }
 
-BOOL gfx3d_glMultMatrix4x4(signed long v)
+BOOL gfx3d_glMultMatrix4x4(s32 v)
 {
 	mtxTemporal[MM4x4ind] = fix2float(v);
 
@@ -616,10 +669,10 @@ static void SetVertex()
 	{
 		last_s =((coord[0]*mtxCurrent[3][0] +
 					coord[1]*mtxCurrent[3][4] +
-					coord[2]*mtxCurrent[3][8]) + _s);
+					coord[2]*mtxCurrent[3][8]) + _s * 16.0f) / 16.0f;
 		last_t =((coord[0]*mtxCurrent[3][1] +
 					coord[1]*mtxCurrent[3][5] +
-					coord[2]*mtxCurrent[3][9]) + _t);
+					coord[2]*mtxCurrent[3][9]) + _t * 16.0f) / 16.0f;
 	}
 
 	//refuse to do anything if we have too many verts or polys
@@ -858,13 +911,13 @@ BOOL gfx3d_glShininess (u32 val)
 
 void gfx3d_UpdateToonTable(u8 offset, u16 val)
 {
-	gfx3d.rgbToonTable[offset] =  RGB15TO32(val, 255);
+	gfx3d.u16ToonTable[offset] =  val;
 }
 
 void gfx3d_UpdateToonTable(u8 offset, u32 val)
 {
-	gfx3d.rgbToonTable[offset] =  RGB15TO32(val & 0xFFFF, 255);
-	gfx3d.rgbToonTable[offset+1] =  RGB15TO32(val >> 8, 255);
+	gfx3d.u16ToonTable[offset] = val & 0xFFFF;
+	gfx3d.u16ToonTable[offset+1] = val >> 8;
 }
 
 static void gfx3d_glTexImage_cache()
@@ -909,9 +962,10 @@ void gfx3d_glTexCoord(u32 val)
 void gfx3d_glNormal(u32 v)
 {
 	int i,c;
-	ALIGN(16) float normal[3] = { normalTable[v&1023],
+	ALIGN(16) float normal[4] = { normalTable[v&1023],
 						normalTable[(v>>10)&1023],
-						normalTable[(v>>20)&1023]};
+						normalTable[(v>>20)&1023],
+						1};
 
 	if (texCoordinateTransform == 2)
 	{
@@ -966,12 +1020,25 @@ void gfx3d_glNormal(u32 v)
 
 				if(dsSpecular & 0x8000)
 				{
-					shininessLevel = shininessTable[(int)(shininessLevel * 128)];	
+					int shininessIndex = (int)(shininessLevel * 128);
+					if(shininessIndex >= (int)ARRAY_SIZE(shininessTable)) {
+						//we can't print this right now, because when a game triggers this it triggers it _A_LOT_
+						//so wait until we have per-frame diagnostics.
+						//this was tested using Princess Debut (US) after proceeding through the intro and getting the tiara.
+						//After much research, I determined that this was caused by the game feeding in a totally jacked matrix
+						//to mult4x4 from 0x02129B80 (after feeding two other valid matrices)
+						//the game seems to internally index these as: ?, 0x37, 0x2B <-- error
+						//but, man... this is seriously messed up. there must be something going wrong.
+						//maybe it has something to do with what looks like a mirror room effect that is going on during this time?
+						//PROGINFO("ERROR: shininess table out of bounds.\n  maybe an emulator error; maybe a non-unit normal; setting to 0\n");
+						shininessIndex = 0;
+					}
+					shininessLevel = shininessTable[shininessIndex];
 				}
 
 				for(c = 0; c < 3; c++)
 				{
-					vertexColor[c] += (((specular[c] * _lightColor[c] * shininessLevel)
+                                  vertexColor[c] += (int)(((specular[c] * _lightColor[c] * shininessLevel)
 						+ (diffuse[c] * _lightColor[c] * diffuseLevel)
 						+ (ambient[c] * _lightColor[c])) / 31.0f);
 				}
@@ -984,20 +1051,20 @@ void gfx3d_glNormal(u32 v)
 }
 
 
-signed long gfx3d_GetClipMatrix (unsigned int index)
+s32 gfx3d_GetClipMatrix (unsigned int index)
 {
 	float val = MatrixGetMultipliedIndex (index, mtxCurrent[0], mtxCurrent[1]);
 
 	val *= (1<<12);
 
-	return (signed long)val;
+	return (s32)val;
 }
 
-signed long gfx3d_GetDirectionalMatrix (unsigned int index)
+s32 gfx3d_GetDirectionalMatrix (unsigned int index)
 {
 	int _index = (((index / 3) * 4) + (index % 3));
 
-	return (signed long)(mtxCurrent[2][_index]*(1<<12));
+	return (s32)(mtxCurrent[2][_index]*(1<<12));
 }
 
 static void gfx3d_glLightDirection_cache(int index)
@@ -1049,6 +1116,7 @@ void gfx3d_glAlphaFunc(u32 v)
 BOOL gfx3d_glBoxTest(u32 v)
 {
 	u32 gxstat = T1ReadLong(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x600);
+	gxstat &= 0xFFFFFFFD;		// clear boxtest bit
 	gxstat |= 0x00000001;		// busy
 	T1WriteLong(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x600, gxstat);
 
@@ -1137,7 +1205,8 @@ faceInside:
 	gxstat |= 0x2;
 
 noFaceInside:*/
-	gxstat &= 0xFFFFFFFE;
+	gxstat &= 0xFFFFFFFE;		// clear busy bit
+	gxstat |= 0x00000002;		// hack
 	T1WriteLong(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x600, gxstat);
 	return TRUE;
 }
@@ -1321,7 +1390,7 @@ static void gfx3d_FlushFIFO()
 #ifdef USE_GEOMETRY_FIFO_EMULATION
 	for (int i=0; i < gxFIFO.tail; i++)
 	{
-		//INFO("3D execute command 0x%02X with param 0x%08X (pos %03i)\n", gxFIFO.cmd[i], gxFIFO.param[i], i);
+		//INFO("3D execute command 0x%02X with param 0x%08X (pos %03i)\n", gxFIFO.cmd[i], gxFIFO.param[i], i);	
 		gfx3d_execute(gxFIFO.cmd[i], gxFIFO.param[i]);
 	}
 #endif
@@ -1330,15 +1399,39 @@ static void gfx3d_FlushFIFO()
 
 void gfx3d_glFlush(u32 v)
 {
-	gfx3d.frameCtr++;
-
-	gfx3d_FlushFIFO();
-
-	//assert(!flushPending);
 	flushPending = TRUE;
 	gfx3d.sortmode = BIT0(v);
 	gfx3d.wbuffer = BIT1(v);
 
+	//see discussion at top of file
+	if(CommonSettings.gfx3d_flushMode == 0)
+		gfx3d_doFlush();
+}
+
+static bool gfx3d_ysort_compare(int num1, int num2)
+{
+	const POLY &poly1 = polylist->list[num1];
+	const POLY &poly2 = polylist->list[num2];
+
+	//this may be verified by checking the game create menus in harvest moon island of happiness
+	if (poly1.maxy > poly2.maxy) return false;
+	if (poly1.maxy < poly2.maxy) return true;
+	if (poly1.miny < poly2.miny) return true;
+	if (poly1.miny > poly2.miny) return false;
+	
+	//make sure we respect the game's ordering in cases of complete ties
+	//this makes it a stable sort.
+	//this must be a stable sort or else advance wars DOR will flicker in the main map mode
+	if (num1 < num2) return true;
+	else return false;
+}
+
+static void gfx3d_doFlush()
+{
+	gfx3d.frameCtr++;
+
+	gfx3d_FlushFIFO();
+
 	// reset
 	clInd = 0;
 	clCmd = 0;
@@ -1347,15 +1440,33 @@ void gfx3d_glFlush(u32 v)
 	gfx3d.polylist = polylist;
 	gfx3d.vertlist = vertlist;
 
+	int polycount = polylist->count;
+
+	//find the min and max y values for each poly.
+	//TODO - this could be a small waste of time if we are manual sorting the translucent polys
+	for(int i=0; i<polycount; i++)
+	{
+		POLY &poly = polylist->list[i];
+		float verty = 0.0f; 
+		poly.miny = poly.maxy = vertlist->list[poly.vertIndexes[0]].y;
+
+		for(int j=1; j<poly.type; j++)
+		{
+			verty = vertlist->list[poly.vertIndexes[j]].y;
+			poly.miny = min(poly.miny, verty);
+			poly.maxy = max(poly.maxy, verty);
+		}
+	}
+
 	//we need to sort the poly list with alpha polys last
 	//first, look for opaque polys
-	int polycount = polylist->count;
 	int ctr=0;
 	for(int i=0;i<polycount;i++) {
 		POLY &poly = polylist->list[i];
 		if(!poly.isTranslucent())
 			gfx3d.indexlist[ctr++] = i;
 	}
+	int opaqueCount = ctr;
 	//then look for translucent polys
 	for(int i=0;i<polycount;i++) {
 		POLY &poly = polylist->list[i];
@@ -1363,8 +1474,23 @@ void gfx3d_glFlush(u32 v)
 			gfx3d.indexlist[ctr++] = i;
 	}
 
+	//now we have to sort the opaque polys by y-value.
+	//(test case: harvest moon island of happiness character cretor UI)
+	//should this be done after clipping??
+	std::sort(gfx3d.indexlist, gfx3d.indexlist + opaqueCount, gfx3d_ysort_compare);
+	
+	if(!gfx3d.sortmode)
+	{
+		//if we are autosorting translucent polys, we need to do this also
+		//TODO - this is unverified behavior. need a test case
+		std::sort(gfx3d.indexlist + opaqueCount, gfx3d.indexlist + polycount, gfx3d_ysort_compare);
+	}
+
 	//switch to the new lists
 	twiddleLists();
+
+	flushPending = FALSE;
+	drawPending = TRUE;
 }
 
 void gfx3d_VBlankSignal()
@@ -1377,8 +1503,9 @@ void gfx3d_VBlankSignal()
 		return;
 	}
 
-	flushPending = FALSE;
-	drawPending = TRUE;
+	//see discussion at top of file
+	if(CommonSettings.gfx3d_flushMode == 1)
+		gfx3d_doFlush();
 }
 
 void gfx3d_VBlankEndSignal(bool skipFrame)
@@ -1389,7 +1516,16 @@ void gfx3d_VBlankEndSignal(bool skipFrame)
 	if(!drawPending) return;
 
 	drawPending = FALSE;
-	gpu3D->NDS_3D_Render();
+
+	if(CommonSettings.showGpu.main)
+		gpu3D->NDS_3D_Render();
+
+	//if the null 3d core is chosen, then we need to clear out the 3d buffers to keep old data from being rendered
+	if(gpu3D == &gpu3DNull || !CommonSettings.showGpu.main)
+	{
+		memset(gfx3d_convertedScreen,0,sizeof(gfx3d_convertedScreen));
+		memset(gfx3d_convertedScreen,0,sizeof(gfx3d_convertedAlpha));
+	}
 }
 
 #ifdef USE_GEOMETRY_FIFO_EMULATION
@@ -2084,23 +2220,15 @@ static void gfx3d_Control_cache()
 {
 	u32 v = control;
 
-	if(v&1) gfx3d.enableTexturing = TRUE;
-	else gfx3d.enableTexturing = FALSE;
-
-	if((v>>1)&1) gfx3d.shading = GFX3D::HIGHLIGHT;
+	if(BIT1(v)) gfx3d.shading = GFX3D::HIGHLIGHT;
 	else gfx3d.shading = GFX3D::TOON;
 
-	if((v>>2)&1) gfx3d.enableAlphaTest = TRUE;
-	else gfx3d.enableAlphaTest = FALSE;
-
-	if((v>>3)&1) gfx3d.enableAlphaBlending = TRUE;
-	else gfx3d.enableAlphaBlending = FALSE;
-
-	if((v>>4)&1) gfx3d.enableAntialiasing = TRUE;
-	else gfx3d.enableAntialiasing = FALSE;
-
-	if((v>>5)&1) gfx3d.enableEdgeMarking = TRUE;
-	else gfx3d.enableEdgeMarking = FALSE;
+	gfx3d.enableTexturing = BIT0(v);
+	gfx3d.enableAlphaTest = BIT2(v);
+	gfx3d.enableAlphaBlending = BIT3(v);
+	gfx3d.enableAntialiasing = BIT4(v);
+	gfx3d.enableEdgeMarking = BIT5(v);
+	gfx3d.enableClearImage = BIT14(v);
 
 	//other junk
 	if (v&(1<<14))
@@ -2139,6 +2267,15 @@ void gfx3d_glGetLightColor(unsigned int index, unsigned int* dest)
 	*dest = lightColor[index];
 }
 
+void gfx3d_GetLineData(int line, u16** dst, u8** dstAlpha)
+{
+	*dst = gfx3d_convertedScreen+((line)<<8);
+	if(dstAlpha != NULL)
+	{
+		*dstAlpha = gfx3d_convertedAlpha+((line)<<8);
+	}
+}
+
 
 //http://www.opengl.org/documentation/specs/version1.1/glspec1.1/node17.html
 //talks about the state required to process verts in quadlists etc. helpful ideas.
@@ -2154,14 +2291,6 @@ SFORMAT SF_GFX3D[]={
 	{ "GMOD", 4, 1, &mode},
 	{ "GMTM", 4,16, mtxTemporal},
 	{ "GMCU", 4,64, mtxCurrent},
-	{ "GM0P", 4, 1, &mtxStack[0].position},
-	{ "GM0M", 4,16, mtxStack[0].matrix},
-	{ "GM1P", 4, 1, &mtxStack[1].position},
-	{ "GM1M", 4,496,mtxStack[1].matrix},
-	{ "GM2P", 4, 1, &mtxStack[2].position},
-	{ "GM2M", 4,496,mtxStack[2].matrix},
-	{ "GM3P", 4, 1, &mtxStack[3].position},
-	{ "GM3M", 4,16, mtxStack[3].matrix},
 	{ "ML4I", 1, 1, &ML4x4ind},
 	{ "ML3I", 1, 1, &ML4x3ind},
 	{ "MM4I", 1, 1, &MM4x4ind},
@@ -2187,8 +2316,8 @@ SFORMAT SF_GFX3D[]={
 	{ "GLPT", 4, 1, &PTind},
 	{ "GLPC", 4, 4, PTcoords},
 	{ "GLF9", 4, 1, &gxFIFO.tail},
-	{ "GLF9", 1, 261, &gxFIFO.cmd},
-	{ "GLF9", 4, 261, &gxFIFO.param},
+	{ "GLF9", 1, 261, &gxFIFO.cmd[0]},
+	{ "GLF9", 4, 261, &gxFIFO.param[0]},
 	{ "GCOL", 1, 4, colorRGB},
 	{ "GLCO", 4, 4, lightColor},
 	{ "GLDI", 4, 4, lightDirection},
@@ -2196,11 +2325,6 @@ SFORMAT SF_GFX3D[]={
 	{ "GMAM", 2, 1, &dsAmbient},
 	{ "GMSP", 2, 1, &dsSpecular},
 	{ "GMEM", 2, 1, &dsEmission},
-	{ "GTST", 4, 1, &triStripToggle},
-	{ "GTVC", 4, 1, &tempVertInfo.count},
-	{ "GTVM", 4, 4, &tempVertInfo.map[0]},
-	{ "GTVF", 4, 1, &tempVertInfo.first},
-	{ "GLTW", 4, 1, &listTwiddle},
 	{ "GFLP", 4, 1, &flushPending},
 	{ "GDRP", 4, 1, &drawPending},
 	{ "GSET", 4, 1, &gfx3d.enableTexturing},
@@ -2217,21 +2341,48 @@ SFORMAT SF_GFX3D[]={
 	{ "GSCD", 4, 1, &gfx3d.clearDepth},
 	{ "GSFC", 4, 4, gfx3d.fogColor},
 	{ "GSFO", 4, 1, &gfx3d.fogOffset},
-	{ "GSTT", 4, 32, gfx3d.rgbToonTable},
+	{ "GST2", 2, 32, gfx3d.u16ToonTable},
 	{ "GSST", 4, 128, shininessTable},
 	{ "GSSI", 4, 1, &shininessInd},
+	//------------------------
+	{ "GTST", 4, 1, &triStripToggle},
+	{ "GTVC", 4, 1, &tempVertInfo.count},
+	{ "GTVM", 4, 4, tempVertInfo.map},
+	{ "GTVF", 4, 1, &tempVertInfo.first},
+	{ "G3CS", 2, 256*192, gfx3d_convertedScreen},
+	{ "G3CA", 2, 256*192, gfx3d_convertedAlpha}, 
 	{ 0 }
 };
 
 //-------------savestate
 void gfx3d_savestate(std::ostream* os)
 {
+	//version
+	write32le(2,os);
+
 	//dump the render lists
-	//TODO!!!!
+	OSWRITE(vertlist->count);
+	for(int i=0;i<vertlist->count;i++)
+		vertlist->list[i].save(os);
+	OSWRITE(polylist->count);
+	for(int i=0;i<polylist->count;i++)
+		polylist->list[i].save(os);
+
+	for(int i=0;i<4;i++)
+	{
+		OSWRITE(mtxStack[i].position);
+		for(int j=0;j<mtxStack[i].size*16+16;j++)
+			OSWRITE(mtxStack[i].matrix[j]);
+	}
 }
 
-bool gfx3d_loadstate(std::istream* is)
+bool gfx3d_loadstate(std::istream* is, int size)
 {
+	int version;
+	if(read32le(&version,is) != 1) return false;
+	if(size==8) version = 0;
+
+
 	gfx3d_glPolygonAttrib_cache();
 	gfx3d_glTexImage_cache();
 	gfx3d_Control_cache();
@@ -2241,10 +2392,30 @@ bool gfx3d_loadstate(std::istream* is)
 	gfx3d_glLightDirection_cache(3);
 
 	//jiggle the lists. and also wipe them. this is clearly not the best thing to be doing.
+	listTwiddle = 0;
 	polylist = &polylists[listTwiddle];
 	vertlist = &vertlists[listTwiddle];
-	polylist->count = 0;
-	vertlist->count = 0;
+
+	if(version>=1)
+	{
+		OSREAD(vertlist->count);
+		for(int i=0;i<vertlist->count;i++)
+			vertlist->list[i].load(is);
+		OSREAD(polylist->count);
+		for(int i=0;i<polylist->count;i++)
+			polylist->list[i].load(is);
+	}
+
+	if(version>=2)
+	{
+		for(int i=0;i<4;i++)
+		{
+			OSREAD(mtxStack[i].position);
+			for(int j=0;j<mtxStack[i].size*16+16;j++)
+				OSREAD(mtxStack[i].matrix[j]);
+		}
+	}
+
 	gfx3d.polylist = &polylists[listTwiddle^1];
 	gfx3d.vertlist = &vertlists[listTwiddle^1];
 	gfx3d.polylist->count=0;
