@@ -31,8 +31,6 @@ static BOOL regMainLoop = FALSE;
 static BOOL noticed_3D=FALSE;
 volatile BOOL execute = FALSE;
 BOOL click = FALSE;
-int savetype=MC_TYPE_AUTODETECT;
-u32 savesize=1;
 
 void desmume_mem_init();
 
@@ -65,12 +63,16 @@ int desmume_open(const char *filename)
   int i;
   noticed_3D=FALSE;
   clear_savestates();
-  i = NDS_LoadROM(filename, savetype, savesize, NULL);
+#ifdef EXPERIMENTAL_GBASLOT
+  i = NDS_LoadROM(filename);
+#else
+  i = NDS_LoadROM(filename, NULL);
+#endif
   return i;
 }
 
 void desmume_savetype(int type) {
-	mmu_select_savetype(type, &savetype, &savesize);
+	backup_setManualBackupType(type);
 }
 
 
@@ -116,9 +118,8 @@ INLINE void desmume_cycle()
   /* Update keypad value */
   update_keypad(keypad);
 
-  desmume_last_cycle = NDS_exec<false>((560190 << 1) - desmume_last_cycle);
+  NDS_exec<false>();
   SPU_Emulate_user();
-  SPU_Emulate_core();
 }
 
 
@@ -131,22 +132,11 @@ static void Draw()
 
 gboolean EmuLoop(gpointer data)
 {
-	/*
-	int i;
-	if (!noticed_3D) {
-		GtkWidget * dlg = glade_xml_get_widget(xml, "w3Dop");
-		gtk_widget_show(dlg);
-		noticed_3D=TRUE;
-	}*/
-		
 	if(desmume_running())	/* Si on est en train d'executer le programme ... */
 	{
-	 // static int limiter_frame_counter = 0;	
-		if(Frameskip == 0)
-			NDS_SkipFrame(false);
-		else
-			NDS_SkipFrame(((fsFrameCount % (Frameskip+1)) == 0) ? false : true);
-		
+		if(Frameskip != 0 && (fsFrameCount % (Frameskip+1)) != 0)
+			NDS_SkipNextFrame();	
+	
 		fsFrameCount++;
 	  
 		fps_FrameCount++;
@@ -182,16 +172,6 @@ gboolean EmuLoop(gpointer data)
 		
 		ticksPrevFrame = SDL_GetTicks();
 
-             /*   if ( !glade_fps_limiter_disabled) {
-                  limiter_frame_counter += 1;
-                  if ( limiter_frame_counter >= FPS_LIMITER_FRAME_PERIOD) {
-                    limiter_frame_counter = 0;
-
-                    /* wait for the timer to expire *-/
-                    SDL_SemWait( glade_fps_limiter_semaphore);
-                  }
-                }*/
-
 		return TRUE;
 	}
 	gtk_widget_queue_draw(pDrawingArea);
