@@ -22,17 +22,19 @@
 #ifndef __DESMUME_H__
 #define __DESMUME_H__
 
-#include "globals.h"
+extern void desmume_init( struct armcpu_memory_iface *arm9_mem_if,
+                          struct armcpu_ctrl_iface **arm9_ctrl_iface,
+                          struct armcpu_memory_iface *arm7_mem_if,
+                          struct armcpu_ctrl_iface **arm7_ctrl_iface,
+                          int disable_sound);
+extern void desmume_free( void);
 
-extern void desmume_init();
-extern void desmume_free();
+extern void desmume_pause( void);
+extern void desmume_resume( void);
+extern void desmume_toggle( void);
+extern BOOL desmume_running( void);
 
-extern void desmume_pause();
-extern void desmume_resume();
-extern void desmume_toggle();
-extern BOOL desmume_running();
-
-extern void desmume_cycle();
+extern void desmume_cycle( void);
 
 #endif /*__DESMUME_H__*/
 
