@@ -22,6 +22,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <math.h>
+#include <assert.h>
 #include "matrix.h"
 
 extern "C" {
@@ -33,7 +34,7 @@ void MatrixInit  (float *matrix)
 	matrix[0] = matrix[5] = matrix[10] = matrix[15] = 1.f;
 }
 
-#ifndef SSE2
+#ifdef NOSSE2
 void MATRIXFASTCALL MatrixMultVec4x4 (const float *matrix, float *vecPtr)
 {
 	float x = vecPtr[0];
@@ -194,7 +195,7 @@ void MatrixStackSetStackPosition (MatrixStack *stack, int pos)
 	if (stack->position < 0)
 		stack->position = 0;
 	else if (stack->position > stack->size)	
-		stack->position = stack->size;
+		stack->position = stack->size+1;
 }
 
 void MatrixStackPushMatrix (MatrixStack *stack, const float *ptr)
@@ -213,6 +214,7 @@ float * MatrixStackPopMatrix (MatrixStack *stack, int size)
 
 float * MatrixStackGetPos (MatrixStack *stack, int pos)
 {
+	assert(pos<31);
 	return &stack->matrix[pos*16];
 }
 
@@ -223,6 +225,7 @@ float * MatrixStackGet (MatrixStack *stack)
 
 void MatrixStackLoadMatrix (MatrixStack *stack, int pos, const float *ptr)
 {
+	assert(pos<31);
 	MatrixCopy (&stack->matrix[pos*16], ptr);
 }
 
