@@ -1,6 +1,6 @@
 /* movie.cpp
  *
- * Copyright (C) 2006-2008 Zeromus
+ * Copyright (C) 2006-2009 DeSmuME team
  *
  * This file is part of DeSmuME
  *
@@ -22,14 +22,21 @@
 #include <assert.h>
 #include <limits.h>
 #include <fstream>
-
 #include "utils/guid.h"
 #include "utils/xstring.h"
 #include "movie.h"
 #include "NDSSystem.h"
+#include "readwrite.h"
 #include "debug.h"
+#include "rtc.h"
+#include "common.h"
+#include "mic.h"
+#include "version.h"
+#include "memorystream.h"
 
 using namespace std;
+bool freshMovie = false;	  //True when a movie loads, false when movie is altered.  Used to determine if a movie has been altered since opening
+bool autoMovieBackup = true;
 
 #define FCEU_PrintError LOG
 
@@ -50,7 +57,9 @@ bool movie_readonly = true;
 char curMovieFilename[512] = {0};
 MovieData currMovieData;
 int currRerecordCount;
-
+bool ShowInputDisplay = false;
+bool movie_reset_command = false;
+bool movie_lid = false;
 //--------------
 
 
@@ -159,11 +168,10 @@ void MovieRecord::dump(MovieData* md, std::ostream* os, int index)
 MovieData::MovieData()
 	: version(MOVIE_VERSION)
 	, emuVersion(DESMUME_VERSION_NUMERIC)
+	, romChecksum(0)
 	, rerecordCount(1)
 	, binaryFlag(false)
-	//, greenZoneCount(0)
 {
-	memset(&romChecksum,0,sizeof(MD5DATA));
 }
 
 void MovieData::truncateAt(int frame)
@@ -185,10 +193,14 @@ void MovieData::installValue(std::string& key, std::string& val)
 		romFilename = val;
 	else if(key == "romChecksum")
 		StringToBytes(val,&romChecksum,MD5DATA::size);
+	else if(key == "romSerial")
+		romSerial = val;
 	else if(key == "guid")
 		guid = Desmume_Guid::fromString(val);
 	else if(key == "comment")
 		comments.push_back(mbstowcs(val));
+	else if(key == "binary")
+		installBool(val,binaryFlag);
 	else if(key == "savestate")
 	{
 		int len = Base64StringToBytesLength(val);
@@ -199,6 +211,16 @@ void MovieData::installValue(std::string& key, std::string& val)
 			StringToBytes(val,&savestate[0],len); // decodes either base64 or hex
 		}
 	}
+	else if(key == "sram")
+	{
+		int len = Base64StringToBytesLength(val);
+		if(len == -1) len = HexStringToBytesLength(val); // wasn't base64, try hex
+		if(len >= 1)
+		{
+			sram.resize(len);
+			StringToBytes(val,&sram[0],len); // decodes either base64 or hex
+		}
+	}
 }
 
 
@@ -209,8 +231,13 @@ int MovieData::dump(std::ostream *os, bool binary)
 	*os << "emuVersion " << emuVersion << endl;
 	*os << "rerecordCount " << rerecordCount << endl;
 	*os << "romFilename " << romFilename << endl;
-	*os << "romChecksum " << BytesToString(romChecksum.data,MD5DATA::size) << endl;
+	*os << "romChecksum " << u32ToHexString(gameInfo.crc) << endl;
+	*os << "romSerial " << romSerial << endl;
 	*os << "guid " << guid.toString() << endl;
+	*os << "useExtBios " << CommonSettings.UseExtBIOS << endl;
+
+	if(CommonSettings.UseExtBIOS)
+		*os << "swiFromBios " << CommonSettings.SWIFromBIOS << endl;
 
 	for(uint32 i=0;i<comments.size();i++)
 		*os << "comment " << wcstombs(comments[i]) << endl;
@@ -220,12 +247,14 @@ int MovieData::dump(std::ostream *os, bool binary)
 		
 	if(savestate.size() != 0)
 		*os << "savestate " << BytesToString(&savestate[0],savestate.size()) << endl;
+	if(sram.size() != 0)
+		*os << "sram " << BytesToString(&sram[0],sram.size()) << endl;
 	if(binary)
 	{
-		////put one | to start the binary dump
-		//os->put('|');
-		//for(int i=0;i<(int)records.size();i++)
-		//	records[i].dumpBinary(this,os,i);
+		//put one | to start the binary dump
+		os->put('|');
+		for(int i=0;i<(int)records.size();i++)
+			records[i].dumpBinary(this,os,i);
 	}
 	else
 		for(int i=0;i<(int)records.size();i++)
@@ -236,7 +265,7 @@ int MovieData::dump(std::ostream *os, bool binary)
 }
 
 //yuck... another custom text parser.
-static bool LoadFM2(MovieData& movieData, std::istream* fp, int size, bool stopAfterHeader)
+bool LoadFM2(MovieData& movieData, std::istream* fp, int size, bool stopAfterHeader)
 {
 	//TODO - start with something different. like 'desmume movie version 1"
 	std::ios::pos_type curr = fp->tellg();
@@ -268,8 +297,8 @@ static bool LoadFM2(MovieData& movieData, std::istream* fp, int size, bool stopA
 		isnewline = (c==10||c==13);
 		if(isrecchar && movieData.binaryFlag && !stopAfterHeader)
 		{
-			//LoadFM2_binarychunk(movieData, fp, size);
-			return false;
+			LoadFM2_binarychunk(movieData, fp, size);
+			return true;
 		}
 		switch(state)
 		{
@@ -349,7 +378,7 @@ static void closeRecordingMovie()
 /// Stop movie playback.
 static void StopPlayback()
 {
-	//FCEU_DispMessageOnMovie("Movie playback stopped.");
+	driver->USR_InfoMessage("Movie playback stopped.");
 	movieMode = MOVIEMODE_INACTIVE;
 }
 
@@ -357,8 +386,7 @@ static void StopPlayback()
 /// Stop movie recording
 static void StopRecording()
 {
-	//FCEU_DispMessage("Movie recording stopped.");
-
+	driver->USR_InfoMessage("Movie recording stopped.");
 	movieMode = MOVIEMODE_INACTIVE;
 	
 	closeRecordingMovie();
@@ -366,22 +394,20 @@ static void StopRecording()
 
 
 
-static void FCEUI_StopMovie()
+void FCEUI_StopMovie()
 {
-	//if(suppressMovieStop)
-	//	return;
-	
 	if(movieMode == MOVIEMODE_PLAY)
 		StopPlayback();
 	else if(movieMode == MOVIEMODE_RECORD)
 		StopRecording();
 
 	curMovieFilename[0] = 0;
+	freshMovie = false;
 }
 
 
 //begin playing an existing movie
-static void FCEUI_LoadMovie(const char *fname, bool _read_only, bool tasedit, int _pauseframe)
+void _CDECL_ FCEUI_LoadMovie(const char *fname, bool _read_only, bool tasedit, int _pauseframe)
 {
 	//if(!tasedit && !FCEU_IsValidUI(FCEUI_PLAYMOVIE))
 	//	return;
@@ -416,23 +442,38 @@ static void FCEUI_LoadMovie(const char *fname, bool _read_only, bool tasedit, in
 	//fully reload the game to reinitialize everything before playing any movie
 	//poweron(true);
 
+	NDS_Reset();
+
 	////WE NEED TO LOAD A SAVESTATE
 	//if(currMovieData.savestate.size() != 0)
 	//{
 	//	bool success = MovieData::loadSavestateFrom(&currMovieData.savestate);
 	//	if(!success) return;
 	//}
+	lagframecounter=0;
+	LagFrameFlag=0;
+	lastLag=0;
+	TotalLagFrames=0;
 
 	currFrameCounter = 0;
 	pauseframe = _pauseframe;
 	movie_readonly = _read_only;
 	movieMode = MOVIEMODE_PLAY;
 	currRerecordCount = currMovieData.rerecordCount;
+	InitMovieTime();
+	MMU_new.backupDevice.movie_mode();
+	if(currMovieData.sram.size() != 0)
+	{
+		bool success = MovieData::loadSramFrom(&currMovieData.sram);
+		if(!success) return;
+	}
+	freshMovie = true;
+	ClearAutoHold();
 
-	//if(movie_readonly)
-	//	FCEU_DispMessage("Replay started Read-Only.");
-	//else
-	//	FCEU_DispMessage("Replay started Read+Write.");
+	if(movie_readonly)
+		driver->USR_InfoMessage("Replay started Read-Only.");
+	else
+		driver->USR_InfoMessage("Replay started Read+Write.");
 }
 
 static void openRecordingMovie(const char* fname)
@@ -444,10 +485,55 @@ static void openRecordingMovie(const char* fname)
 	strcpy(curMovieFilename, fname);
 }
 
+bool MovieData::loadSramFrom(std::vector<char>* buf)
+{
+	memorystream ms(buf);
+	MMU_new.backupDevice.load_movie(&ms);
+	return true;
+}
+
+static bool FCEUSS_SaveSRAM(std::ostream* outstream, std:: string fname)
+{
+	//a temp memory stream. we'll dump some data here and then compress
+	//TODO - support dumping directly without compressing to save a buffer copy
+
+	memorystream ms;
+	std::ostream* os = (std::ostream*)&ms;
+
+	//size it
+	FILE * fp = fopen( fname.c_str(), "r" );
+	if(!fp)
+		return 0;
+	
+	fseek( fp, 0, SEEK_END );
+	int size = ftell(fp);
+	fclose(fp);
+
+	filebuf fb;
+	fb.open (fname.c_str(), ios::in | ios::binary);//ios::in
+	istream is(&fb);
+
+	char *buffer = new char[size];
+
+	is.read(buffer, size);
+
+	outstream->write((char*)buffer,size);
+
+	fb.close();
+
+	return true;
+}
+
+void MovieData::dumpSramTo(std::vector<char>* buf, std::string sramfname) {
+
+	memorystream ms(buf);
+	FCEUSS_SaveSRAM(&ms, sramfname);
+	ms.trim();
+}
 
 //begin recording a new movie
 //TODO - BUG - the record-from-another-savestate doesnt work.
-static void FCEUI_SaveMovie(const char *fname, std::wstring author)
+void _CDECL_ FCEUI_SaveMovie(const char *fname, std::wstring author, int flag, std::string sramfname)
 {
 	//if(!FCEU_IsValidUI(FCEUI_RECORDMOVIE))
 	//	return;
@@ -465,96 +551,156 @@ static void FCEUI_SaveMovie(const char *fname, std::wstring author)
 	currMovieData.guid.newGuid();
 
 	if(author != L"") currMovieData.comments.push_back(L"author " + author);
-	//currMovieData.romChecksum = GameInfo->MD5;
-	//currMovieData.romFilename = FileBase;
+	currMovieData.romChecksum = gameInfo.crc;
+	currMovieData.romSerial = gameInfo.ROMserial;
+	currMovieData.romFilename = GetRomName();
+	
+	NDS_Reset();
 
 	//todo ?
 	//poweron(true);
 	//else
 	//	MovieData::dumpSavestateTo(&currMovieData.savestate,Z_BEST_COMPRESSION);
 
+	if(flag == 1)
+		MovieData::dumpSramTo(&currMovieData.sram, sramfname);
+
 	//we are going to go ahead and dump the header. from now on we will only be appending frames
 	currMovieData.dump(osRecordingMovie, false);
 
+	currFrameCounter=0;
+	lagframecounter=0;
+	LagFrameFlag=0;
+	lastLag=0;
+	TotalLagFrames=0;
+
 	movieMode = MOVIEMODE_RECORD;
 	movie_readonly = false;
 	currRerecordCount = 0;
-	
-	//FCEU_DispMessage("Movie recording started.");
-}
+	InitMovieTime();
+	MMU_new.backupDevice.movie_mode();
 
-
-//the main interaction point between the emulator and the movie system.
-//either dumps the current joystick state or loads one state from the movie
-static void FCEUMOV_AddInputState()
-{
-	//todo - for tasedit, either dump or load depending on whether input recording is enabled
-	//or something like that
-	//(input recording is just like standard read+write movie recording with input taken from gamepad)
-	//otherwise, it will come from the tasedit data.
-
-	if(movieMode == MOVIEMODE_PLAY)
+	if(currMovieData.sram.size() != 0)
 	{
-		//stop when we run out of frames
-		if(currFrameCounter == (int)currMovieData.records.size())
-		{
-			StopPlayback();
-		}
-		else
-		{
-			MovieRecord* mr = &currMovieData.records[currFrameCounter];
-			
-			//reset if necessary
-			if(mr->command_reset())
-			{}
-				//ResetNES();
-
-			NDS_setPadFromMovie(mr->pad);
-		}
-
-		//if we are on the last frame, then pause the emulator if the player requested it
-		if(currFrameCounter == (int)currMovieData.records.size()-1)
-		{
-			/*if(FCEUD_PauseAfterPlayback())
-			{
-				FCEUI_ToggleEmulationPause();
-			}*/
-		}
-
-		//pause the movie at a specified frame 
-		//if(FCEUMOV_ShouldPause() && FCEUI_EmulationPaused()==0)
-		//{
-		//	FCEUI_ToggleEmulationPause();
-		//	FCEU_DispMessage("Paused at specified movie frame");
-		//}
-		
-	}
-	else if(movieMode == MOVIEMODE_RECORD)
-	{
-		MovieRecord mr;
-
-		mr.commands = 0;
-		mr.pad = nds.pad;
-		if(nds.isTouch) {
-			mr.touch.x = nds.touchX;
-			mr.touch.y = nds.touchY;
-			mr.touch.touch = 1;
-		} else {
-			mr.touch.x = 0;
-			mr.touch.y = 0;
-			mr.touch.touch = 0;
-		}
-
-		mr.dump(&currMovieData, osRecordingMovie,currMovieData.records.size());
-		currMovieData.records.push_back(mr);
+		bool success = MovieData::loadSramFrom(&currMovieData.sram);
+		if(!success) return;
 	}
 
-	currFrameCounter++;
-
-	/*extern uint8 joy[4];
-	memcpy(&cur_input_display,joy,4);*/
+	driver->USR_InfoMessage("Movie recording started.");
 }
 
+ void NDS_setTouchFromMovie(void) {
+
+	 if(movieMode == MOVIEMODE_PLAY)
+	 {
+
+		 MovieRecord* mr = &currMovieData.records[currFrameCounter];
+		 nds.touchX=mr->touch.x << 4;
+		 nds.touchY=mr->touch.y << 4;
+
+		 if(mr->touch.touch) {
+			 nds.isTouch=mr->touch.touch;
+			 MMU.ARM7_REG[0x136] &= 0xBF;
+		 }
+		 else {
+			 nds.touchX=0;
+			 nds.touchY=0;
+			 nds.isTouch=0;
+
+			 MMU.ARM7_REG[0x136] |= 0x40;
+		 }
+		 //osd->addFixed(mr->touch.x, mr->touch.y, "%s", "X");
+	 }
+ }
+
+ //the main interaction point between the emulator and the movie system.
+ //either dumps the current joystick state or loads one state from the movie
+ void FCEUMOV_AddInputState()
+ {
+	 if(movieMode == MOVIEMODE_PLAY)
+	 {
+		 //stop when we run out of frames
+		 if(currFrameCounter == (int)currMovieData.records.size())
+		 {
+			 StopPlayback();
+		 }
+		 else
+		 {
+			 MovieRecord* mr = &currMovieData.records[currFrameCounter];
+
+			 if(mr->command_microphone()) MicButtonPressed=1;
+			 else MicButtonPressed=0;
+
+			 if(mr->command_reset()) NDS_Reset();
+
+			 if(mr->command_lid()) movie_lid = true;
+			 else movie_lid = false;
+
+			 NDS_setPadFromMovie(mr->pad);
+			 NDS_setTouchFromMovie();
+		 }
+
+		 //if we are on the last frame, then pause the emulator if the player requested it
+		 if(currFrameCounter == (int)currMovieData.records.size()-1)
+		 {
+			 /*if(FCEUD_PauseAfterPlayback())
+			 {
+			 FCEUI_ToggleEmulationPause();
+			 }*/
+		 }
+
+		 //pause the movie at a specified frame 
+		 //if(FCEUMOV_ShouldPause() && FCEUI_EmulationPaused()==0)
+		 //{
+		 //	FCEUI_ToggleEmulationPause();
+		 //	FCEU_DispMessage("Paused at specified movie frame");
+		 //}
+		 osd->addFixed(180, 176, "%s", "Playback");
+
+	 }
+	 else if(movieMode == MOVIEMODE_RECORD)
+	 {
+		 MovieRecord mr;
+
+		 mr.commands = 0;
+
+		 if(MicButtonPressed == 1)
+			 mr.commands=1;
+
+		 mr.pad = nds.pad;
+
+		 if(movie_lid) {
+			 mr.commands=4;
+			 movie_lid = false;
+		 }
+
+		 if(movie_reset_command) {
+			 mr.commands=2;
+			 movie_reset_command = false;
+		 }
+
+		 if(nds.isTouch) {
+			 mr.touch.x = nds.touchX >> 4;
+			 mr.touch.y = nds.touchY >> 4;
+			 mr.touch.touch = 1;
+		 } else {
+			 mr.touch.x = 0;
+			 mr.touch.y = 0;
+			 mr.touch.touch = 0;
+		 }
+
+		 mr.dump(&currMovieData, osRecordingMovie,currMovieData.records.size());
+		 currMovieData.records.push_back(mr);
+		 osd->addFixed(180, 176, "%s", "Recording");
+	 }
+
+	 /*extern uint8 joy[4];
+	 memcpy(&cur_input_display,joy,4);*/
+
+	if (ShowInputDisplay && nds.isTouch)
+		osd->addFixed(nds.touchX >> 4, (nds.touchY >> 4) + 192 , "%s %d %d", "X", nds.touchX >> 4, nds.touchY >> 4);
+ }
+
 
 //TODO 
 static void FCEUMOV_AddCommand(int cmd)
@@ -569,39 +715,64 @@ static void FCEUMOV_AddCommand(int cmd)
 	//DoEncode((cmd>>3)&0x3,cmd&0x7,1);
 }
 
+//little endian 4-byte cookies
+static const int kMOVI = 0x49564F4D;
+static const int kNOMO = 0x4F4D4F4E;
 
-static int FCEUMOV_WriteState(std::ostream* os)
+void mov_savestate(std::ostream* os)
 {
 	//we are supposed to dump the movie data into the savestate
+	//if(movieMode == MOVIEMODE_RECORD || movieMode == MOVIEMODE_PLAY)
+	//	return currMovieData.dump(os, true);
+	//else return 0;
 	if(movieMode == MOVIEMODE_RECORD || movieMode == MOVIEMODE_PLAY)
-		return currMovieData.dump(os, true);
-	else return 0;
+	{
+		write32le(kMOVI,os);
+		currMovieData.dump(os, true);
+	}
+	else
+	{
+		write32le(kNOMO,os);
+	}
 }
 
 
-//TODO EVERYTHING BELOW
 
 static bool load_successful;
 
-static bool FCEUMOV_ReadState(std::istream* is, uint32 size)
+bool mov_loadstate(std::istream* is, int size)
 {
 	load_successful = false;
 
+	int cookie;
+	if(read32le(&cookie,is) != 1) return false;
+	if(cookie == kNOMO)
+		return true;
+	else if(cookie != kMOVI)
+		return false;
+
+	size -= 4;
+
+	if (!movie_readonly && autoMovieBackup && freshMovie) //If auto-backup is on, movie has not been altered this session and the movie is in read+write mode
+	{
+		FCEUI_MakeBackupMovie(false);	//Backup the movie before the contents get altered, but do not display messages						  
+	}
+
 	//a little rule: cant load states in read+write mode with a movie from an archive.
 	//so we are going to switch it to readonly mode in that case
-	if(!movie_readonly 
-		//*&& FCEU_isFileInArchive(curMovieFilename)*/
-		) {
-		FCEU_PrintError("Cannot loadstate in Read+Write with movie from archive. Movie is now Read-Only.");
-		movie_readonly = true;
-	}
+//	if(!movie_readonly 
+//		//*&& FCEU_isFileInArchive(curMovieFilename)*/
+//		) {
+//		FCEU_PrintError("Cannot loadstate in Read+Write with movie from archive. Movie is now Read-Only.");
+//		movie_readonly = true;
+//	}
 
 	MovieData tempMovieData = MovieData();
 	std::ios::pos_type curr = is->tellg();
 	if(!LoadFM2(tempMovieData, is, size, false)) {
 		
-		/*is->seekg((uint32)curr+size);
-		extern bool FCEU_state_loading_old_format;
+	//	is->seekg((uint32)curr+size);
+	/*	extern bool FCEU_state_loading_old_format;
 		if(FCEU_state_loading_old_format) {
 			if(movieMode == MOVIEMODE_PLAY || movieMode == MOVIEMODE_RECORD) {
 				FCEUI_StopMovie();			
@@ -652,12 +823,29 @@ static bool FCEUMOV_ReadState(std::istream* is, uint32 size)
 
 		if(movie_readonly)
 		{
+			//-------------------------------------------------------------
+			//this code would reload the movie from disk. allegedly it is helpful to hexers, but
+			//it is way too slow with dsm format. so it is only here as a reminder, and in case someone
+			//wants to play with it
+			//-------------------------------------------------------------
+			//{
+			//	fstream fs (curMovieFilename);
+			//	if(!LoadFM2(tempMovieData, &fs, INT_MAX, false))
+			//	{
+			//		FCEU_PrintError("Failed to reload DSM after loading savestate");
+			//	}
+			//	fs.close();
+			//	currMovieData = tempMovieData;
+			//}
+			//-------------------------------------------------------------
+
 			//if the frame counter is longer than our current movie, then error
 			if(currFrameCounter > (int)currMovieData.records.size())
 			{
 				FCEU_PrintError("Savestate is from a frame (%d) after the final frame in the movie (%d). This is not permitted.", currFrameCounter, currMovieData.records.size()-1);
 				return false;
 			}
+
 			movieMode = MOVIEMODE_PLAY;
 		}
 		else
@@ -666,20 +854,22 @@ static bool FCEUMOV_ReadState(std::istream* is, uint32 size)
 			tempMovieData.truncateAt(currFrameCounter);
 			currMovieData = tempMovieData;
 			
-			#ifdef _S9XLUA_H
-			if(!FCEU_LuaRerecordCountSkip())
+		//	#ifdef _S9XLUA_H
+		//	if(!FCEU_LuaRerecordCountSkip())
 				currRerecordCount++;
-			#endif
+		//	#endif
 			
 			currMovieData.rerecordCount = currRerecordCount;
 
 			openRecordingMovie(curMovieFilename);
+			//printf("DUMPING MOVIE: %d FRAMES\n",currMovieData.records.size());
 			currMovieData.dump(osRecordingMovie, false);
 			movieMode = MOVIEMODE_RECORD;
 		}
 	}
 	
 	load_successful = true;
+	freshMovie = false;
 
 	//// Maximus: Show the last input combination entered from the
 	//// movie within the state
@@ -722,3 +912,138 @@ bool FCEUI_MovieGetInfo(std::istream* fp, MOVIE_INFO& info, bool skipFrameCount)
 
 	return true;
 }
+
+bool MovieRecord::parseBinary(MovieData* md, std::istream* is)
+{
+	commands=is->get();
+	is->read((char *) &pad, sizeof pad);
+	is->read((char *) &touch.x, sizeof touch.x);
+	is->read((char *) &touch.y, sizeof touch.y);
+	is->read((char *) &touch.touch, sizeof touch.touch);
+	return true;
+}
+
+
+void MovieRecord::dumpBinary(MovieData* md, std::ostream* os, int index)
+{
+	os->put(md->records[index].commands);
+	os->write((char *) &md->records[index].pad, sizeof md->records[index].pad);
+	os->write((char *) &md->records[index].touch.x, sizeof md->records[index].touch.x);
+	os->write((char *) &md->records[index].touch.y, sizeof md->records[index].touch.y);
+	os->write((char *) &md->records[index].touch.touch, sizeof md->records[index].touch.touch);
+}
+
+void LoadFM2_binarychunk(MovieData& movieData, std::istream* fp, int size)
+{
+	int recordsize = 1; //1 for the command
+
+	recordsize = 6;
+
+	assert(size%6==0);
+
+	//find out how much remains in the file
+	int curr = fp->tellg();
+	fp->seekg(0,std::ios::end);
+	int end = fp->tellg();
+	int flen = end-curr;
+	fp->seekg(curr,std::ios::beg);
+
+	//the amount todo is the min of the limiting size we received and the remaining contents of the file
+	int todo = std::min(size, flen);
+
+	int numRecords = todo/recordsize;
+	//printf("LOADED MOVIE: %d records; currFrameCounter: %d\n",numRecords,currFrameCounter);
+	movieData.records.resize(numRecords);
+	for(int i=0;i<numRecords;i++)
+	{
+		movieData.records[i].parseBinary(&movieData,fp);
+	}
+}
+
+#include <sstream>
+
+static bool CheckFileExists(const char* filename)
+{
+	//This function simply checks to see if the given filename exists
+	string checkFilename; 
+
+	if (filename)
+		checkFilename = filename;
+			
+	//Check if this filename exists
+	fstream test;
+	test.open(checkFilename.c_str(),fstream::in);
+		
+	if (test.fail())
+	{
+		test.close();
+		return false; 
+	}
+	else
+	{
+		test.close();
+		return true; 
+	}
+}
+
+void FCEUI_MakeBackupMovie(bool dispMessage)
+{
+	//This function generates backup movie files
+	string currentFn;					//Current movie fillename
+	string backupFn;					//Target backup filename
+	string tempFn;						//temp used in back filename creation
+	stringstream stream;
+	int x;								//Temp variable for string manip
+	bool exist = false;					//Used to test if filename exists
+	bool overflow = false;				//Used for special situation when backup numbering exceeds limit
+
+	currentFn = curMovieFilename;		//Get current moviefilename
+	backupFn = curMovieFilename;		//Make backup filename the same as current moviefilename
+	x = backupFn.find_last_of(".");		 //Find file extension
+	backupFn = backupFn.substr(0,x);	//Remove extension
+	tempFn = backupFn;					//Store the filename at this point
+	for (unsigned int backNum=0;backNum<999;backNum++) //999 = arbituary limit to backup files
+	{
+		stream.str("");					 //Clear stream
+		if (backNum > 99)
+			stream << "-" << backNum;	 //assign backNum to stream
+		 else if (backNum <= 99 && backNum >= 10)
+			stream << "-0" << backNum;      //Make it 010, etc if two digits
+		else
+			stream << "-00" << backNum;	 //Make it 001, etc if single digit
+		backupFn.append(stream.str());	 //add number to bak filename
+		backupFn.append(".bak");		 //add extension
+
+		exist = CheckFileExists(backupFn.c_str());	//Check if file exists
+		
+		if (!exist) 
+			break;						//Yeah yeah, I should use a do loop or something
+		else
+		{
+			backupFn = tempFn;			//Before we loop again, reset the filename
+			
+			if (backNum == 999)			//If 999 exists, we have overflowed, let's handle that
+			{
+				backupFn.append("-001.bak"); //We are going to simply overwrite 001.bak
+				overflow = true;		//Flag that we have exceeded limit
+				break;					//Just in case
+			}
+		}
+	}
+
+	MovieData md = currMovieData;								//Get current movie data
+	std::fstream* outf = new fstream(backupFn.c_str(),std::ios_base::out); //FCEUD_UTF8_fstream(backupFn, "wb");	//open/create file
+	md.dump(outf,false);										//dump movie data
+	delete outf;												//clean up, delete file object
+	
+	//TODO, decide if fstream successfully opened the file and print error message if it doesn't
+
+	if (dispMessage)	//If we should inform the user 
+	{
+//		if (overflow)
+//			FCEUI_DispMessage("Backup overflow, overwriting %s",backupFn.c_str()); //Inform user of overflow
+//		else
+//			FCEUI_DispMessage("%s created",backupFn.c_str()); //Inform user of backup filename
+	}
+}
+
