@@ -52,6 +52,8 @@
 
 //#undef FORCEINLINE
 //#define FORCEINLINE
+//#undef INLINE
+//#define INLINE
 
 using std::min;
 using std::max;
@@ -69,6 +71,17 @@ static u8 decal_table[32][32][32];
 static u8 index_lookup_table[65];
 static u8 index_start_table[8];
 
+static struct TClippedPoly
+{
+	int type;
+	POLY* poly;
+	VERT clipVerts[MAX_CLIPPED_VERTS];
+} *clippedPolys = NULL;
+static int clippedPolyCounter;
+
+TClippedPoly tempClippedPoly;
+TClippedPoly outClippedPoly;
+
 ////optimized float floor useful in limited cases
 ////from http://www.stereopsis.com/FPU.html#convert
 ////(unfortunately, it relies on certain FPU register settings)
@@ -202,16 +215,19 @@ struct PolyAttr
 
 } polyAttr;
 
+union FragmentColor {
+	u32 color;
+	struct {
+#ifdef WORDS_BIGENDIAN
+		u8 a,b,g,r;
+#else
+		u8 r,g,b,a;
+#endif
+	};
+};
+
 struct Fragment
 {
-	union Color {
-		u32 color;
-		struct {
-			//#ifdef WORDS_BIGENDIAN ?
-			u8 r,g,b,a;
-		} components;
-	} color;
-
 	u32 depth;
 
 	struct {
@@ -220,7 +236,7 @@ struct Fragment
 
 	u8 stencil;
 
-	u8 pad[5];
+	u8 pad;
 };
 
 static VERT* verts[MAX_CLIPPED_VERTS];
@@ -231,11 +247,15 @@ INLINE static void SubmitVertex(int vert_index, VERT& rawvert)
 }
 
 static Fragment screen[256*192];
+static FragmentColor screenColor[256*192];
+static FragmentColor toonTable[32];
+
 
 FORCEINLINE int iround(float f) {
 	return (int)f; //lol
 }
 
+
 static struct Sampler
 {
 	int width, height;
@@ -300,21 +320,16 @@ static struct Sampler
 		}
 	}
 
-	FORCEINLINE Fragment::Color sample(float u, float v)
+	FORCEINLINE FragmentColor sample(float u, float v)
 	{
 		//finally, we can use floor here. but, it is slower than we want.
 		//the best solution is probably to wait until the pipeline is full of fixed point
-		int iu = floorf(u);
-		int iv = floorf(v);
+		s32 iu = s32floor(u);
+		s32 iv = s32floor(v);
 		dowrap(iu,iv);
 
-		Fragment::Color color;
-		u32 col32 = ((u32*)textures.currentData)[(iv<<wshift)+iu];
-		//todo - teach texcache how to provide these already in 5555
-		col32 >>= 3;
-		col32 &= 0x1F1F1F1F;
-		color.color = col32;
-
+		FragmentColor color;
+		color.color = ((u32*)textures.currentData)[(iv<<wshift)+iu];
 		return color;
 	}
 
@@ -332,11 +347,11 @@ struct Shader
 	}
 
 	float invu, invv, w;
-	Fragment::Color materialColor;
+	FragmentColor materialColor;
 	
-	FORCEINLINE void shade(Fragment& dst)
+	FORCEINLINE void shade(FragmentColor& dst)
 	{
-		Fragment::Color texColor;
+		FragmentColor texColor;
 		float u,v;
 
 		switch(mode)
@@ -345,19 +360,19 @@ struct Shader
 			u = invu*w;
 			v = invv*w;
 			texColor = sampler.sample(u,v);
-			dst.color.components.r = modulate_table[texColor.components.r][materialColor.components.r];
-			dst.color.components.g = modulate_table[texColor.components.g][materialColor.components.g];
-			dst.color.components.b = modulate_table[texColor.components.b][materialColor.components.b];
-			dst.color.components.a = modulate_table[texColor.components.a][materialColor.components.a];
+			dst.r = modulate_table[texColor.r][materialColor.r];
+			dst.g = modulate_table[texColor.g][materialColor.g];
+			dst.b = modulate_table[texColor.b][materialColor.b];
+			dst.a = modulate_table[texColor.a][materialColor.a];
 			//dst.color.components.a = 31;
 			//#ifdef _MSC_VER
 			//if(GetAsyncKeyState(VK_SHIFT)) {
 			//	//debugging tricks
-			//	dst.color = materialColor;
+			//	dst = materialColor;
 			//	if(GetAsyncKeyState(VK_TAB)) {
-			//		u8 alpha = dst.color.components.a;
-			//		dst.color.color = polynum*8+8;
-			//		dst.color.components.a = alpha;
+			//		u8 alpha = dst.a;
+			//		dst.color = polynum*8+8;
+			//		dst.a = alpha;
 			//	}
 			//}
 			//#endif
@@ -366,37 +381,56 @@ struct Shader
 			u = invu*w;
 			v = invv*w;
 			texColor = sampler.sample(u,v);
-			dst.color.components.r = decal_table[texColor.components.a][texColor.components.r][materialColor.components.r];
-			dst.color.components.g = decal_table[texColor.components.a][texColor.components.g][materialColor.components.g];
-			dst.color.components.b = decal_table[texColor.components.a][texColor.components.b][materialColor.components.b];
-			dst.color.components.a = materialColor.components.a;
+			dst.r = decal_table[texColor.a][texColor.r][materialColor.r];
+			dst.g = decal_table[texColor.a][texColor.g][materialColor.g];
+			dst.b = decal_table[texColor.a][texColor.b][materialColor.b];
+			dst.a = materialColor.a;
 			break;
 		case 2: //toon/highlight shading
-			u = invu*w;
-			v = invv*w;
-			texColor = sampler.sample(u,v);
-			u32 toonColorVal; toonColorVal = gfx3d.rgbToonTable[materialColor.components.r];
-			Fragment::Color toonColor;
-			toonColor.components.r = ((toonColorVal & 0x0000FF) >>  3);
-			toonColor.components.g = ((toonColorVal & 0x00FF00) >> 11);
-			toonColor.components.b = ((toonColorVal & 0xFF0000) >> 19);
-			dst.color.components.r = modulate_table[texColor.components.r][toonColor.components.r];
-			dst.color.components.g = modulate_table[texColor.components.g][toonColor.components.g];
-			dst.color.components.b = modulate_table[texColor.components.b][toonColor.components.b];
-			dst.color.components.a = modulate_table[texColor.components.a][materialColor.components.a];
-			if(gfx3d.shading == GFX3D::HIGHLIGHT)
 			{
-				dst.color.components.r = min<u8>(31, (dst.color.components.r + toonColor.components.r));
-				dst.color.components.g = min<u8>(31, (dst.color.components.g + toonColor.components.g));
-				dst.color.components.b = min<u8>(31, (dst.color.components.b + toonColor.components.b));
+				u = invu*w;
+				v = invv*w;
+				texColor = sampler.sample(u,v);
+				if(texColor.r != 0x1f || texColor.g != 0x1f || texColor.b != 0x1f) {
+					int zzz=9;
+				}
+				FragmentColor toonColor = toonTable[materialColor.r];
+				if(sampler.texFormat == 0)
+				{
+					//if no texture is set then we dont need to modulate texture with toon 
+					//but rather just use toon directly
+					dst = toonColor;
+					dst.a = materialColor.a;
+				}
+				else
+				{
+					if(gfx3d.shading == GFX3D::HIGHLIGHT)
+					{
+						dst.r = modulate_table[texColor.r][materialColor.r];
+						dst.g = modulate_table[texColor.g][materialColor.r];
+						dst.b = modulate_table[texColor.b][materialColor.r];
+						dst.a = modulate_table[texColor.a][materialColor.a];
+
+						dst.r = min<u8>(31, (dst.r + toonColor.r));
+						dst.g = min<u8>(31, (dst.g + toonColor.g));
+						dst.b = min<u8>(31, (dst.b + toonColor.b));
+					}
+					else
+					{
+						dst.r = modulate_table[texColor.r][toonColor.r];
+						dst.g = modulate_table[texColor.g][toonColor.g];
+						dst.b = modulate_table[texColor.b][toonColor.b];
+						dst.a = modulate_table[texColor.a][materialColor.a];
+					}
+				}
 			}
 			break;
 		case 3: //shadows
 			//is this right? only with the material color?
-			dst.color = materialColor;
+			dst = materialColor;
 			break;
 		case 4: //our own special mode which only uses the material color (for when texturing is disabled)
-			dst.color = materialColor;
+			dst = materialColor;
 			break;
 
 		}
@@ -404,57 +438,57 @@ struct Shader
 
 } shader;
 
-static FORCEINLINE void alphaBlend(Fragment::Color & dst, const Fragment::Color & src)
+static FORCEINLINE void alphaBlend(FragmentColor & dst, const FragmentColor & src)
 {
 	if(gfx3d.enableAlphaBlending)
 	{
-		if(src.components.a == 0)
+		if(src.a == 0)
 		{
-			dst.components.a = max(src.components.a,dst.components.a);	
+			dst.a = max(src.a,dst.a);	
 		}
-		else if(src.components.a == 31 || dst.components.a == 0)
+		else if(src.a == 31 || dst.a == 0)
 		{
-			dst.color = src.color;
-			dst.components.a = max(src.components.a,dst.components.a);
+			dst = src;
+			dst.a = max(src.a,dst.a);
 		}
 		else
 		{
-			u8 alpha = src.components.a+1;
+			u8 alpha = src.a+1;
 			u8 invAlpha = 32 - alpha;
-			dst.components.r = (alpha*src.components.r + invAlpha*dst.components.r)>>5;
-			dst.components.g = (alpha*src.components.g + invAlpha*dst.components.g)>>5;
-			dst.components.b = (alpha*src.components.b + invAlpha*dst.components.b)>>5;
-			dst.components.a = max(src.components.a,dst.components.a);
+			dst.r = (alpha*src.r + invAlpha*dst.r)>>5;
+			dst.g = (alpha*src.g + invAlpha*dst.g)>>5;
+			dst.b = (alpha*src.b + invAlpha*dst.b)>>5;
+			dst.a = max(src.a,dst.a);
 		}
 	}
 	else
 	{
-		if(src.components.a == 0)
+		if(src.a == 0)
 		{
 			//do nothing; the fragment is totally transparent
 		}
 		else
 		{
-			dst.color = src.color;
+			dst = src;
 		}
 	}
 }
 
 static FORCEINLINE void pixel(int adr,float r, float g, float b, float invu, float invv, float w, float z) {
 	Fragment &destFragment = screen[adr];
+	FragmentColor &destFragmentColor = screenColor[adr];
 
 	//depth test
 	u32 depth;
 	if(gfx3d.wbuffer) {
 		//not sure about this
 		//this value was chosen to make the skybox, castle window decals, and water level render correctly in SM64
-		depth = (4096*w);
+		depth = u32floor(4096*w);
 	}
 	else
 	{
-		//depth = fastFloor(z*0x7FFF)>>8;
-		depth = z*0x7FFF;
-		//depth = z*0xFFFFFF;
+		depth = u32floor(z*0x7FFF);
+		depth <<= 9;
 	}
 	if(polyAttr.decalMode)
 	{
@@ -483,25 +517,25 @@ static FORCEINLINE void pixel(int adr,float r, float g, float b, float invu, flo
 	//this is a HACK: 
 	//we are being very sloppy with our interpolation precision right now
 	//and rather than fix it, i just want to clamp it
-	shader.materialColor.components.r = max(0,min(31,(int)r));
-	shader.materialColor.components.g = max(0,min(31,(int)g));
-	shader.materialColor.components.b = max(0,min(31,(int)b));
+	shader.materialColor.r = max(0U,min(31U,u32floor(r)));
+	shader.materialColor.g = max(0U,min(31U,u32floor(g)));
+	shader.materialColor.b = max(0U,min(31U,u32floor(b)));
 
-	shader.materialColor.components.a = polyAttr.alpha;
+	shader.materialColor.a = polyAttr.alpha;
 
 	//pixel shader
-	Fragment shaderOutput;
+	FragmentColor shaderOutput;
 	shader.shade(shaderOutput);
 
 	//alpha test
 	if(gfx3d.enableAlphaTest)
 	{
-		if(shaderOutput.color.components.a < gfx3d.alphaTestRef)
+		if(shaderOutput.a < gfx3d.alphaTestRef)
 			goto rejected_fragment;
 	}
 
 	//we shouldnt do any of this if we generated a totally transparent pixel
-	if(shaderOutput.color.components.a != 0)
+	if(shaderOutput.a != 0)
 	{
 		//handle shadow polys
 		if(shader.mode == 3)
@@ -538,7 +572,7 @@ static FORCEINLINE void pixel(int adr,float r, float g, float b, float invu, flo
 		}
 
 		//handle polyids
-		bool isOpaquePixel = shaderOutput.color.components.a == 31;
+		bool isOpaquePixel = shaderOutput.a == 31;
 		if(isOpaquePixel)
 		{
 			destFragment.polyid.opaque = polyAttr.polyid;
@@ -566,7 +600,7 @@ static FORCEINLINE void pixel(int adr,float r, float g, float b, float invu, flo
 		}
 
 		//alpha blending and write color
-		alphaBlend(destFragment.color, shaderOutput.color);
+		alphaBlend(destFragmentColor, shaderOutput);
 
 		//depth writing
 		if(isOpaquePixel || polyAttr.translucentDepthWrite)
@@ -588,12 +622,13 @@ static FORCEINLINE void pixel(int adr,float r, float g, float b, float invu, flo
 	;
 }
 
+
 typedef int fixed28_4;
 
 static bool failure;
 
 // handle floor divides and mods correctly 
-inline void FloorDivMod(long Numerator, long Denominator, long &Floor, long &Mod)
+INLINE void FloorDivMod(long Numerator, long Denominator, long &Floor, long &Mod)
 {
 	//These must be caused by invalid or degenerate shapes.. not sure yet.
 	//check it out in the mario face intro of SM64
@@ -624,25 +659,25 @@ inline void FloorDivMod(long Numerator, long Denominator, long &Floor, long &Mod
 	}
 }
 
-inline fixed28_4 FloatToFixed28_4( float Value ) {
-	return Value * 16;
+INLINE fixed28_4 FloatToFixed28_4( float Value ) {
+	return (fixed28_4)(Value * 16);
 }
-inline float Fixed28_4ToFloat( fixed28_4 Value ) {
+INLINE float Fixed28_4ToFloat( fixed28_4 Value ) {
 	return Value / 16.0;
 }
 //inline fixed16_16 FloatToFixed16_16( float Value ) {
-//	return Value * 65536;
+//	return (fixed16_6)(Value * 65536);
 //}
 //inline float Fixed16_16ToFloat( fixed16_16 Value ) {
 //	return Value / 65536.0;
 //}
-inline fixed28_4 Fixed28_4Mul( fixed28_4 A, fixed28_4 B ) {
+INLINE fixed28_4 Fixed28_4Mul( fixed28_4 A, fixed28_4 B ) {
 	// could make this asm to prevent overflow
 	return (A * B) / 16;	// 28.4 * 28.4 = 24.8 / 16 = 28.4
 }
-inline long Ceil28_4( fixed28_4 Value ) {
-	long ReturnValue;
-	long Numerator = Value - 1 + 16;
+INLINE int Ceil28_4( fixed28_4 Value ) {
+	int ReturnValue;
+	int Numerator = Value - 1 + 16;
 	if(Numerator >= 0) {
 		ReturnValue = Numerator/16;
 	} else {
@@ -693,8 +728,8 @@ FORCEINLINE edge_fx_fl::edge_fx_fl(int Top, int Bottom) {
 
 	if(Height)
 	{
-		long dN = verts[Bottom]->y - verts[Top]->y;
-		long dM = verts[Bottom]->x - verts[Top]->x;
+		long dN = long(verts[Bottom]->y - verts[Top]->y);
+		long dM = long(verts[Bottom]->x - verts[Top]->x);
 	
 		long InitialNumerator = dM*16*Y - dM*verts[Top]->y + dN*verts[Top]->x - 1 + dN*16;
 		FloorDivMod(InitialNumerator,dN*16,X,ErrorTerm);
@@ -771,6 +806,7 @@ static void drawscanline(edge_fx_fl *pLeft, edge_fx_fl *pRight)
 	{
 		if(x<0 || x>255) {
 			printf("rasterizer rendering at x=%d! oops!\n",x);
+			return;
 		}
 		pixel(adr,color[0],color[1],color[2],u,v,1.0f/invw,z);
 		adr++;
@@ -800,7 +836,7 @@ static void runscanlines(edge_fx_fl *left, edge_fx_fl *right)
 
 //rotates verts counterclockwise
 template<int type>
-inline static void rot_verts() {
+INLINE static void rot_verts() {
 	#define ROTSWAP(X) if(type>X) swap(verts[X-1],verts[X]);
 	ROTSWAP(1); ROTSWAP(2); ROTSWAP(3); ROTSWAP(4);
 	ROTSWAP(5); ROTSWAP(6); ROTSWAP(7);
@@ -896,6 +932,9 @@ static char SoftRastInit(void)
 	if(!tables_generated)
 	{
 		tables_generated = true;
+
+		clippedPolys = new TClippedPoly[POLYLIST_SIZE*2];
+
 		for(int i=0;i<32;i++)
 		{
 			for(int j=0;j<32;j++)
@@ -929,7 +968,8 @@ static char SoftRastInit(void)
 	return 1;
 }
 
-static void SoftRastReset() {}
+static void SoftRastReset() {
+}
 
 static void SoftRastClose()
 {
@@ -939,58 +979,35 @@ static void SoftRastVramReconfigureSignal() {
 	TexCache_Invalidate();
 }
 
-static void SoftRastGetLine(int line, u16* dst, u8* dstAlpha)
+static void SoftRastConvertFramebuffer()
 {
-	Fragment* src = screen+((line)<<8);
-	for(int i=0;i<256;i++)
-	{
-		const bool testRenderAlpha = false;
-		u8 r = src->color.components.r;
-		u8 g = src->color.components.g;
-		u8 b = src->color.components.b;
-		*dst = R5G5B5TORGB15(r,g,b);
-		if(src->color.components.a > 0) 
-			*dst |= 0x8000;
-		*dstAlpha = alpha_5bit_to_4bit[src->color.components.a];
+	FragmentColor* src = screenColor;
+	u16* dst = gfx3d_convertedScreen;
+	u8* dstAlpha = gfx3d_convertedAlpha;
 
-		if(testRenderAlpha)
+	//in an effort to speed this up, the misc pixel buffers and the color buffer were separated.
+
+	for(int i=0,y=0;y<192;y++)
+	{
+		#ifndef NOSSE2
+			u8* wanx = (u8*)&src[i];
+			#define ASS(X,Y) __asm { prefetchnta [wanx+32*0x##X##Y] }
+			#define PUNK(X) ASS(X,0) ASS(X,1) ASS(X,2) ASS(X,3) ASS(X,4) ASS(X,5) ASS(X,6) ASS(X,7) ASS(X,8) ASS(X,9) ASS(X,A) ASS(X,B) ASS(X,C) ASS(X,D) ASS(X,E) ASS(X,F) 
+			PUNK(0); PUNK(1);
+		#endif
+
+		for(int x=0;x<256;x++,i++)
 		{
-			*dst = 0x8000 | R5G5B5TORGB15(src->color.components.a,src->color.components.a,src->color.components.a);
-			*dstAlpha = 16;
+			const u8 r = src[i].r;
+			const u8 g = src[i].g;
+			const u8 b = src[i].b;
+			const u8 a = src[i].a;
+			dst[i] = R5G5B5TORGB15(r,g,b) | alpha_lookup[a];
+			dstAlpha[i] = alpha_5bit_to_4bit[a];
 		}
-
-		src++;
-		dst++;
-		dstAlpha++;
-	}
-
-}
-
-static void SoftRastGetLineCaptured(int line, u16* dst) {
-	Fragment* src = screen+((line)<<8);
-	for(int i=0;i<256;i++)
-	{
-		u8 r = src->color.components.r;
-		u8 g = src->color.components.g;
-		u8 b = src->color.components.b;
-		*dst = R5G5B5TORGB15(r,g,b);
-		if(src->color.components.a > 0) 
-			*dst |= 0x8000;
-		src++;
-		dst++;
 	}
 }
 
-static struct TClippedPoly
-{
-	int type;
-	POLY* poly;
-	VERT clipVerts[MAX_CLIPPED_VERTS];
-} clippedPolys[POLYLIST_SIZE*2];
-static int clippedPolyCounter;
-
-TClippedPoly tempClippedPoly;
-TClippedPoly outClippedPoly;
 
 template<typename T>
 static T interpolate(const float ratio, const T& x0, const T& x1) {
@@ -1157,10 +1174,12 @@ static void clipPoly(POLY* poly)
 static void SoftRastRender()
 {
 	Fragment clearFragment;
-	clearFragment.color.components.r = gfx3d.clearColor&0x1F;
-	clearFragment.color.components.g = (gfx3d.clearColor>>5)&0x1F;
-	clearFragment.color.components.b = (gfx3d.clearColor>>10)&0x1F;
-	clearFragment.color.components.a = (gfx3d.clearColor>>16)&0x1F;
+	FragmentColor clearFragmentColor;
+	clearFragment.pad = 0;
+	clearFragmentColor.r = gfx3d.clearColor&0x1F;
+	clearFragmentColor.g = (gfx3d.clearColor>>5)&0x1F;
+	clearFragmentColor.b = (gfx3d.clearColor>>10)&0x1F;
+	clearFragmentColor.a = (gfx3d.clearColor>>16)&0x1F;
 	clearFragment.polyid.opaque = (gfx3d.clearColor>>24)&0x3F;
 	//special value for uninitialized translucent polyid. without this, fires in spiderman2 dont display
 	//I am not sure whether it is right, though. previously this was cleared to 0, as a guess,
@@ -1171,6 +1190,56 @@ static void SoftRastRender()
 	for(int i=0;i<256*192;i++)
 		screen[i] = clearFragment;
 
+	if(gfx3d.enableClearImage)
+	{
+		u16* clearImage = (u16*)ARM9Mem.texInfo.textureSlotAddr[2];
+		u16* clearDepth = (u16*)ARM9Mem.texInfo.textureSlotAddr[3];
+
+		//not emulated until we actually have a test case
+		//(since in the meantime it would just slow us down)
+		//(and when we find a case, it will be obvious that it isnt scrolling, instead of possibly obscured by a bug)
+		u16 scroll = T1ReadWord(ARM9Mem.ARM9_REG,0x356); //CLRIMAGE_OFFSET
+		u16 xscroll = scroll&0xFF;
+		u16 yscroll = (scroll>>8)&0xFF;
+
+		FragmentColor *dstColor = screenColor;
+		Fragment *dst = screen;
+
+		for(int y=0;y<192;y++)
+			for(int x=0;x<256;x++) {
+				
+				//this hasnt been tested
+				//TODO - dont do this if we are mapped to blank memory (such as in sonic chronicles)
+				//(or use a special zero fill in the bulk clearing above)
+				u16 col = *clearImage;
+				dstColor->color = RGB15TO32(255*(col>>15),col);
+				
+				//this is tested quite well in the sonic chronicles main map mode
+				//where depth values are used for trees etc you can walk behind
+				u32 depth = *clearDepth;
+				//masking off fog for now
+				depth &= 0x7FFF;
+				//TODO - might consider a lookup table for this
+				dst->depth = gfx3d_extendDepth_15_to_24(depth);
+
+				clearDepth++;
+				clearImage++;
+				dstColor++;
+				dst++;
+			}
+	}
+	else 
+		for(int i=0;i<256*192;i++)
+			screenColor[i] = clearFragmentColor;
+
+	//convert the toon colors
+	//TODO for a slight speedup this could be cached in gfx3d (oglrenderer could benefit as well)
+	for(int i=0;i<32;i++) {
+		toonTable[i].r = gfx3d.u16ToonTable[i]&0x1F;
+		toonTable[i].g = (gfx3d.u16ToonTable[i]>>5)&0x1F;
+		toonTable[i].b = (gfx3d.u16ToonTable[i]>>10)&0x1F;
+	}
+
 	//convert colors to float to get more precision in case we need it
 	for(int i=0;i<gfx3d.vertlist->count;i++)
 		gfx3d.vertlist->list[i].color_to_float();
@@ -1216,10 +1285,12 @@ static void SoftRastRender()
 			vert.coord[0] += viewport.x;
 			vert.coord[1] *= viewport.height;
 			vert.coord[1] += viewport.y;
-			if(vert.coord[1]>192||vert.color[1]<0) {
-				int zzz=9;
-			}
 			vert.coord[1] = 192 - vert.coord[1];
+
+			//well, i guess we need to do this to keep Princess Debut from rendering huge polys.
+			//there must be something strange going on
+			vert.coord[0] = max(0.0f,min(256.0f,vert.coord[0]));
+			vert.coord[1] = max(0.0f,min(192.0f,vert.coord[1]));
 		}
 	}
 
@@ -1266,7 +1337,7 @@ static void SoftRastRender()
 	
 		if(needInitTexture || lastTextureFormat != poly->texParam || lastTexturePalette != poly->texPalette)
 		{
-			TexCache_SetTexture(poly->texParam,poly->texPalette);
+			TexCache_SetTexture<TexFormat_15bpp>(poly->texParam,poly->texPalette);
 			sampler.setup(poly->texParam);
 			lastTextureFormat = poly->texParam;
 			lastTexturePalette = poly->texPalette;
@@ -1290,7 +1361,7 @@ static void SoftRastRender()
 	}
 
 	//	printf("rendered %d of %d polys after backface culling\n",gfx3d.polylist->count-culled,gfx3d.polylist->count);
-
+	SoftRastConvertFramebuffer();
 }
 
 GPU3DInterface gpu3DRasterize = {
@@ -1300,6 +1371,4 @@ GPU3DInterface gpu3DRasterize = {
 	SoftRastClose,
 	SoftRastRender,
 	SoftRastVramReconfigureSignal,
-	SoftRastGetLine,
-	SoftRastGetLineCaptured
 };
