@@ -30,6 +30,10 @@
 #include "armcpu.h"
 #include <time.h>
 #include <string.h>
+#ifdef WIN32
+#include "windows/main.h"
+#endif
+#include "movie.h"
 
 typedef struct
 {
@@ -62,6 +66,58 @@ u8 cmdBitsSize[8] = {8, 8, 56, 24, 0, 24, 8, 8};
 
 #define toBCD(x) ((x / 10) << 4) | (x % 10);
 
+struct movietime {
+
+	int sec;
+	int minute;
+	int hour;
+	int monthday;
+	int month;
+	int year;
+	int weekday;
+};
+
+struct movietime movie;
+
+bool moviemode=false;
+
+void InitMovieTime(void)
+{
+	movie.year=9;
+	movie.month=1;
+	movie.monthday=1;
+	movie.weekday=4;
+}	
+
+#ifdef WIN32
+static void MovieTime(void) {
+
+	//now, you might think it is silly to go through all these conniptions
+	//when we could just assume that there are 60fps and base the seconds on frameCounter/60
+	//but, we were imagining that one day we might need more precision
+
+	const u32 arm9rate_unitsperframe = 560190<<1;
+	const u32 arm9rate_unitspersecond = (u32)(arm9rate_unitsperframe * 59.8261);
+	const u64 noon = (u64)arm9rate_unitspersecond * 60 * 60 * 12;
+	           
+	u64 frameCycles = (u64)arm9rate_unitsperframe * currFrameCounter;
+	u64 totalcycles = frameCycles + noon;
+	u32 totalseconds=totalcycles/arm9rate_unitspersecond;
+
+	movie.sec=totalseconds % 60;
+	movie.minute=totalseconds/60;
+	movie.hour=movie.minute/60;
+
+	//convert to sane numbers
+	movie.minute=movie.minute % 60;
+	movie.hour=movie.hour % 24;
+}
+#else
+static void MovieTime(void)
+{
+}
+#endif
+
 static void rtcRecv()
 {
 	//INFO("RTC Read command 0x%02X\n", (rtc.cmd >> 1));
@@ -86,15 +142,33 @@ static void rtcRecv()
 				struct tm *tm_local= localtime(&tm);
 				tm_local->tm_year %= 100;
 				tm_local->tm_mon++;
-				rtc.data[0] = toBCD(tm_local->tm_year);
-				rtc.data[1] = toBCD(tm_local->tm_mon);
-				rtc.data[2] = toBCD(tm_local->tm_mday);
-				rtc.data[3] =  (tm_local->tm_wday + 6) & 7;
-				if (!(rtc.regStatus1 & 0x02)) tm_local->tm_hour %= 12;
-				rtc.data[4] = ((tm_local->tm_hour < 12) ? 0x00 : 0x40) | toBCD(tm_local->tm_hour);
-				rtc.data[5] =  toBCD(tm_local->tm_min);
-				rtc.data[6] =  toBCD(tm_local->tm_sec);
-				break;
+
+				if(movieMode != MOVIEMODE_INACTIVE) {
+
+					MovieTime();
+					
+					rtc.data[0]=toBCD(movie.year);
+					rtc.data[1]=toBCD(movie.month);
+					rtc.data[2]=toBCD(movie.monthday);
+					rtc.data[3]=(movie.weekday + 6) & 7;
+					if (!(rtc.regStatus1 & 0x02)) movie.hour %= 12;
+					rtc.data[4] = ((movie.hour < 12) ? 0x00 : 0x40) | toBCD(movie.hour);		
+					rtc.data[5]=toBCD(movie.minute);
+					rtc.data[6]=toBCD(movie.sec);
+					break;
+				}
+				else {
+
+					rtc.data[0] = toBCD(tm_local->tm_year);
+					rtc.data[1] = toBCD(tm_local->tm_mon);
+					rtc.data[2] = toBCD(tm_local->tm_mday);
+					rtc.data[3] =  (tm_local->tm_wday + 6) & 7;
+					if (!(rtc.regStatus1 & 0x02)) tm_local->tm_hour %= 12;
+					rtc.data[4] = ((tm_local->tm_hour < 12) ? 0x00 : 0x40) | toBCD(tm_local->tm_hour);
+					rtc.data[5] =  toBCD(tm_local->tm_min);
+					rtc.data[6] =  toBCD(tm_local->tm_sec);
+					break;
+				}
 			}
 		case 3:				// time
 			{
@@ -102,11 +176,24 @@ static void rtcRecv()
 				time_t	tm;
 				time(&tm);
 				struct tm *tm_local= localtime(&tm);
-				if (!(rtc.regStatus1 & 0x02)) tm_local->tm_hour %= 12;
-				rtc.data[0] = ((tm_local->tm_hour < 12) ? 0x00 : 0x40) | toBCD(tm_local->tm_hour);
-				rtc.data[1] =  toBCD(tm_local->tm_min);
-				rtc.data[2] =  toBCD(tm_local->tm_sec);
-				break;
+
+				if(movieMode != MOVIEMODE_INACTIVE) {
+
+					MovieTime();
+
+					if (!(rtc.regStatus1 & 0x02)) movie.hour %= 12;
+					rtc.data[0] = ((movie.hour < 12) ? 0x00 : 0x40) | toBCD(movie.hour);
+					rtc.data[1] =  toBCD(movie.minute);
+					rtc.data[2] =  toBCD(movie.sec);
+				}
+				else {
+
+					if (!(rtc.regStatus1 & 0x02)) tm_local->tm_hour %= 12;
+					rtc.data[0] = ((tm_local->tm_hour < 12) ? 0x00 : 0x40) | toBCD(tm_local->tm_hour);
+					rtc.data[1] =  toBCD(tm_local->tm_min);
+					rtc.data[2] =  toBCD(tm_local->tm_sec);
+					break;
+				}
 			}
 		case 4:				// freq/alarm 1
 			/*if (cmdBitsSize[0x04] == 8)
