@@ -32,10 +32,16 @@
 #include "NDSSystem.h"
 #include "render3D.h"
 #include "cp15.h"
+#include "GPU_osd.h"
+#include "version.h"
 
 #include "memorystream.h"
 #include "readwrite.h"
 #include "gfx3d.h"
+#include "movie.h"
+#ifdef _MSC_VER
+#include "windows/main.h"
+#endif
 
 
 //void*v is actually a void** which will be indirected before reading
@@ -47,6 +53,9 @@ savestates_t savestates[NB_STATES];
 #define SAVESTATE_VERSION       11
 static const char* magic = "DeSmuME SState\0";
 
+//a savestate chunk loader can set this if it wants to permit a silent failure (for compatibility)
+static bool SAV_silent_fail_flag;
+
 #ifndef MAX_PATH
 #define MAX_PATH 256
 #endif
@@ -129,8 +138,8 @@ SFORMAT SF_ARM9[]={
 };
 
 SFORMAT SF_MEM[]={
-	{ "ITCM", 1, 0x8000,   ARM9Mem.ARM9_ITCM},
-	{ "DTCM", 1, 0x4000,   ARM9Mem.ARM9_DTCM},
+	{ "ITCM", 1, sizeof(ARM9Mem.ARM9_ITCM),   ARM9Mem.ARM9_ITCM},
+	{ "DTCM", 1, sizeof(ARM9Mem.ARM9_DTCM),   ARM9Mem.ARM9_DTCM},
 
 	 //for legacy purposes, WRAX is a separate variable. shouldnt be a problem.
 	{ "WRAM", 1, 0x400000, ARM9Mem.MAIN_MEM},
@@ -141,13 +150,11 @@ SFORMAT SF_MEM[]={
 	//but there are actually no more registers than this
 	{ "9REG", 1, 0x2000,   ARM9Mem.ARM9_REG},
 
-	{ "VMEM", 1, 0x800,    ARM9Mem.ARM9_VMEM},
-	{ "OAMS", 1, 0x800,    ARM9Mem.ARM9_OAM},
-	{ "ABGM", 1, 0x80000,  ARM9Mem.ARM9_ABG},
-	{ "BBGM", 1, 0x20000,  ARM9Mem.ARM9_BBG},
-	{ "AOBJ", 1, 0x40000,  ARM9Mem.ARM9_AOBJ},
-	{ "BOBJ", 1, 0x20000,  ARM9Mem.ARM9_BOBJ},
-	{ "LCDM", 1, 0xA4000,  ARM9Mem.ARM9_LCD},
+	{ "VMEM", 1, sizeof(ARM9Mem.ARM9_VMEM),    ARM9Mem.ARM9_VMEM},
+	{ "OAMS", 1, sizeof(ARM9Mem.ARM9_OAM),    ARM9Mem.ARM9_OAM},
+
+	//this size is specially chosen to avoid saving the blank space at the end
+	{ "LCDM", 1, 0xA4000,		ARM9Mem.ARM9_LCD},
 	{ 0 }
 };
 
@@ -155,6 +162,7 @@ SFORMAT SF_NDS[]={
 	{ "_9CY", 4, 1, &nds.ARM9Cycle},
 	{ "_7CY", 4, 1, &nds.ARM7Cycle},
 	{ "_CYC", 4, 1, &nds.cycles},
+	{ "_WCY", 4, 1, &nds.wifiCycle},
 	{ "_TCY", 4, 8, nds.timerCycle},
 	{ "_TOV", 4, 8, nds.timerOver},
 	{ "_NHB", 4, 1, &nds.nextHBlank},
@@ -173,15 +181,12 @@ extern u32 DMASrc[2][4];
 extern u32 DMADst[2][4];
 
 SFORMAT SF_MMU[]={
-	{ "M7BI", 1, 0x4000,  MMU.ARM7_BIOS},
-	{ "M7ER", 1, 0x10000, MMU.ARM7_ERAM},
-	{ "M7RG", 1, 0x10000, MMU.ARM7_REG},
-	{ "M7WI", 1, 0x10000, MMU.ARM7_WIRAM},
-	{ "MVRM", 1, 4,       MMU.VRAM_MAP},
-	{ "MVRM", 4, 9,       MMU.LCD_VRAM_ADDR},
-	{ "MVRM", 1, 9,       MMU.LCDCenable},
-	{ "MSWI", 1, 0x8000,  MMU.SWIRAM},
-	{ "MCRA", 1, 0x10000, MMU.CART_RAM},
+	{ "M7BI", 1, sizeof(MMU.ARM7_BIOS), MMU.ARM7_BIOS},
+	{ "M7ER", 1, sizeof(MMU.ARM7_ERAM), MMU.ARM7_ERAM},
+	{ "M7RG", 1, sizeof(MMU.ARM7_REG), MMU.ARM7_REG},
+	{ "M7WI", 1, sizeof(MMU.ARM7_WIRAM), MMU.ARM7_WIRAM},
+	{ "MSWI", 1, sizeof(MMU.SWIRAM), MMU.SWIRAM},
+	{ "MCRA", 1, sizeof(MMU.CART_RAM), MMU.CART_RAM},
 	{ "M9RW", 1, 1,       &MMU.ARM9_RW_MODE},
 	{ "MDTC", 4, 1,       &MMU.DTCMRegion},
 	{ "MITC", 4, 1,       &MMU.ITCMRegion},
@@ -233,40 +238,77 @@ SFORMAT SF_MMU[]={
 
 	//fifos
 	{ "F0TL", 1, 1,       &ipc_fifo[0].tail},
-	{ "F0BF", 4, 16,      &ipc_fifo[0].buf[0]},
+	{ "F0BF", 4, 16,      ipc_fifo[0].buf},
 	{ "F1TL", 1, 1,       &ipc_fifo[1].tail},
-	{ "F1BF", 4, 16,      &ipc_fifo[1].buf[0]},
+	{ "F1BF", 4, 16,      ipc_fifo[1].buf},
 
 	{ "FDHD", 4, 1,       &disp_fifo.head},
 	{ "FDTL", 4, 1,       &disp_fifo.tail},
-	{ "FDBF", 4, 0x6000,  &disp_fifo.buf[0]},
+	{ "FDBF", 4, 0x6000,  disp_fifo.buf},
 	
 	{ 0 }
 };
 
+SFORMAT SF_MOVIE[]={
+	{ "FRAC", 4, 1, &currFrameCounter},
+	{ "LAGC", 4, 1, &TotalLagFrames},
+	{ 0 }
+};
 
 static void mmu_savestate(std::ostream* os)
 {
 	//version
-	write32le(0,os);
-
-	write32le(MMU.bupmem.size,os);
-	os->write((char*)MMU.bupmem.data,MMU.bupmem.size);
+	write32le(2,os);
+	
+	//newer savefile system:
+	MMU_new.backupDevice.save_state(os);
 }
 
-static bool mmu_loadstate(std::istream* is)
+static bool mmu_loadstate(std::istream* is, int size)
 {
 	//read version
 	int version;
 	if(read32le(&version,is) != 1) return false;
-	if(version != 0) return false;
+	
+	if(version == 0 || version == 1)
+	{
+		u32 bupmem_size;
+		u32 addr_size;
 
-	u32 bupmem_size;
-	if(read32le(&bupmem_size,is) != 1) return false;
-	if(bupmem_size != MMU.bupmem.size) return false; //mismatch between current initialized and saved size
+		if(version == 0)
+		{
+			//version 0 was buggy and didnt save the type. 
+			//it would silently fail if there was a size mismatch
+			SAV_silent_fail_flag = true;
+			if(read32le(&bupmem_size,is) != 1) return false;
+			//if(bupmem_size != MMU.bupmem.size) return false; //mismatch between current initialized and saved size
+			addr_size = BackupDevice::addr_size_for_old_save_size(bupmem_size);
+		}
+		else if(version == 1)
+		{
+			//version 1 reinitializes the save system with the type that was saved
+			int bupmem_type;
+			if(read32le(&bupmem_type,is) != 1) return false;
+			if(read32le(&bupmem_size,is) != 1) return false;
+			addr_size = BackupDevice::addr_size_for_old_save_type(bupmem_type);
+			if(addr_size == 0xFFFFFFFF)
+				addr_size = BackupDevice::addr_size_for_old_save_size(bupmem_size);
+		}
 
-	is->read((char*)MMU.bupmem.data,bupmem_size);
-	if(is->fail()) return false;
+		if(addr_size == 0xFFFFFFFF)
+			return false;
+
+		u8* temp = new u8[bupmem_size];
+		is->read((char*)temp,bupmem_size);
+		MMU_new.backupDevice.load_old_state(addr_size,temp,bupmem_size);
+		delete[] temp;
+		if(is->fail()) return false;
+	}
+	else if(version == 2)
+	{
+		//newer savefile system:
+		MMU_new.backupDevice.load_state(is);
+	}
 
 	return true;
 }
@@ -368,7 +410,7 @@ static bool cp15_loadone(armcp15_t *cp15, std::istream* is)
     return true;
 }
 
-static bool cp15_loadstate(std::istream* is)
+static bool cp15_loadstate(std::istream* is, int size)
 {
 	//read version
 	int version;
@@ -387,12 +429,12 @@ static bool cp15_loadstate(std::istream* is)
 static char * format_time(time_t cal_time)
 {
   struct tm *time_struct;
-  static char string[30];
+  static char str[64];
 
   time_struct=localtime(&cal_time);
-  strftime(string, sizeof string, "%Y-%m-%d %H:%M", time_struct);
+  strftime(str, sizeof str, "%d-%b-%Y %H:%M:%S", time_struct);
 
-  return(string);
+  return(str);
 }
 
 void clear_savestates()
@@ -432,7 +474,18 @@ void savestate_slot(int num)
    strncpy(filename, pathFilenameToROMwithoutExt, MAX_PATH);
    if (strlen(filename) + strlen(".dsx") + strlen("-2147483648") /* = biggest string for num */ >MAX_PATH) return ;
    sprintf(filename+strlen(filename), ".ds%d", num);
-   savestate_save(filename);
+
+   if (savestate_save(filename))
+   {
+	   osd->setLineColor(255, 255, 255);
+	   osd->addLine("Saved to %i slot", num);
+   }
+   else
+   {
+	   osd->setLineColor(255, 0, 0);
+	   osd->addLine("Error save to %i slot", num);
+	   return;
+   }
 
    savestates[num-1].exists = TRUE;
    if( stat(filename,&sbuf) == -1 ) return;
@@ -445,7 +498,16 @@ void loadstate_slot(int num)
    strncpy(filename, pathFilenameToROMwithoutExt, MAX_PATH);
    if (strlen(filename) + strlen(".dsx") + strlen("-2147483648") /* = biggest string for num */ >MAX_PATH) return ;
    sprintf(filename+strlen(filename), ".ds%d", num);
-   savestate_load(filename);
+   if (savestate_load(filename))
+   {
+	   osd->setLineColor(255, 255, 255);
+	   osd->addLine("Loaded from %i slot", num);
+   }
+   else
+   {
+	   osd->setLineColor(255, 0, 0);
+	   osd->addLine("Error from load %i slot", num);
+   }
 }
 
 u8 sram_read (u32 address) {
@@ -480,6 +542,9 @@ int sram_load (const char *file_name) {
 
 	fclose ( file );
 
+	osd->setLineColor(255, 255, 255);
+	osd->addLine("Loaded SRAM");
+
 	return 1;
 
 }
@@ -497,6 +562,9 @@ int sram_save (const char *file_name) {
 
 	fclose ( file );
 
+	osd->setLineColor(255, 255, 255);
+	osd->addLine("Saved SRAM");
+
 	return 1;
 
 }
@@ -589,7 +657,7 @@ static int SubWrite(std::ostream* os, SFORMAT *sf)
 		int size = sf->size;
 
         //add size of current node to the accumulator
-		acc += sizeof(sf->desc) + sizeof(sf->size) + sizeof(sf->count);
+		acc += 4 + sizeof(sf->size) + sizeof(sf->count);
 		acc += count * size;
 
 		if(os)			//Are we writing or calculating the size of this block?
@@ -637,6 +705,9 @@ static int savestate_WriteChunk(std::ostream* os, int type, SFORMAT *sf)
 	return (bsize+8);
 }
 
+//TODO TODO TODO TODO TODO TODO TODO 
+// - this is retarded. why not write placeholders for size and then write directly to the stream
+//and then go back and fill them in
 static void savestate_WriteChunk(std::ostream* os, int type, void (*saveproc)(std::ostream* os))
 {
 	//get the size
@@ -720,8 +791,6 @@ bool savestate_save (const char *file_name)
 	} else return false;
 }
 
-//u8 GPU_screen[4*256*192];
-
 static void writechunks(std::ostream* os) {
 	savestate_WriteChunk(os,1,SF_ARM9);
 	savestate_WriteChunk(os,2,SF_ARM7);
@@ -734,6 +803,8 @@ static void writechunks(std::ostream* os) {
 	savestate_WriteChunk(os,8,spu_savestate);
 	savestate_WriteChunk(os,90,SF_GFX3D);
 	savestate_WriteChunk(os,91,gfx3d_savestate);
+	savestate_WriteChunk(os,100,SF_MOVIE);
+	savestate_WriteChunk(os,101,mov_savestate);
 	savestate_WriteChunk(os,0xFFFFFFFF,(SFORMAT*)0);
 }
 
@@ -751,15 +822,17 @@ static bool ReadStateChunks(std::istream* is, s32 totalsize)
 		{
 			case 1: if(!ReadStateChunk(is,SF_ARM9,size)) ret=false; break;
 			case 2: if(!ReadStateChunk(is,SF_ARM7,size)) ret=false; break;
-			case 3: if(!cp15_loadstate(is)) ret=false; break;
+			case 3: if(!cp15_loadstate(is,size)) ret=false; break;
 			case 4: if(!ReadStateChunk(is,SF_MEM,size)) ret=false; break;
 			case 5: if(!ReadStateChunk(is,SF_NDS,size)) ret=false; break;
 			case 60: if(!ReadStateChunk(is,SF_MMU,size)) ret=false; break;
-			case 61: if(!mmu_loadstate(is)) ret=false; break;
-			case 7: if(!gpu_loadstate(is)) ret=false; break;
-			case 8: if(!spu_loadstate(is)) ret=false; break;
+			case 61: if(!mmu_loadstate(is,size)) ret=false; break;
+			case 7: if(!gpu_loadstate(is,size)) ret=false; break;
+			case 8: if(!spu_loadstate(is,size)) ret=false; break;
 			case 90: if(!ReadStateChunk(is,SF_GFX3D,size)) ret=false; break;
-			case 91: if(!gfx3d_loadstate(is)) ret=false; break;
+			case 91: if(!gfx3d_loadstate(is,size)) ret=false; break;
+			case 100: if(!ReadStateChunk(is,SF_MOVIE, size)) ret=false; break;
+			case 101: if(!mov_loadstate(is, size)) ret=false; break;
 			default:
 				ret=false;
 				break;
@@ -791,6 +864,7 @@ static void loadstate()
 
 static bool savestate_load(std::istream* is)
 {
+	SAV_silent_fail_flag = false;
 	char header[16];
 	is->read(header,16);
 	if(is->fail() || memcmp(header,magic,16))
@@ -832,7 +906,10 @@ static bool savestate_load(std::istream* is)
 	//while the series of resets below should work,
 	//we are testing the robustness of the savestate system with this full reset.
 	//the full reset wipes more things, so we can make sure that they are being restored correctly
+	extern bool _HACK_DONT_STOPMOVIE;
+	_HACK_DONT_STOPMOVIE = true;
 	NDS_Reset();
+	_HACK_DONT_STOPMOVIE = false;
 
 	//reset some options to their old defaults which werent saved
 	nds.debugConsole = FALSE;
@@ -846,13 +923,23 @@ static bool savestate_load(std::istream* is)
 	memorystream mstemp(&buf);
 	bool x = ReadStateChunks(&mstemp,(s32)len);
 
+	if(!x && !SAV_silent_fail_flag)
+	{
+		printf("Error loading savestate. It failed halfway through;\nSince there is no savestate backup system, your current game session is wrecked");
+#ifdef _MSC_VER
+		//HACK! we really need a better way to handle this kind of feedback
+		MessageBox(0,"Error loading savestate. It failed halfway through;\nSince there is no savestate backup system, your current game session is wrecked",0,0);
+#endif
+		return false;
+	}
+
 	loadstate();
 
 	if((nds.debugConsole!=0) != CommonSettings.DebugConsole) {
 		printf("WARNING: forcing console debug mode to: debugmode=%s\n",nds.debugConsole?"TRUE":"FALSE");
 	}
 
-	return x;
+	return true;
 }
 
 bool savestate_load(const char *file_name)
