@@ -21,10 +21,6 @@
 #ifndef _SRAM_H
 #define _SRAM_H
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 #include "types.h"
 
 #define SRAM_ADDRESS	0x0A000000
@@ -37,7 +33,23 @@ typedef struct
   char date[40];
 } savestates_t;
 
-savestates_t savestates[NB_STATES];
+
+struct SFORMAT
+{
+	//a string description of the element
+	const char *desc;
+
+	//the size of each element
+	u32 size;
+
+	//the number of each element
+	u32 count;
+
+	//a void* to the data or a void** to the data
+	void *v;
+};
+
+extern savestates_t savestates[NB_STATES];
 
 void clear_savestates();
 void scan_savestates();
@@ -46,14 +58,10 @@ void sram_write (u32 address, u8 value);
 int sram_load (const char *file_name);
 int sram_save (const char *file_name);
 
-int savestate_load (const char *file_name);
-int savestate_save (const char *file_name);
+bool savestate_load (const char *file_name);
+bool savestate_save (const char *file_name);
 
 void savestate_slot(int num);
 void loadstate_slot(int num);
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif
