@@ -3,8 +3,14 @@
 
 #include "common.h"
 
+enum TexCache_TexFormat
+{
+	TexFormat_32bpp,
+	TexFormat_15bpp
+};
+
 #define MAX_TEXTURE 500
-#ifdef SSE2
+#ifndef NOSSE2
 struct ALIGN(16) TextureCache
 #else
 struct ALIGN(8) TextureCache
@@ -30,16 +36,19 @@ struct ALIGN(8) TextureCache
 
 };
 
-extern TextureCache	texcache[MAX_TEXTURE+1];
+extern TextureCache	*texcache;
 
 extern void (*TexCache_BindTexture)(u32 texnum);
 extern void (*TexCache_BindTextureData)(u32 texnum, u8* data);
 
 void TexCache_Reset();
-void TexCache_SetTexture(unsigned int format, unsigned int texpal);
+
+template<TexCache_TexFormat format>
+void TexCache_SetTexture(u32 format, u32 texpal);
+
 void TexCache_Invalidate();
 
-extern u8 TexCache_texMAP[1024*2048*4]; 
+extern u8 *TexCache_texMAP;
 TextureCache* TexCache_Curr();
 
 #endif
