@@ -24,7 +24,8 @@
 
 #include "armcpu.h"
 
-extern u32 (* FASTCALL thumb_instructions_set[1024])(armcpu_t * cpu);
+extern u32 (FASTCALL* thumb_instructions_set_0[1024])();
+extern u32 (FASTCALL* thumb_instructions_set_1[1024])();
 
 #endif
  
