@@ -1,4 +1,5 @@
 /*  Copyright (C) 2005 Guillaume Duhamel
+	Copyright (C) 2008-2009 DeSmuME team
 
     This file is part of DeSmuME
 
@@ -14,71 +15,21 @@
 
     You should have received a copy of the GNU General Public License
     along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 #ifndef TYPES_HPP
 #define TYPES_HPP
 
-//--------------
-//configuration
-#define DEVELOPER
-//--------------
-
-#define DESMUME_NAME "DeSmuME"
-
-#ifdef _WIN64
-#define DESMUME_PLATFORM_STRING " x64"
-#else
-#ifdef _WIN32
-#define DESMUME_PLATFORM_STRING " x86"
-#else
-#define DESMUME_PLATFORM_STRING ""
-#endif
+//todo - everyone will want to support this eventually, i suppose
+#ifdef _MSC_VER
+#include "config.h"
 #endif
 
-#ifdef SSE2
-#define DESMUME_CPUEXT_STRING " SSE2"
-#else
-#define DESMUME_CPUEXT_STRING ""
+#ifndef _MSC_VER
+#define NOSSE2
 #endif
 
-//#ifdef DEVELOPER
-//#define DESMUME_FEATURE_STRING " dev+"
-//#else
-#define DESMUME_FEATURE_STRING ""
-//#endif
-
-#ifdef DEBUG
-#define DESMUME_SUBVERSION_STRING " debug"
-#else
-#ifdef RELEASE
-#define DESMUME_SUBVERSION_STRING ""
-#else
-#define DESMUME_SUBVERSION_STRING " prerelease"
-#endif
-#endif
-
-#ifdef __INTEL_COMPILER
-#define DESMUME_COMPILER " (Intel) "
-#define DESMUME_COMPILER_DETAIL " (Intel) "
-#elif defined(_MSC_VER)
-#define DESMUME_COMPILER ""
-#define DESMUME_COMPILER_DETAIL " msvc " _Py_STRINGIZE(_MSC_VER)
-#define _Py_STRINGIZE(X) _Py_STRINGIZE1((X))
-#define _Py_STRINGIZE1(X) _Py_STRINGIZE2 ## X
-#define _Py_STRINGIZE2(X) #X
-//re: http://72.14.203.104/search?q=cache:HG-okth5NGkJ:mail.python.org/pipermail/python-checkins/2002-November/030704.html+_msc_ver+compiler+version+string&hl=en&gl=us&ct=clnk&cd=5
-#else
-// TODO: make for others compilers
-#define DESMUME_COMPILER ""
-#define DESMUME_COMPILER_DETAIL ""
-#endif
-
-#define DESMUME_VERSION_NUMERIC 90200
-#define DESMUME_VERSION_STRING " " "0.9.2" DESMUME_FEATURE_STRING DESMUME_PLATFORM_STRING DESMUME_CPUEXT_STRING DESMUME_SUBVERSION_STRING DESMUME_COMPILER
-#define DESMUME_NAME_AND_VERSION " " DESMUME_NAME DESMUME_VERSION_STRING
-
 #ifdef _WIN32
 #define strcasecmp(x,y) _stricmp(x,y)
 #else
@@ -112,6 +63,12 @@
 #endif
 #endif
 
+#ifdef _MSC_VER
+#define _CDECL_ __cdecl
+#else
+#define _CDECL_
+#endif
+
 #ifndef INLINE
 #if defined(_MSC_VER) || defined(__INTEL_COMPILER)
 #define INLINE _inline
@@ -123,36 +80,50 @@
 #ifndef FORCEINLINE
 #if defined(_MSC_VER) || defined(__INTEL_COMPILER)
 #define FORCEINLINE __forceinline
+#define MSC_FORCEINLINE __forceinline
 #else
 #define FORCEINLINE INLINE
+#define MSC_FORCEINLINE
 #endif
 #endif
 
+//#ifndef _PREFETCH
+//#if (defined(_MSC_VER) || defined(__INTEL_COMPILER)) && !defined(NOSSE2)
+//#include <xmmintrin.h>
+//#include <intrin.h>
+//#define _PREFETCH(X) _mm_prefetch((char*)(X),_MM_HINT_T0);
+//#define _PREFETCHNTA(X) _mm_prefetch((char*)(X),_MM_HINT_NTA);
+//#else
+#define _PREFETCH(X) {}
+#define _PREFETCHNTA(X) {}
+//#endif
+//#endif
+
+
+
 #if defined(__LP64__)
 typedef unsigned char u8;
 typedef unsigned short u16;
 typedef unsigned int u32;
 typedef unsigned long u64;
-typedef unsigned long pointer;
 
 typedef signed char s8;
 typedef signed short s16;
 typedef signed int s32;
-typedef signed long s64;
+typedef signed long long s64;
 #else
 typedef unsigned char u8;
 typedef unsigned short u16;
-typedef unsigned long u32;
+typedef unsigned int u32;
 #if defined(_MSC_VER) || defined(__INTEL_COMPILER)
 typedef unsigned __int64 u64;
 #else
 typedef unsigned long long u64;
 #endif
-typedef unsigned long pointer;
 
 typedef signed char s8;
 typedef signed short s16;
-typedef signed long s32;
+typedef signed int s32;
 #if defined(_MSC_VER) || defined(__INTEL_COMPILER)
 typedef __int64 s64;
 #else
@@ -298,6 +269,24 @@ inline double u64_to_double(u64 u) {
 	return fuxor.b;
 }
 
+inline u32 float_to_u32(float f) {
+	union {
+		u32 a;
+		float b;
+	} fuxor;
+	fuxor.b = f;
+	return fuxor.a;
+}
+
+inline float u32_to_float(u32 u) {
+	union {
+		u32 a;
+		float b;
+	} fuxor;
+	fuxor.a = u;
+	return fuxor.b;
+}
+
 
 ///stores a 32bit value into the provided byte array in guaranteed little endian form
 inline void en32lsb(u8 *buf, u32 morp)
