@@ -449,16 +449,13 @@ static void decrypt_arm9(u32 cardheader_gamecode, unsigned char *data)
 }
 
 
-void DecryptSecureArea(u8 *romdata, long romlen)
+bool DecryptSecureArea(u8 *romdata, long romlen)
 {
 	//this looks like it will only work on little endian hosts
 	Header* header = (Header*)romdata;
 
 	int romType = DetectRomType(*header,(char*)romdata);
 
-	unsigned char data[0x4000];
-	memcpy(data,romdata+0x4000,0x4000);
-
 	/*bool do_decrypt = (endecrypt_option == 'd');
 	bool do_encrypt = (endecrypt_option == 'e') || (endecrypt_option == 'E');
 	unsigned int rounds_offsets = (endecrypt_option == 'E') ? 0x2000 : 0x1600;
@@ -468,6 +465,9 @@ void DecryptSecureArea(u8 *romdata, long romlen)
 	unsigned int sbox_offsets = 0x2800;
 #endif
 
+	if(romType == ROMTYPE_INVALID)
+		return false;
+
 	// check if ROM is already encrypted
 	if (romType == ROMTYPE_NDSDUMPED)
 	{
@@ -475,6 +475,8 @@ void DecryptSecureArea(u8 *romdata, long romlen)
 	}
 	else if (romType >= ROMTYPE_ENCRSECURE)		// includes ROMTYPE_MASKROM
 	{
+		unsigned char data[0x4000];
+		memcpy(data,romdata+0x4000,0x4000);
 
 		decrypt_arm9(*(u32 *)header->gamecode, data);
 
@@ -499,4 +501,6 @@ void DecryptSecureArea(u8 *romdata, long romlen)
 	{
 		printf("File doesn't appear to have a secure area.\n");
 	}
+
+	return true;
 }
