@@ -192,6 +192,13 @@ private:
 	unsigned char data[256];
 } Base64Table;
 
+std::string u32ToHexString(u32 val)
+{
+	char temp[16];
+	sprintf(temp,"%08X",val);
+	return temp;
+}
+
 ///Converts the provided data to a string in a standard, user-friendly, round-trippable format
 std::string BytesToString(const void* data, int len)
 {
@@ -582,7 +589,7 @@ namespace UtfConverter
         }
         else if (sizeof(wchar_t) == 4)
         {
-            wchar_t* widestringnative = new wchar_t[widesize];
+            wchar_t* widestringnative = new wchar_t[widesize+1];
             const UTF8* sourcestart = reinterpret_cast<const UTF8*>(utf8string.c_str());
             const UTF8* sourceend = sourcestart + widesize;
             UTF32* targetstart = reinterpret_cast<UTF32*>(widestringnative);
