@@ -22,18 +22,12 @@
 #include "wifi.h"
 #include "armcpu.h"
 #include "NDSSystem.h"
+#include "debug.h"
 
 #ifdef EXPERIMENTAL_WIFI
 
 wifimac_t wifiMac ;
 
-socket_t 	WIFI_Host_OpenChannel(u8 num) ;
-void 		WIFI_Host_CloseChannel(socket_t sock) ;
-void 		WIFI_Host_SendData(socket_t sock, u8 channel, u8 *data, u16 length) ;
-u16			WIFI_Host_RecvData(socket_t sock, u8 *data, u16 maxLength) ;
-BOOL 		WIFI_Host_InitSystem(void) ;
-void 		WIFI_Host_ShutdownSystem(void) ;
-
 #endif
 
 /*******************************************************************************
@@ -192,6 +186,137 @@ FW_WFCProfile FW_WFCProfile3 = {"",
 
 #ifdef EXPERIMENTAL_WIFI
 
+/*******************************************************************************
+
+	Logging
+
+ *******************************************************************************/
+
+// 0: disable logging
+// 1: lowest logging, shows most important messages such as errors
+// 2: low logging, shows things such as warnings
+// 3: medium logging, for debugging, shows lots of stuff
+// 4: high logging, for debugging, shows almost everything, may slow down
+// 5: highest logging, for debugging, shows everything, may slow down a lot
+#define WIFI_LOGGING_LEVEL 3
+
+#if (WIFI_LOGGING_LEVEL >= 1)
+#define WIFI_LOG(level, ...) if(level <= WIFI_LOGGING_LEVEL) LOGC(8, "WIFI: "__VA_ARGS__);
+#else
+#define WIFI_LOG(level, ...)
+#endif
+
+/*******************************************************************************
+
+	Helpers
+
+ *******************************************************************************/
+
+INLINE u32 WIFI_alignedLen(u32 len)
+{
+	return ((len + 3) & ~3);
+}
+
+#ifdef WIN32
+static pcap_t *desmume_pcap_open(const char *source, int snaplen, int flags,
+		int read_timeout, char *errbuf)
+{
+	return PCAP::pcap_open(source, snaplen, flags, read_timeout, NULL, errbuf);
+}
+
+static int desmume_pcap_findalldevs(pcap_if_t **alldevs, char *errbuf)
+{
+	return PCAP::pcap_findalldevs_ex(PCAP_SRC_IF_STRING, NULL, alldevs, errbuf);
+}
+
+static void desmume_pcap_freealldevs(pcap_if_t *alldevs)
+{
+	return PCAP::pcap_freealldevs(alldevs);
+}
+
+static void desmume_pcap_close(pcap_t *p)
+{
+	return PCAP::pcap_close(p);
+}
+
+static int desmume_pcap_sendpacket(pcap_t *p, u_char *buf, int size)
+{
+	return PCAP::pcap_sendpacket(p, buf, size);
+}
+
+#else
+static pcap_t *desmume_pcap_open(const char *device, int snaplen, int promisc,
+		int to_ms, char *errbuf)
+{
+	return pcap_open_live(device, snaplen, promisc, to_ms, errbuf);
+}
+
+static int desmume_pcap_findalldevs(pcap_if_t **alldevs, char *errbuf)
+{
+	return pcap_findalldevs(alldevs, errbuf);
+}
+
+static void desmume_pcap_freealldevs(pcap_if_t *alldevs)
+{
+	return pcap_freealldevs(alldevs);
+}
+
+static void desmume_pcap_close(pcap_t *p)
+{
+	return pcap_close(p);
+}
+
+static int desmume_pcap_sendpacket(pcap_t *p, u_char *buf, int size)
+{
+	return pcap_sendpacket(p, buf, size);
+}
+#endif
+
+/*******************************************************************************
+
+	CRC32 (http://www.codeproject.com/KB/recipes/crc32_large.aspx)
+
+ *******************************************************************************/
+
+u32 WIFI_CRC32Table[256];
+
+static u32 reflect(u32 ref, char ch)
+{
+    u32 value = 0;
+
+    for(int i = 1; i < (ch + 1); i++)
+    {
+        if (ref & 1)
+            value |= 1 << (ch - i);
+        ref >>= 1;
+    } 
+	
+	return value;
+}
+
+static u32 WIFI_getCRC32(u8 *data, int len)
+{
+	u32 crc = 0xFFFFFFFF;
+
+	while(len--)
+        crc = (crc >> 8) ^ WIFI_CRC32Table[(crc & 0xFF) ^ *data++];
+
+	return (crc ^ 0xFFFFFFFF);
+}
+
+static void WIFI_initCRC32Table()
+{
+	u32 polynomial = 0x04C11DB7;
+
+	for(int i = 0; i < 0x100; i++)
+    {
+        WIFI_CRC32Table[i] = reflect(i, 8) << 24;
+        for(int j = 0; j < 8; j++)
+            WIFI_CRC32Table[i] = (WIFI_CRC32Table[i] << 1) ^ (WIFI_CRC32Table[i] & (1 << 31) ? polynomial : 0);
+        WIFI_CRC32Table[i] = reflect(WIFI_CRC32Table[i],  32);
+    }
+}
+
 /*******************************************************************************
 
 	RF-Chip
@@ -318,10 +443,6 @@ void WIFI_setRF_DATA(wifimac_t *wifi, u16 val, u8 part)
 								if (channelFreqN<0x00A2E8BA) return ;
 								/* substract base frequency (channel 1) */
 								channelFreqN -= 0x00A2E8BA ;
-								/* every channel is now ~3813000 steps further */
-								WIFI_Host_CloseChannel(wifi->udpSocket) ;
-								wifi->channel = (wifi->udpSocket,channelFreqN / 3813000)+1 ;
-								WIFI_Host_OpenChannel(wifi->channel) ;
 							}
 							return ;
 						case 13:
@@ -402,6 +523,8 @@ static void WIFI_triggerIRQMask(wifimac_t *wifi, u16 mask)
 	oResult = wifi->IE.val & wifi->IF.val ;
 	wifi->IF.val = wifi->IF.val | (mask & ~0x0400) ;
 	nResult = wifi->IE.val & wifi->IF.val ;
+	//printf("wifi: trigger irq: ie=%04X, if=%04X, mask=%08X, oldresult=%04X, newresult=%04X\n", 
+	//	wifi->IE.val, wifi->IF.val, mask, oResult, nResult);
 	if (!oResult && nResult)
 	{
 		NDS_makeARM7Int(24) ;   /* cascade it via arm7 wifi irq */
@@ -413,41 +536,24 @@ static void WIFI_triggerIRQ(wifimac_t *wifi, u8 irq)
 	WIFI_triggerIRQMask(wifi,1<<irq) ;
 }
 
-/*int WIFI_uSleep(socket_t sock, long usec)
-{
-    struct timeval tv;
-    fd_set dummy;
-    FD_ZERO(&dummy);
-    FD_SET(sock, &dummy);
-    tv.tv_sec = (usec / 1000000);
-    tv.tv_usec = (usec % 1000000);
-    return select(0, 0, 0, &dummy, &tv);
-}*/
-
 
 void WIFI_Init(wifimac_t *wifi)
 {
+	WIFI_initCRC32Table();
+
 	WIFI_resetRF(&wifi->RF) ;
 	wifi->netEnabled = false;
 	if(driver->WIFI_Host_InitSystem())
 	{
 		wifi->netEnabled = true;
-		wifi->udpSocket = WIFI_Host_OpenChannel(1) ;
 	}
 	wifi->powerOn = FALSE;
 	wifi->powerOnPending = FALSE;
+	
+	wifi->rfStatus = 0x0000;
+	wifi->rfPins = 0x0004;
 }
 
-/*void WIFI_Thread(wifimac_t *wifi)
-{
-	for(;;)
-	{
-		WIFI_usTrigger(wifi);
-		WIFI_SoftAP_usTrigger(wifi);
-		WIFI_uSleep(wifi->udpSocket, 1);
-	}
-}*/
-
 static void WIFI_RXPutWord(wifimac_t *wifi,u16 val)
 {
 //	printf("wifi: rx circbuf write attempt: rxcnt=%04X, rxread=%04X, rxwrite=%04X\n", 
@@ -477,33 +583,46 @@ static void WIFI_RXPutWord(wifimac_t *wifi,u16 val)
 
 static void WIFI_TXStart(wifimac_t *wifi,u8 slot)
 {
-	//printf("wifi: send attempt on slot %i, txcnt=%04X, slotcnt=%04X\n", 
-	//	slot, wifi->TXCnt, wifi->TXSlot[slot]);
+	WIFI_LOG(3, "TX slot %i trying to send a packet: TXCnt = %04X, TXBufLoc = %04X\n", 
+		slot, wifi->TXCnt, wifi->TXSlot[slot]);
+
 	if (wifi->TXSlot[slot] & 0x8000)	/* is slot enabled? */
 	{
-		u16 txLen ;
+		u16 txLen;
 		/* the address has to be somewhere in the circular buffer, so drop the other bits */
-		u16 address = (wifi->TXSlot[slot] & 0x7FFF) ;
+		u16 address = (wifi->TXSlot[slot] & 0x0FFF);
 		/* is there even enough space for the header (6 hwords) in the tx buffer? */
-		if (address > 0x1000-6) return ; 
+		if (address > 0x1000-6)
+		{
+			WIFI_LOG(1, "TX slot %i trying to send a packet overflowing from the TX buffer (address %04X). Attempt ignored.\n", 
+				slot, (address << 1));
+			return;
+		}
 
 		/* 12 byte header TX Header: http://www.akkit.org/info/dswifi.htm#FmtTx */
-		txLen = /*ntohs*/(wifi->circularBuffer[address+5]) ;
+		txLen = wifi->circularBuffer[address+5];
 		/* zero length */
-		if (txLen==0) return ;
+		if (txLen == 0)
+		{
+			WIFI_LOG(1, "TX slot %i trying to send a packet with length field set to zero. Attempt ignored.\n", 
+				slot);
+			return;
+		}
 		/* unsupported txRate */
-		switch (/*ntohs*/(wifi->circularBuffer[address+4] & 0xFF))
+		switch (wifi->circularBuffer[address+4] & 0xFF)
 		{
 			case 10: /* 1 mbit */
 			case 20: /* 2 mbit */
-				break ;
+				break;
 			default: /* other rates */
-				return ;
+				WIFI_LOG(1, "TX slot %i trying to send a packet with transfer rate field set to an invalid value of %i. Attempt ignored.\n", 
+					slot, wifi->circularBuffer[address+4] & 0xFF);
+				return;
 		}
 
 		/* FIXME: calculate FCS */
 
-		WIFI_triggerIRQ(wifi,/*WIFI_IRQ_SENDSTART*/7) ;
+		WIFI_triggerIRQ(wifi, WIFI_IRQ_SENDSTART) ;
 
 		if(slot > wifi->txCurSlot)
 			wifi->txCurSlot = slot;
@@ -513,12 +632,16 @@ static void WIFI_TXStart(wifimac_t *wifi,u8 slot)
 		wifi->txSlotLen[slot] = txLen;
 		wifi->txSlotRemainingBytes[slot] = (txLen + 12);
 
-	/*	WIFI_Host_SendData(wifi->udpSocket,wifi->channel,(u8 *)&wifi->circularBuffer[address],txLen) ;
+		wifi->rfStatus = 0x0003;
+		wifi->rfPins = 0x0046;
+
+#if 0
 		WIFI_SoftAP_RecvPacketFromDS(wifi, (u8*)&wifi->circularBuffer[address+6], txLen);
-		WIFI_triggerIRQ(wifi,/*WIFI_IRQ_SENDCOMPLETE*-/1) ;
+		WIFI_triggerIRQ(wifi, WIFI_IRQ_SENDCOMPLETE) ;
 
 		wifi->circularBuffer[address] = 0x0001;
-		wifi->circularBuffer[address+4] &= 0x00FF;*/
+		wifi->circularBuffer[address+4] &= 0x00FF;
+#endif
 	}
 } 
 
@@ -526,7 +649,9 @@ void WIFI_write16(wifimac_t *wifi,u32 address, u16 val)
 {
 	BOOL action = FALSE ;
 	if (!(MMU_read32(ARMCPU_ARM7,REG_PWRCNT) & 0x0002)) return ;		/* access to wifi hardware was disabled */
-	if ((address & 0xFF800000) != 0x04800000) return ;      /* error: the address does not describe a wiifi register */
+	if ((address & 0xFF800000) != 0x04800000) return ;      /* error: the address does not describe a wifi register */
+
+	WIFI_LOG(5, "Write at address %08X, %04X\n", address, val);
 
 	/* the first 0x1000 bytes are mirrored at +0x1000,+0x2000,+0x3000,+06000,+0x7000 */
 	/* only the first mirror causes an special action */
@@ -537,8 +662,8 @@ void WIFI_write16(wifimac_t *wifi,u32 address, u16 val)
 	{
 		/* access to the circular buffer */
 		address &= 0x1FFF ;
-		//printf("wifi: circbuf write at %04X, %04X, readcsr=%04X, wrcsr=%04X(%04X)\n", 
-		//	address, val, wifi->RXReadCursor, wifi->RXHWWriteCursorReg, wifi->RXHWWriteCursor);
+	//	printf("wifi: circbuf write at %04X, %04X, readcsr=%04X, wrcsr=%04X(%04X)\n", 
+	//		address, val, wifi->RXReadCursor << 1, wifi->RXHWWriteCursorReg << 1, wifi->RXHWWriteCursor << 1);
 	/*	if((address == 0x0BFE) && (val == 0x061E))
 		{
 			extern void emu_halt();
@@ -567,6 +692,13 @@ void WIFI_write16(wifimac_t *wifi,u32 address, u16 val)
 				/* this bit does not save */
 				val &= ~0x4000 ;
 			}
+
+			if(val & 0x0001)
+			{
+				wifi->rfStatus = 0x0009;
+				wifi->rfPins = 0x0004;
+			}
+
 			wifi->macMode = val ;
 			break ;
 		case REG_WIFI_WEP:
@@ -602,14 +734,15 @@ void WIFI_write16(wifimac_t *wifi,u32 address, u16 val)
 		case REG_WIFI_FORCEPS:
 			if((val & 0x8000) && (!wifi->powerOnPending))
 			{
-				BOOL newPower = ((val & 0x0001)?FALSE:TRUE);
+			/*	BOOL newPower = ((val & 0x0001)?FALSE:TRUE);
 				if(newPower != wifi->powerOn)
 				{
 					if(!newPower)
 						wifi->powerOn = FALSE;
 					else
 						wifi->powerOnPending = TRUE;
-				}
+				}*/
+				wifi->powerOn = ((val & 0x0001) ? FALSE : TRUE);
 			}
 			break;
 		case REG_WIFI_POWERACK:
@@ -649,7 +782,7 @@ void WIFI_write16(wifimac_t *wifi,u32 address, u16 val)
 			wifi->CircBufReadAddress = (val & 0x1FFE);
 			break ;
 		case REG_WIFI_RXREADCSR:
-			//printf("wifi: write readcsr %04X\n", val);
+			//printf("wifi: write readcsr: %04X\n", val << 1);
 			wifi->RXReadCursor = val ;
 			break ;
 		case REG_WIFI_CIRCBUFWADR:
@@ -684,17 +817,19 @@ void WIFI_write16(wifimac_t *wifi,u32 address, u16 val)
 			wifi->TXSlot[(address - REG_WIFI_TXLOC1) >> 2] = val ;
 			break ;
 		case REG_WIFI_TXOPT:
-		/*	if (val == 0xFFFF)
+#if 0
+			if (val == 0xFFFF)
 			{
-				/* reset TX logic *-/
-				/* CHECKME *-/
+				/* reset TX logic */
+				/* CHECKME */
             //    wifi->TXSlot[0] = 0 ; wifi->TXSlot[1] = 0 ; wifi->TXSlot[2] = 0 ;
                 wifi->TXOpt = 0 ;
                 wifi->TXCnt = 0 ;
 			} else
 			{
 				wifi->TXOpt = val ;
-			}*/
+			}
+#endif
 			wifi->TXCnt &= ~val;
 			break ;
 		case REG_WIFI_TXCNT:
@@ -765,6 +900,9 @@ void WIFI_write16(wifimac_t *wifi,u32 address, u16 val)
 		//case 0x40:
 		//	printf("wifi power reg %03X write %04X\n", address, val);
 		//	break;
+		case 0xD0:
+		//	printf("wifi: rxfilter=%04X\n", val);
+			break;
 		default:
 		//	printf("wifi: write unhandled reg %03X, %04X\n", address, val);
 		//	val = 0 ;       /* not handled yet */
@@ -773,13 +911,15 @@ void WIFI_write16(wifimac_t *wifi,u32 address, u16 val)
 
 	wifi->ioMem[address >> 1] = val;
 }
-
+//int packet_arrived=0;
 u16 WIFI_read16(wifimac_t *wifi,u32 address)
 {
 	BOOL action = FALSE ;
 	u16 temp ;
 	if (!(MMU_read32(ARMCPU_ARM7,REG_PWRCNT) & 0x0002)) return 0 ;		/* access to wifi hardware was disabled */
-	if ((address & 0xFF800000) != 0x04800000) return 0 ;      /* error: the address does not describe a wiifi register */
+	if ((address & 0xFF800000) != 0x04800000) return 0 ;      /* error: the address does not describe a wifi register */
+
+	WIFI_LOG(5, "Read at address %08X\n", address);
 
 	/* the first 0x1000 bytes are mirrored at +0x1000,+0x2000,+0x3000,+06000,+0x7000 */
 	/* only the first mirror causes an special action */
@@ -789,9 +929,10 @@ u16 WIFI_read16(wifimac_t *wifi,u32 address)
 	if (((address & 0x00007000) >= 0x00004000) && ((address & 0x00007000) < 0x00006000))
 	{
 		/* access to the circular buffer */
-		//printf("wifi: circbuf read at %04X, readcsr=%04X, wrcsr=%04X(%04X)\n", 
-		//	(address & 0x1FFF), wifi->RXReadCursor, wifi->RXHWWriteCursorReg, wifi->RXHWWriteCursor);
-	/*	if((address == 0x04804C38) && 
+	//	printf("wifi: circbuf read at %04X, %04X, readcsr=%04X, wrcsr=%04X(%04X)\n", 
+		//	address, wifi->circularBuffer[(address & 0x1FFF) >> 1], wifi->RXReadCursor << 1, wifi->RXHWWriteCursorReg << 1, wifi->RXHWWriteCursor << 1);
+		/*if(((address == 0x04804C38) || 
+			(address == 0x04804C04)) && 
 			(wifi->circularBuffer[(address & 0x1FFF) >> 1] != 0x0000) && 
 			(wifi->circularBuffer[(address & 0x1FFF) >> 1] != 0x5A5A) &&
 			(wifi->circularBuffer[(address & 0x1FFF) >> 1] != 0xA5A5))
@@ -799,8 +940,21 @@ u16 WIFI_read16(wifimac_t *wifi,u32 address)
 			extern void emu_halt();
 			emu_halt();
 		}*/
-	//	if(address == 0x04804C3E)
-	//		return 0x0014;
+
+		// hax
+	/*	if(packet_arrived)
+		{
+			if(address == 0x04804C38)
+				return 0x0010;
+			if(address == 0x04804C3E)
+				return 0x0014;
+
+			if(address == 0x048047D0)
+				return 0x0010;
+			if(address == 0x048047D6)
+				return 0x0014;
+		}*/
+
         return wifi->circularBuffer[(address & 0x1FFF) >> 1] ;
 	}
 	if (!(address & 0x00007000)) action = TRUE ;
@@ -890,9 +1044,11 @@ u16 WIFI_read16(wifimac_t *wifi,u32 address)
 		case REG_WIFI_RXHWWRITECSR:
 			//printf("wifi: read writecsr (%04X)\n", wifi->RXHWWriteCursorReg);
 			return wifi->RXHWWriteCursorReg;
+			//return wifi->RXReadCursor;
 		case REG_WIFI_RXREADCSR:
 			//printf("wifi: read readcsr (%04X)\n", wifi->RXReadCursor);
 			return wifi->RXReadCursor;
+			//return wifi->RXHWWriteCursorReg;
 		case REG_WIFI_RXBUF_COUNT:
 			return wifi->RXBufCount ;
 		case REG_WIFI_TXREQ_READ:
@@ -924,11 +1080,14 @@ u16 WIFI_read16(wifimac_t *wifi,u32 address)
 		case REG_WIFI_AID_HIGH:
 			return wifi->aid ;
 		case 0x214:
-			//printf("wifi: read reg 0x0214\n");
+			//printf("wifi: read rfstatus (%04X)\n", wifi->rfStatus);
 			return 0x0009;
+			//return wifi->rfStatus;
 		case 0x19C:
-			//luigi, please pick something to return from here
-			return 0;
+			//assert(false); //luigi, please pick something to return from here
+			//printf("wifi: read rfpins (%04X)\n", wifi->rfPins);
+			//return wifi->rfPins;
+			return 0x0004;
 		default:
 		//	printf("wifi: read unhandled reg %03X\n", address);
 			return wifi->ioMem[address >> 1];
@@ -955,26 +1114,9 @@ void WIFI_usTrigger(wifimac_t *wifi)
 	}
 	if ((wifi->ucmpEnable) && (wifi->ucmp == wifi->usec))
 	{
-			WIFI_triggerIRQ(wifi,/*WIFI_IRQ_TIMEBEACON*/14) ;
+			WIFI_triggerIRQ(wifi, WIFI_IRQ_TIMEBEACON) ;
 	}
-	/* receive check, given a 2 mbit connection, 2 bits per usec can be transfered. */
-	/* for a packet of 32 Bytes, at least 128 usec passed, we will use the 32 byte accuracy to reduce load */
-	/*if (!(wifi->RXCheckCounter++ & 0x7F))
-	{
-		/* check if data arrived in the meantime *-/
-		rcvSize = WIFI_Host_RecvData(wifi->udpSocket,dataBuffer,0x2000) ;
-		if (rcvSize)
-		{
-			u16 i ;
-			/* process data, put it into mac memory *-/
-			WIFI_triggerIRQ(wifi,/*WIFI_IRQ_RECVSTART*-/6) ;
-			for (i=0;i<(rcvSize+1) << 1;i++)
-			{
-				WIFI_RXPutWord(wifi,((u16 *)dataBuffer)[i]) ;
-			}
-			WIFI_triggerIRQ(wifi,/*WIFI_IRQ_RECVCOMPLETE*-/0) ;
-		}
-	}*/
+
 	if((wifi->usec & 3) == 0)
 	{
 		int slot = wifi->txCurSlot;
@@ -996,7 +1138,15 @@ void WIFI_usTrigger(wifimac_t *wifi)
 
 				wifi->TXStat = (0x0001 | (slot << 12));
 
-				WIFI_triggerIRQ(wifi, 1);
+				WIFI_triggerIRQ(wifi, WIFI_IRQ_SENDCOMPLETE);
+				
+				//wifi->rfStatus = 0x0001;
+				//wifi->rfPins = 0x0084;
+				wifi->rfStatus = 0x0009;
+				wifi->rfPins = 0x0004;
+
+				WIFI_LOG(3, "TX slot %i finished sending its packet. Next is slot %i. TXStat = %04X\n", 
+					slot, wifi->txCurSlot, wifi->TXStat);
 			}
 		}
 	}
@@ -1008,11 +1158,9 @@ void WIFI_usTrigger(wifimac_t *wifi)
 
  *******************************************************************************/
 
-u32 SoftAP_CRC32Table[256];
-
 u8 SoftAP_MACAddr[6] = {0x00, 0xF0, 0x1A, 0x2B, 0x3C, 0x4D};
 
-u8 SoftAP_Beacon[58] = {
+u8 SoftAP_Beacon[] = {
 	/* 802.11 header */
 	0x80, 0x00,											// Frame control
 	0x00, 0x00,											// Duration ID
@@ -1033,7 +1181,7 @@ u8 SoftAP_Beacon[58] = {
 	0x00, 0x00, 0x00, 0x00
 };
 
-u8 SoftAP_ProbeResponse[52] = {
+u8 SoftAP_ProbeResponse[] = {
 	/* 802.11 header */
 	0x50, 0x00,											// Frame control
 	0x00, 0x00,											// Duration ID
@@ -1045,7 +1193,7 @@ u8 SoftAP_ProbeResponse[52] = {
 	/* Frame body */
 	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,		// Timestamp
 	0x64, 0x00,											// Beacon interval
-	0x0F, 0x00,											// Capablilty information
+	0x01, 0x00,											// Capablilty information
 	0x00, 0x06, 'S', 'o', 'f', 't', 'A', 'P',			// SSID
 	0x01, 0x02, 0x82, 0x84,								// Supported rates
 	
@@ -1053,7 +1201,7 @@ u8 SoftAP_ProbeResponse[52] = {
 	0x00, 0x00, 0x00, 0x00
 };
 
-u8 SoftAP_AuthFrame[34] = {
+u8 SoftAP_AuthFrame[] = {
 	/* 802.11 header */
 	0xB0, 0x00,											// Frame control
 	0x00, 0x00,											// Duration ID
@@ -1071,7 +1219,7 @@ u8 SoftAP_AuthFrame[34] = {
 	0x00, 0x00, 0x00, 0x00
 };
 
-u8 SoftAP_AssocResponse[38] = {
+u8 SoftAP_AssocResponse[] = {
 	/* 802.11 header */
 	0x10, 0x00,											// Frame control
 	0x00, 0x00,											// Duration ID
@@ -1090,33 +1238,6 @@ u8 SoftAP_AssocResponse[38] = {
 	0x00, 0x00, 0x00, 0x00
 };
 
-// http://www.codeproject.com/KB/recipes/crc32_large.aspx
-u32 reflect(u32 ref, char ch)
-{
-    u32 value = 0;
-
-    for(int i = 1; i < (ch + 1); i++)
-    {
-        if (ref & 1)
-            value |= 1 << (ch - i);
-        ref >>= 1;
-    } 
-	
-	return value;
-}
-
-// http://www.codeproject.com/KB/recipes/crc32_large.aspx
-u32 WIFI_SoftAP_GetCRC32(u8 *data, int len)
-{
-	u32 crc = 0xFFFFFFFF;
-
-	while(len--)
-        crc = (crc >> 8) ^ SoftAP_CRC32Table[(crc & 0xFF) ^ *data++];
-
-	return (crc ^ 0xFFFFFFFF);
-//	return 0x1D46B6B8;
-}
-
 //todo - make a class to wrap this
 //todo - zeromus - inspect memory leak safety of all this
 static pcap_if_t * WIFI_index_device(pcap_if_t *alldevs, int index) {
@@ -1137,47 +1258,67 @@ int WIFI_SoftAP_Init(wifimac_t *wifi)
 	wifi->SoftAP.curPacketSize = 0;
 	wifi->SoftAP.curPacketPos = 0;
 	wifi->SoftAP.curPacketSending = FALSE;
-
-	// http://www.codeproject.com/KB/recipes/crc32_large.aspx
-	u32 polynomial = 0x04C11DB7;
-
-	for(int i = 0; i < 0x100; i++)
-    {
-        SoftAP_CRC32Table[i] = reflect(i, 8) << 24;
-        for(int j = 0; j < 8; j++)
-            SoftAP_CRC32Table[i] = (SoftAP_CRC32Table[i] << 1) ^ (SoftAP_CRC32Table[i] & (1 << 31) ? polynomial : 0);
-        SoftAP_CRC32Table[i] = reflect(SoftAP_CRC32Table[i],  32);
-    }
 	
-	if(PCAP::pcap_findalldevs_ex(PCAP_SRC_IF_STRING, NULL, &alldevs, errbuf) == -1)
+	if(wifiMac.netEnabled)
 	{
-		printf("SoftAP: PCAP error with pcap_findalldevs_ex(): %s\n", errbuf);
-		return 0;
-	}
+		if(desmume_pcap_findalldevs(&alldevs, errbuf) == -1)
+		{
+			printf("SoftAP: PCAP error with pcap_findalldevs_ex(): %s\n", errbuf);
+			return 0;
+		}
 
-	wifi->SoftAP.bridge = PCAP::pcap_open(WIFI_index_device(alldevs,CommonSettings.wifiBridgeAdapterNum)->name, PACKET_SIZE, 0, 1, NULL, errbuf);
-	if(wifi->SoftAP.bridge == NULL)
-	{
-		printf("SoftAP: PCAP error with pcap_open(): %s\n", errbuf);
-		return 0;
-	}
+		wifi->SoftAP.bridge = desmume_pcap_open(WIFI_index_device(alldevs,CommonSettings.wifiBridgeAdapterNum)->name, PACKET_SIZE, 0, 1, errbuf);
+		if(wifi->SoftAP.bridge == NULL)
+		{
+			printf("SoftAP: PCAP error with pcap_open(): %s\n", errbuf);
+			return 0;
+		}
 
-	PCAP::pcap_freealldevs(alldevs);
+		desmume_pcap_freealldevs(alldevs);
+	}
 
 	return 1;
 }
 
 void WIFI_SoftAP_Shutdown(wifimac_t *wifi)
 {
-	PCAP::pcap_close(wifi->SoftAP.bridge);
+	if(wifiMac.netEnabled)
+	{
+	if(wifi->SoftAP.bridge != NULL)
+		desmume_pcap_close(wifi->SoftAP.bridge);
+	}
+
+	if(wifi->SoftAP.curPacket)
+		delete wifi->SoftAP.curPacket;
+}
+
+static void WIFI_SoftAP_MakeRXHeader(wifimac_t *wifi, u16 flags, u16 xferRate, u16 len, u8 maxRSSI, u8 minRSSI)
+{
+	*(u16*)&wifi->SoftAP.curPacket[0] = flags;
+
+	// Unknown (usually 0x0400)
+	wifi->SoftAP.curPacket[2] = 0x40;
+	wifi->SoftAP.curPacket[3] = 0x00;
+
+	// Time since last packet??? Random??? Left unchanged???
+	wifi->SoftAP.curPacket[4] = 0x01;
+	wifi->SoftAP.curPacket[5] = 0x00;
+
+	*(u16*)&wifi->SoftAP.curPacket[6] = xferRate;
+
+	*(u16*)&wifi->SoftAP.curPacket[8] = WIFI_alignedLen(len);
+
+	wifi->SoftAP.curPacket[10] = maxRSSI;
+	wifi->SoftAP.curPacket[11] = minRSSI;
 }
 
 void WIFI_SoftAP_RecvPacketFromDS(wifimac_t *wifi, u8 *packet, int len)
 {
-	int alignedLen = ((len+3) & (~3));
+	int alignedLen = WIFI_alignedLen(len);
 	u16 frameCtl = *(u16*)&packet[12];
 
-	//printf("wifi: packet arrived, len = %i (%i), frame ctl = %04X\n", len, alignedLen, frameCtl);
+	WIFI_LOG(3, "SoftAP: Received a packet of length %i bytes (%i aligned). Frame control = %04X\n",
+		len, alignedLen, frameCtl);
 
 	switch((frameCtl >> 2) & 0x3)
 	{
@@ -1185,73 +1326,65 @@ void WIFI_SoftAP_RecvPacketFromDS(wifimac_t *wifi, u8 *packet, int len)
 		{
 			switch((frameCtl >> 4) & 0xF)
 			{
-			case 0x4:		// Probe request
+			case 0x4:		// Probe request (Nintendo WFC config util)
 				{
-					u8 proberes[52];
+					int packetLen = sizeof(SoftAP_ProbeResponse);
+					int totalLen = (packetLen + 12);
 
-					memcpy(proberes, SoftAP_ProbeResponse, 52);
-					memcpy(&proberes[4], FW_Mac, 6);
+					wifi->SoftAP.curPacket = new u8[totalLen];
+
+					// Make the RX header
+					// About the packet length: 
+					// GBATek says the length entry of the RX header is the length of the IEEE
+					// header and frame body, without including the FCS. But the Nintendo WFC
+					// config util expects this length to be the length of the IEEE header and
+					// the frame body AND the FCS. Actually, it expects WRCSR to be equal to
+					// (READCSR + 12 + packet_length).
+					WIFI_SoftAP_MakeRXHeader(wifi, 0x0010, 20, packetLen, 0, 0);
+
+					// Copy the probe response template
+					memcpy(&wifi->SoftAP.curPacket[12], SoftAP_ProbeResponse, packetLen);
+
+					// Add the MAC address
+					memcpy(&wifi->SoftAP.curPacket[12 + 4], FW_Mac, 6);
+
+					// The timestamp
 					u64 timestamp = (wifi->SoftAP.usecCounter / 1000);		// FIXME: is it correct?
-					*(u64*)&proberes[24] = timestamp;
-					u32 crc32 = WIFI_SoftAP_GetCRC32(proberes, 48);
-					*(u32*)&proberes[48] = crc32;
+					*(u64*)&wifi->SoftAP.curPacket[12 + 24] = timestamp;
 
-					if(wifi->SoftAP.curPacket)
-						delete wifi->SoftAP.curPacket;
+					// And the CRC32 (FCS)
+					u32 crc32 = WIFI_getCRC32(&wifi->SoftAP.curPacket[12], (packetLen - 4));
+					*(u32*)&wifi->SoftAP.curPacket[12 + packetLen - 4] = crc32;
 
-					wifi->SoftAP.curPacket = new u8[52+12];
-
-					wifi->SoftAP.curPacket[0] = 0x10;
-					wifi->SoftAP.curPacket[1] = 0x00;
-					wifi->SoftAP.curPacket[2] = 0x40;
-					wifi->SoftAP.curPacket[3] = 0x00;
-					wifi->SoftAP.curPacket[4] = 0x01;
-					wifi->SoftAP.curPacket[5] = 0x00;
-					wifi->SoftAP.curPacket[6] = 0x14;
-					wifi->SoftAP.curPacket[7] = 0x00;
-					wifi->SoftAP.curPacket[8] = 0x30;
-				/*	wifi->SoftAP.curPacket[7] = 0x09;
-					wifi->SoftAP.curPacket[8] = 0x2C;*/
-					wifi->SoftAP.curPacket[9] = 0x00;
-					wifi->SoftAP.curPacket[10] = 0x00;
-					wifi->SoftAP.curPacket[11] = 0x00;
-
-					memcpy((wifi->SoftAP.curPacket+12), proberes, 52);
-					wifi->SoftAP.curPacketSize = 52+12;
-					wifi->SoftAP.curPacketPos = -200;
+					// Let's prepare to send
+					wifi->SoftAP.curPacketSize = totalLen;
+					wifi->SoftAP.curPacketPos = 0;
 					wifi->SoftAP.curPacketSending = TRUE;
 				}
 				break;
 
 			case 0xB:		// Authentication
 				{
-					u8 authframe[36];
+					int packetLen = sizeof(SoftAP_AuthFrame);
+					int totalLen = (packetLen + 12);
 
-					memcpy(authframe, SoftAP_AuthFrame, 34);
-					memcpy(&authframe[4], FW_Mac, 6);
-					u32 crc32 = WIFI_SoftAP_GetCRC32(authframe, 30);
-					*(u32*)&authframe[32] = crc32;
+					wifi->SoftAP.curPacket = new u8[totalLen];
 
-					if(wifi->SoftAP.curPacket)
-						delete wifi->SoftAP.curPacket;
+					// Make the RX header
+					WIFI_SoftAP_MakeRXHeader(wifi, 0x0010, 20, packetLen, 0, 0);
 
-					wifi->SoftAP.curPacket = new u8[36+12];
+					// Copy the authentication frame template
+					memcpy(&wifi->SoftAP.curPacket[12], SoftAP_AuthFrame, packetLen);
 
-					wifi->SoftAP.curPacket[0] = 0x10;
-					wifi->SoftAP.curPacket[1] = 0x00;
-					wifi->SoftAP.curPacket[2] = 0x40;
-					wifi->SoftAP.curPacket[3] = 0x00;
-					wifi->SoftAP.curPacket[4] = 0x01;
-					wifi->SoftAP.curPacket[5] = 0x00;
-					wifi->SoftAP.curPacket[6] = 0x14;
-					wifi->SoftAP.curPacket[7] = 0x00;
-					wifi->SoftAP.curPacket[8] = 0x20;
-					wifi->SoftAP.curPacket[9] = 0x00;
-					wifi->SoftAP.curPacket[10] = 0x00;
-					wifi->SoftAP.curPacket[11] = 0x00;
+					// Add the MAC address
+					memcpy(&wifi->SoftAP.curPacket[12 + 4], FW_Mac, 6);
 
-					memcpy((wifi->SoftAP.curPacket+12), authframe, 36);
-					wifi->SoftAP.curPacketSize = 36+12;
+					// And the CRC32 (FCS)
+					u32 crc32 = WIFI_getCRC32(&wifi->SoftAP.curPacket[12], (packetLen - 4));
+					*(u32*)&wifi->SoftAP.curPacket[12 + packetLen - 4] = crc32;
+
+					// Let's prepare to send
+					wifi->SoftAP.curPacketSize = totalLen;
 					wifi->SoftAP.curPacketPos = 0;
 					wifi->SoftAP.curPacketSending = TRUE;
 				}
@@ -1259,33 +1392,26 @@ void WIFI_SoftAP_RecvPacketFromDS(wifimac_t *wifi, u8 *packet, int len)
 
 			case 0x0:		// Association request
 				{
-					u8 assocres[40];
+					int packetLen = sizeof(SoftAP_AssocResponse);
+					int totalLen = (packetLen + 12);
 
-					memcpy(assocres, SoftAP_AssocResponse, 38);
-					memcpy(&assocres[4], FW_Mac, 6);
-					u32 crc32 = WIFI_SoftAP_GetCRC32(assocres, 34);
-					*(u32*)&assocres[36] = crc32;
+					wifi->SoftAP.curPacket = new u8[totalLen];
 
-					if(wifi->SoftAP.curPacket)
-						delete wifi->SoftAP.curPacket;
+					// Make the RX header
+					WIFI_SoftAP_MakeRXHeader(wifi, 0x0010, 20, packetLen, 0, 0);
 
-					wifi->SoftAP.curPacket = new u8[40+12];
+					// Copy the association response template
+					memcpy(&wifi->SoftAP.curPacket[12], SoftAP_AssocResponse, packetLen);
 
-					wifi->SoftAP.curPacket[0] = 0x10;
-					wifi->SoftAP.curPacket[1] = 0x00;
-					wifi->SoftAP.curPacket[2] = 0x40;
-					wifi->SoftAP.curPacket[3] = 0x00;
-					wifi->SoftAP.curPacket[4] = 0x01;
-					wifi->SoftAP.curPacket[5] = 0x00;
-					wifi->SoftAP.curPacket[6] = 0x14;
-					wifi->SoftAP.curPacket[7] = 0x00;
-					wifi->SoftAP.curPacket[8] = 0x24;
-					wifi->SoftAP.curPacket[9] = 0x00;
-					wifi->SoftAP.curPacket[10] = 0x00;
-					wifi->SoftAP.curPacket[11] = 0x00;
+					// Add the MAC address
+					memcpy(&wifi->SoftAP.curPacket[12 + 4], FW_Mac, 6);
 
-					memcpy((wifi->SoftAP.curPacket+12), assocres, 40);
-					wifi->SoftAP.curPacketSize = 36+12;
+					// And the CRC32 (FCS)
+					u32 crc32 = WIFI_getCRC32(&wifi->SoftAP.curPacket[12], (packetLen - 4));
+					*(u32*)&wifi->SoftAP.curPacket[12 + packetLen - 4] = crc32;
+
+					// Let's prepare to send
+					wifi->SoftAP.curPacketSize = totalLen;
 					wifi->SoftAP.curPacketPos = 0;
 					wifi->SoftAP.curPacketSending = TRUE;
 				}
@@ -1296,12 +1422,12 @@ void WIFI_SoftAP_RecvPacketFromDS(wifimac_t *wifi, u8 *packet, int len)
 
 	case 0x2:				// Data
 		{
-			/* We convert the packet into an Ethernet packet */
+			// We convert the packet into an Ethernet packet
 
 			int eflen = (alignedLen - 4 - 30 + 14);
 			u8 *ethernetframe = new u8[eflen];
 
-			/* Destination address */
+			// Destination address
 			ethernetframe[0] = packet[28];
 			ethernetframe[1] = packet[29];
 			ethernetframe[2] = packet[30];
@@ -1309,7 +1435,7 @@ void WIFI_SoftAP_RecvPacketFromDS(wifimac_t *wifi, u8 *packet, int len)
 			ethernetframe[4] = packet[32];
 			ethernetframe[5] = packet[33];
 
-			/* Source address */
+			// Source address
 			ethernetframe[6] = packet[22];
 			ethernetframe[7] = packet[23];
 			ethernetframe[8] = packet[24];
@@ -1317,17 +1443,18 @@ void WIFI_SoftAP_RecvPacketFromDS(wifimac_t *wifi, u8 *packet, int len)
 			ethernetframe[10] = packet[26];
 			ethernetframe[11] = packet[27];
 
-			/* EtherType */
+			// EtherType
 			ethernetframe[12] = packet[42];
 			ethernetframe[13] = packet[43];
 
-			/* Frame body */
+			// Frame body
 			memcpy((ethernetframe + 14), (packet + 44), (alignedLen - 30 - 4));
 
-			/* Checksum */
-			/* TODO ? */
+			// Checksum 
+			// TODO ?
 
-			PCAP::pcap_sendpacket(wifi->SoftAP.bridge, ethernetframe, eflen);
+			if(wifi->netEnabled) //dont try to pcap out the packet unless network is enabled
+				desmume_pcap_sendpacket(wifi->SoftAP.bridge, ethernetframe, eflen);
 
 			delete ethernetframe;
 		}
@@ -1335,48 +1462,31 @@ void WIFI_SoftAP_RecvPacketFromDS(wifimac_t *wifi, u8 *packet, int len)
 	}
 }
 
-void WIFI_SoftAP_SendToNetwork(wifimac_t *wifi)
+static void WIFI_SoftAP_SendBeacon(wifimac_t *wifi)
 {
-	//
-}
+	int packetLen = sizeof(SoftAP_Beacon);
+	int totalLen = (packetLen + 12);
 
-void WIFI_SoftAP_RecvFromNetwork(wifimac_t *wifi)
-{
-	//
-}
+	if(wifi->SoftAP.curPacket) delete wifi->SoftAP.curPacket;
+	wifi->SoftAP.curPacket = new u8[totalLen];
 
-void WIFI_SoftAP_SendBeacon(wifimac_t *wifi)
-{
-	u8 beacon[58];
+	// Make the RX header
+	WIFI_SoftAP_MakeRXHeader(wifi, 0x0011, 20, packetLen, 0, 0);
 
-	memcpy(beacon, SoftAP_Beacon, 58);
+	// Copy the beacon template
+	memcpy(&wifi->SoftAP.curPacket[12], SoftAP_Beacon, packetLen);
+
+	// Add the timestamp
 	u64 timestamp = (wifi->SoftAP.usecCounter / 1000);		// FIXME: is it correct?
-	*(u64*)&beacon[24] = timestamp;
-	u32 crc32 = WIFI_SoftAP_GetCRC32(beacon, 54);
-	*(u32*)&beacon[54] = crc32;
+	*(u64*)&wifi->SoftAP.curPacket[12 + 24] = timestamp;
 
-	if(wifi->SoftAP.curPacket)
-		delete wifi->SoftAP.curPacket;
+	// And the CRC32 (FCS)
+	u32 crc32 = WIFI_getCRC32(&wifi->SoftAP.curPacket[12], (packetLen - 4));
+	*(u32*)&wifi->SoftAP.curPacket[12 + packetLen - 4] = crc32;
 
-	wifi->SoftAP.curPacket = new u8[58+12];
-
-	wifi->SoftAP.curPacket[0] = 0x11;
-	wifi->SoftAP.curPacket[1] = 0x00;
-	wifi->SoftAP.curPacket[2] = 0x40;
-	wifi->SoftAP.curPacket[3] = 0x00;
-	wifi->SoftAP.curPacket[4] = 0x64;
-	wifi->SoftAP.curPacket[5] = 0x00;
-	wifi->SoftAP.curPacket[6] = 0x14;
-	wifi->SoftAP.curPacket[7] = 0x00;
-	wifi->SoftAP.curPacket[8] = 0x36;
-	wifi->SoftAP.curPacket[9] = 0x00;
-	wifi->SoftAP.curPacket[10] = 0x00;
-	wifi->SoftAP.curPacket[11] = 0x00;
-
-	memcpy((wifi->SoftAP.curPacket+12), beacon, 58);
-
-	wifi->SoftAP.curPacketSize = 58+12;
-	wifi->SoftAP.curPacketPos = -4;
+	// Let's prepare to send
+	wifi->SoftAP.curPacketSize = totalLen;
+	wifi->SoftAP.curPacketPos = 0;
 	wifi->SoftAP.curPacketSending = TRUE;
 }
 
@@ -1400,22 +1510,25 @@ void WIFI_SoftAP_usTrigger(wifimac_t *wifi)
 	/* ie ~8 microseconds to transfer a word. */
 	if((wifi->SoftAP.curPacketSending) && !(wifi->SoftAP.usecCounter & 7))
 	{
-	/*	if(wifi->SoftAP.curPacketPos == -4)
-		{
-			WIFI_triggerIRQ(wifi, 6);
-		}
-		else*/ if(wifi->SoftAP.curPacketPos >= 0)
+		if(wifi->SoftAP.curPacketPos >= 0)
 		{
 			if(wifi->SoftAP.curPacketPos == 0)
 			{
-				WIFI_triggerIRQ(wifi, 6);
+				WIFI_triggerIRQ(wifi, WIFI_IRQ_RECVSTART);
+
+				wifi->rfStatus = 0x0009;
+				wifi->rfPins = 0x0004;
+
+			//	printf("wifi: softap: started sending packet at %04X\n", wifi->RXHWWriteCursor << 1);
+			//	wifi->SoftAP.curPacketPos += 2;
+			}
+			else
+			{
+				wifi->rfStatus = 0x0001;
+				wifi->rfPins = 0x0084;
 			}
 
 			u16 word = (wifi->SoftAP.curPacket[wifi->SoftAP.curPacketPos] | (wifi->SoftAP.curPacket[wifi->SoftAP.curPacketPos+1] << 8));
-			//WIFI_SoftAP_SendWordToDS(wifi, word);
-		//	if(wifi->SoftAP.curPacketPos<12)
-		//		printf("wifi: writing word %i (%04X) of packet to %04X\n", 
-		//			wifi->SoftAP.curPacketPos, word, wifi->RXHWWriteCursor);
 			WIFI_RXPutWord(wifi, word);
 		}
 
@@ -1426,86 +1539,19 @@ void WIFI_SoftAP_usTrigger(wifimac_t *wifi)
 			wifi->SoftAP.curPacketPos = 0;
 			wifi->SoftAP.curPacketSending = FALSE;
 
+			delete wifi->SoftAP.curPacket;
+			wifi->SoftAP.curPacket = NULL;
+
 			wifi->RXHWWriteCursorReg = ((wifi->RXHWWriteCursor + 1) & (~1));
 
-			WIFI_triggerIRQ(wifi, 0);
+			WIFI_triggerIRQ(wifi, WIFI_IRQ_RECVCOMPLETE);
+
+			// hax
+			//packet_arrived=1;
+
+			//printf("wifi: softap: finished sending packet. end at %04X (aligned=%04X)\n", wifi->RXHWWriteCursor << 1, wifi->RXHWWriteCursorReg << 1);
 		}
 	}
 }
 
-/*******************************************************************************
-
-	Host system operations
-
- *******************************************************************************/
-
-socket_t WIFI_Host_OpenChannel(u8 num)
-{
-	sockaddr_t address ;
-	/* create a new socket */
-	socket_t channelSocket = socket(AF_INET,SOCK_DGRAM,0) ;
-	/* set the sockets address */
-	memset(&address,0,sizeof(sockaddr_t)) ;
-	address.sa_family = AF_INET ;
-	*(u32 *)&address.sa_data[2] = htonl(0) ; 				/* IP: any */
-	*(u16 *)&address.sa_data[0] = htons(BASEPORT + num-1) ; /* Port */
-	bind(channelSocket,&address,sizeof(sockaddr_t)) ;
-	return channelSocket ;
-}
-
-void WIFI_Host_CloseChannel(socket_t sock)
-{
-	if (sock!=INVALID_SOCKET)
-	{
-#ifdef WIN32
-		closesocket(sock) ;
-#else
-		close(sock) ;
-#endif
-	}
-}
-
-void WIFI_Host_SendData(socket_t sock, u8 channel, u8 *data, u16 length)
-{
-	//printf("wifi: sending packet of length %i on channel %i\n", length, channel);
-	sockaddr_t address ;
-	/* create the frame to validate data: "DSWIFI" string and a u16 constant of the payload length */
-	u8 *frame = (u8 *)malloc(length + 8) ;
-	sprintf((char *)frame,"DSWIFI") ;
-	*(u16 *)(frame+6) = htons(length) ;
-	memcpy(frame+8,data,length) ;
-	/* target address */
-	memset(&address,0,sizeof(sockaddr_t)) ;
-	address.sa_family = AF_INET ;
-	*(u32 *)&address.sa_data[2] = htonl(0xFFFFFFFF) ; 			/* IP: broadcast */
-	*(u16 *)&address.sa_data[0] = htons(BASEPORT + channel-1) ; /* Port */
-	/* send the data now */
-	sendto(sock,(const char *)frame,length+8,0,&address,sizeof(sockaddr_t)) ;
-	/* clean up frame buffer */
-	free(frame) ;
-}
-
-u16 WIFI_Host_RecvData(socket_t sock, u8 *data, u16 maxLength)
-{
-	fd_set dataCheck ;
-	struct timeval tv ;
-	FD_ZERO(&dataCheck) ;
-	FD_SET(sock,&dataCheck) ;
-	tv.tv_sec = 0 ;
-	tv.tv_usec = 0 ;
-	/* check if there is data, without blocking */
-	if (select(1,&dataCheck,0,0,&tv))
-	{
-		/* there is data available */
-		u32 receivedBytes = recv(sock,(char*)data,maxLength,0) ;
-		//printf("wifi: received %i bytes\n", receivedBytes);
-		if (receivedBytes < 8) return 0 ;			/* this cant be data for us, the header alone is 8 bytes */
-		if (memcmp(data,"DSWIFI",6)!=0) return 0 ;	/* this isnt data for us, as the tag is not our */
-		if (ntohs(*(u16 *)(data+6))+8 != receivedBytes) return 0 ; /* the datalen+headerlen is not what we received */
-		memmove(data,data+8,receivedBytes-8) ;
-		return receivedBytes-8 ;
-	}
-	return 0 ;
-}
-
 #endif
