@@ -1,5 +1,6 @@
-/*  
-	Copyright (C) 2008 shash
+/*  AboutBox.cpp
+
+    Copyright (C) 2008-2009 shash
 
     This file is part of DeSmuME
 
@@ -15,47 +16,64 @@
 
     You should have received a copy of the GNU General Public License
     along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
 
 #include "../common.h"
+#include "version.h"
 
 #include "AboutBox.h"
 #include "resource.h"
 
 #define ABOUT_TIMER_ID 110222
 #define PER_PAGE_TEAM 23
-#define TEAM 27
 #define SIZE_SCROLL_BUFFER PER_PAGE_TEAM + TEAM
-const char	*team[TEAM] = { "Guillaume Duhamel",
-							"Normmatt",
-							"Bernat Muoz (shash)",
-							"thoduv",
-							"Tim Seidel (Mighty Max)",
-							"Pascal Giard (evilynux)",
-							"Ben Jaques (masscat)",
-							"Jeff Bland",
-							"Riccardo Magliocchetti",
-							"Max Tabachenko (CrazyMax)",
-							"zeromus",
-							"Luigi__",
-							"adelikat",
-							"matusz",
-							"pa__",
-							"",
-							"Contributors",
-							"------------",
-							"Allustar",
-							"amponzi",
-							"Anthony Molinaro",
-							"ape",
-							"Damien Nozay (damdoum)",
-							"delfare",
-							"Romain Vallet",
-							"snkmad",
-							"Theo Berkau"};
+const char	*team[] = { 
+	"Current Team",
+	"------------",
+	"Guillaume Duhamel",
+	"Normmatt",
+	"Bernat Muoz (shash)",
+	"Riccardo Magliocchetti",
+	"Max Tabachenko (CrazyMax)",
+	"zeromus",
+	"Luigi__",
+	"adelikat",
+	"matusz",
+	"pa__",
+	"gocha",
+	"",
+	"Contributors",
+	"------------",
+	"Allustar",
+	"amponzi",
+	"Anthony Molinaro",
+	"ape",
+	"Damien Nozay (damdoum)",
+	"delfare",
+	"Romain Vallet",
+	"snkmad",
+	"Theo Berkau",
+	"thoduv",
+	"Tim Seidel (Mighty Max)",
+	"Pascal Giard (evilynux)",
+	"Ben Jaques (masscat)",
+	"Jeff Bland",
+	"",
+	"Honorary Nagmasters",
+	"(Thanks to our super testers for this release!)",
+	"------------",
+	"nash679",
+	"pokefan999",
+	"dottorleo",
+	"lbalbalba",
+	"",
+	"average time from bug checkin to bugreport:",
+	"23 seconds",
+};
 
+const int TEAM = ARRAY_SIZE(team);
 
 u8	scroll_start;
 u8	scroll_buffer[SIZE_SCROLL_BUFFER][255];
