@@ -409,15 +409,15 @@ BOOL CALLBACK ViewDisasm_ARM7Proc (HWND hwnd, UINT message, WPARAM wParam, LPARA
                              /* address to line correction */
                              switch (DisView7->mode) {
                                     case 0:    /* auto */
-                                         DisView7->curr_ligne = adr>>1;
+                                         DisView7->curr_ligne = adr>>2;
                                          if (DisView7->cpu->CPSR.bits.T) {
                                              DisView7->curr_ligne = adr>>1;
                                          }
                                          break ;
-                                    case 1:    /* thumb */
+                                    case 1:    /* arm */
                                          DisView7->curr_ligne = adr>>2;
                                          break ;
-                                    case 2:    /* arm */
+                                    case 2:    /* thumb */
                                          DisView7->curr_ligne = adr>>1;
                                          break ;
                              } ;
@@ -587,7 +587,7 @@ BOOL CALLBACK ViewDisasm_ARM9Proc (HWND hwnd, UINT message, WPARAM wParam, LPARA
                                    DisView9->curr_ligne <<= 1 ;
                              } else if (!(DisView9->cpu->CPSR.bits.T) && (DisView9->mode == 2)) {
                                   /* from thumb to arm, line / 2 */
-                                   DisView9->curr_ligne >>= 1 ;
+                                   DisView9->curr_ligne >>= 2 ;
                              }
                              DisView9->mode = 0;
                              InvalidateRect(GetDlgItem(hwnd, IDC_DES_BOX), NULL, FALSE);
@@ -595,7 +595,7 @@ BOOL CALLBACK ViewDisasm_ARM9Proc (HWND hwnd, UINT message, WPARAM wParam, LPARA
                         case IDC_ARM :
                              /* address to line correction */
                              if ((DisView9->mode==2) || ((DisView9->mode==0) && (DisView9->cpu->CPSR.bits.T))) {
-                                   DisView9->curr_ligne >>= 1 ;
+                                   DisView9->curr_ligne >>= 2 ;
                              } ;
                              DisView9->mode = 1;
                              InvalidateRect(GetDlgItem(hwnd, IDC_DES_BOX), NULL, FALSE);
@@ -626,9 +626,9 @@ BOOL CALLBACK ViewDisasm_ARM9Proc (HWND hwnd, UINT message, WPARAM wParam, LPARA
                              return 1;
                         case IDC_STEP :					// TODO: draw in DD
                              {
-                                  int ndstep;
-                                  ndstep = GetDlgItemInt(hwnd, IDC_SETPNUM, NULL, FALSE);
-                                  NDS_exec<TRUE>(ndstep);
+									int ndstep;
+									ndstep = GetDlgItemInt(hwnd, IDC_SETPNUM, NULL, FALSE);
+									NDS_exec<TRUE>(ndstep);
                              }
                              return 1;
                         case IDC_GO :
@@ -653,15 +653,15 @@ BOOL CALLBACK ViewDisasm_ARM9Proc (HWND hwnd, UINT message, WPARAM wParam, LPARA
                              /* address to line correction */
                              switch (DisView9->mode) {
                                     case 0:    /* auto */
-                                         DisView9->curr_ligne = adr>>1;
+                                         DisView9->curr_ligne = adr>>2;
                                          if (DisView9->cpu->CPSR.bits.T) {
                                              DisView9->curr_ligne = adr>>1;
                                          }
                                          break ;
-                                    case 1:    /* thumb */
+                                    case 1:    /* arm */
                                          DisView9->curr_ligne = adr>>2;
                                          break ;
-                                    case 2:    /* arm */
+                                    case 2:    /* thumb */
                                          DisView9->curr_ligne = adr>>1;
                                          break ;
                              } ;
@@ -710,7 +710,8 @@ BOOL CALLBACK ViewDisasm_ARM9Proc (HWND hwnd, UINT message, WPARAM wParam, LPARA
 	return FALSE;
 }
 
-void DisassemblerTools_Refresh(u8 proc)
+template<int proc>
+FORCEINLINE void DisassemblerTools_Refresh()
 {
 	if (DisViewWnd[proc] == NULL) return;
 	if (proc == 0)
@@ -725,4 +726,8 @@ void DisassemblerTools_Refresh(u8 proc)
 		InvalidateRect(DisViewWnd[proc], NULL, FALSE);
 		DisView7->autogo=false;
 	}
-}
\ No newline at end of file
+}
+
+//these templates needed to be instantiated manually
+template void DisassemblerTools_Refresh<0>();
+template void DisassemblerTools_Refresh<1>();
\ No newline at end of file
